package com.ruoyi.project.workflow.common;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.TypeReference;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.security.LoginUser;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.system.domain.SysRole;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.workflow.domain.base.DateFromTo;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;

import javax.mail.internet.MimeMessage;
import java.io.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.lang.reflect.Field;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class ClsBase {
    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    /*
    trim
     */
    public static String getTrimData(String str) {
        String reStr = Definition.NULL_STR_1;
        if (StringUtils.isNotEmpty(str)) {
            reStr = str.trim();

            if (Definition.NULL_STR_2.equalsIgnoreCase(reStr)) {
                reStr = Definition.NULL_STR_1;
            }
        }
        return reStr;
    }

    /*
    日期_区间
     */
    public static DateFromTo toDate(String str) {
        DateFromTo rDate = new DateFromTo();
        String inStr = getTrimData(str);
        if (StringUtils.isNotEmpty(inStr)) {
            if (inStr.indexOf(Definition.ROLE_DELIMETER) > 0) {
                rDate.setDateFrom(getTrimData(inStr.split(Definition.ROLE_DELIMETER)[0]));
                rDate.setDateTo(getTrimData(inStr.split(Definition.ROLE_DELIMETER)[1]));
            } else {
                rDate.setDateFrom(inStr);
            }
        }
        return rDate;
    }

    /*
    十位日期格式检查（YYYY-MM-DD）
     */
    public static Boolean isDateType10(String inStr) {
        if (StringUtils.isEmpty(inStr)) {
            return true;
        }
        String str = inStr.trim();
        if (str.length() != 10) {
            return false;
        }
        if (!Definition.HENG_GANG.equals(str.substring(4,5))) {
            return false;
        }
        if (!Definition.HENG_GANG.equals(str.substring(7,8))) {
            return false;
        }
        try {
            DateUtils.parseDate(str, new String[]{Definition.DATE_TODAY});
        } catch (ParseException e) {
            return false;
        }
        return true;
    }

    /*
    excel名
     */
    public static String getEccelName(String strName, String userId) {
        String rName = Definition.EXPORT_INFO;
        if (StringUtils.isNotEmpty(strName)) {
            rName = rName + Definition.HENG_GANG_X + strName;
        }
        if (StringUtils.isNotEmpty(userId)) {
            rName = rName + Definition.HENG_GANG_X + userId;
        }
        //时间
        SimpleDateFormat fm = new SimpleDateFormat(Definition.DATE_FORMATE);
        String currenTime = fm.format(new Date());
        rName = rName + Definition.HENG_GANG_X + currenTime + Definition.EXCEL_TYPE;

        return rName;
    }

    /*
    现在的日期（yyyy-MM-dd）
     */
    public static String getNowDate(String inFormat) {
        String sFormat = inFormat;
        if (StringUtils.isEmpty(sFormat)) {
            sFormat = Definition.DATE_TODAY;
        }
        SimpleDateFormat fm = new SimpleDateFormat(sFormat);
        return fm.format(new Date());
    }


    /*
    TO Date
     */
    public static Date strToDate(String str) {
        try {
            return DateUtils.parseDate(str, new String[]{Definition.DATE_TODAY});
        } catch (ParseException e) {
            log.error("日期" + str + "异常{}", e.getMessage());
            return null;
        }
    }


    /*
    用户NAME
     */
    public static String getCurrentUserName() {
        String reStr = Definition.NULL_STR_1;
        // 获取当前的用户
        SysUser currentUser = getSysUser();
        if (currentUser != null) {
            reStr = currentUser.getUserName();
        }

        return  reStr;
    }

    /*
   用户id
    */
    public static String getCurrentUser() {
        String reStr = Definition.NULL_STR_1;
        //获取当前的用户
        SysUser currentUser = getSysUser();
        if (currentUser != null) {
            Long userid = currentUser.getUserId();
            if (userid != null) {
                reStr = userid.toString();
            }
        }
        return  reStr;
    }


    /*
     用户部门
     */
    public static SysDept getCurrentUserDept() {
        SysDept reDept = null;
        //获取当前的用户
        SysUser currentUser = getSysUser();
        if (currentUser != null) {
            reDept = currentUser.getDept();
        }
        return  reDept;
    }

    /*
    默认语言(中文)
     */
    public static String defaultLanguage() {
        return Definition.I_LANGUAGE_CH;
    }
    /*
    用户部门ID
     */
    public static String getUserDeptId() {
        String reStr = Definition.NULL_STR_1;
        //获取当前的用户
        SysUser currentUser = getSysUser();
        if (currentUser != null) {
            if (currentUser.getDeptId() != null) {
                reStr = currentUser.getDeptId().toString();
            }
        }

        return  reStr;
    }

    /*
    admin判断(1:是，0：否)
     */
    public static String isAdmin() {
        String reStr = Definition.FLG_ZERO;
        // 获取当前的用户
        SysUser currentUser = getSysUser();
        if (currentUser != null) {
            if (currentUser.getRoles() != null && currentUser.getRoles().size() > 0) {
                for (SysRole sRole : currentUser.getRoles()) {
                    if (Definition.ADMIN.equals(sRole.getRoleKey())) {
                        reStr = Definition.FLG_1;
                        break;
                    }
                }
            }
        }

        return  reStr;
    }

    /*
     bean转map
    */
    public static Map<String, Object> beanToMap(Object obj) {
        Map<String, Object> reMap = new HashMap<>();
        if (obj == null) {
            return reMap;
        }
        Class clazz = obj.getClass();
        Field[] fields = clazz.getDeclaredFields();
        try{
            for (Field field : fields) {
                field.setAccessible(true);
                reMap.put(field.getName(), field.get(obj));
            }
        } catch (Exception e) {
            log.error("bean转map异常{}", e.getMessage());
        }
        return reMap;
    }

    /*
     json转map lsit
    */
    public static List<Map<String, String>> jsonoMapList(String str) {
        if (StringUtils.isEmpty(str)) {
            return null;
        } else {
            JSONArray jsonArray = JSON.parseArray(str);
            List<Map<String, String>> mapList
                    = JSON.parseObject(jsonArray.toJSONString(), new TypeReference<List<HashMap<String,String>>>(){});

            return mapList;
        }
    }

    /*
    获取当前的用户
     */
    private static SysUser getSysUser() {
        // 获取当前的用户
        try {
            LoginUser loginUser = SecurityUtils.getLoginUser();
            if (loginUser != null) {
                return loginUser.getUser();
            } else {
                log.error("用户null异常");
                return null;
            }
        } catch (Exception e) {
            log.error("用户异常:" + e.getMessage());
            return null;
        }
    }

    /**
     * @param sourceFilePath 待压缩文件（夹）路径
     * @param targetPath 压缩文件所在目录
     * @param zipFileName 压缩后的文件名称{.zip结尾}
     * @return
     * @Description: 创建zip文件
     */
    public static boolean createZipFile(String sourceFilePath, String targetPath, String zipFileName){

        Boolean flag = false;
        FileOutputStream fos = null;
        ZipOutputStream zos = null;

        // 要压缩的文件资源
        File sourceFile = new File(sourceFilePath);
        // zip文件存放路径
        String zipPath = "";

        if(com.ruoyi.common.utils.StringUtils.isNotEmpty(targetPath)){
            zipPath = targetPath + File.separator + zipFileName;
        } else {
            zipPath = new File(sourceFilePath).getParent() + File.separator + zipFileName;
        }

        if (!sourceFile.exists()) {
            return false;
        }

        try {
            File zipFile = new File(zipPath);
            if (zipFile.exists()) {
                zipFile.delete();
            }
            fos = new FileOutputStream(zipPath);
            zos = new ZipOutputStream(new BufferedOutputStream(fos));
            // 生成压缩文件
            writeZip(sourceFile, "", zos);
            flag = true;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //关闭流
            try {
                if (null != zos) {
                    zos.close();
                }
                if (null != fos){
                    fos.close();
                }
            } catch (IOException e) {
                log.error("压缩失败{}", e);
            }
        }
        return flag;
    }

    /**
     * 压缩
     */
    private static void writeZip(File file, String parentPath, ZipOutputStream zos) {
        if (file.exists()) {
            // 处理文件夹
            if (file.isDirectory()) {
                parentPath += file.getName() + File.separator;
                File[] files = file.listFiles();
                if (files.length != 0) {
                    for (File f : files) {
                        // 递归调用
                        writeZip(f, parentPath, zos);
                    }
                } else {
                    // 空目录则创建当前目录的ZipEntry
                    try {
                        zos.putNextEntry(new ZipEntry(parentPath));
                    } catch (IOException e) {
                        log.error("压缩失败{}", e);
                    }
                }
            } else {
                FileInputStream fis = null;
                try {
                    fis = new FileInputStream(file);
                    ZipEntry ze = new ZipEntry(parentPath + file.getName());
                    zos.putNextEntry(ze);
                    byte[] content = new byte[1024];
                    int len;
                    while ((len = fis.read(content)) != -1) {
                        zos.write(content, 0, len);
                        zos.flush();
                    }
                } catch (FileNotFoundException e) {
                    log.error("创建ZIP文件失败", e);
                } catch (IOException e) {
                    log.error("创建ZIP文件失败", e);
                } finally {
                    try {
                        if (fis != null) {
                            fis.close();
                        }
                    } catch (IOException e) {
                        log.error("创建ZIP文件失败", e);
                    }
                }
            }
        }
    }

    /**
     * 邮件
     * @param mailSmtp 邮箱Smtp参数
     * @param mailAddressFrom 送信邮箱
     * @param mailPswFrom 授权码
     * @param mailUserFrom 送信人
     * @param strMailSubject 邮件主题
     * @param strBody 邮件正文
     * @param strAddressTo 收信人
     * @param strAddressCC cc收信人
     * @param logoFile Logo图标文件
     * @param files 附件
     * @return 正常"0" or 错误信息
     */
    public static String sendMail(String mailSmtp, String mailAddressFrom, String mailPswFrom, String mailUserFrom,
                                  String strMailSubject, String strBody, String strAddressTo, String strAddressCC,
                                  File logoFile, List<File> files) {
        //发送
        try {
            //配置信息
            JavaMailSenderImpl sender = new JavaMailSenderImpl();
            sender.setHost(mailSmtp);
            sender.setUsername(mailAddressFrom);
            sender.setPassword(mailPswFrom);
            Properties p = new Properties();
            p.setProperty("mail.smtp.auth", "true");
            sender.setJavaMailProperties(p);
            //消息
            MimeMessage msg =  sender.createMimeMessage();
            MimeMessageHelper helper = new MimeMessageHelper(msg, true,"utf-8");
            //送信人
            helper.setFrom(mailUserFrom);
            //主题
            helper.setSubject(strMailSubject);
            //内容
            helper.setText(StringUtils.replace(strBody, Definition.STR_VBCRLF, "</br>"), true);
            //追加logo
            if (logoFile != null) {
                helper.addInline(Definition.STR_LOGO, logoFile);
            }
            //追加所有附件
            if (files != null && files.size() > 0) {
                for (File iFile : files) {
                    helper.addAttachment(iFile.getName(), iFile);
                }
            }
            //CC
            if (StringUtils.isNotEmpty(strAddressCC)) {
                for (String str : strAddressCC.split(Definition.STR_FEN_HAO)) {
                    helper.addCc(str);
                }
            }
            //to
            if (StringUtils.isNotEmpty(strAddressTo)) {
                for (String str : strAddressTo.split(Definition.STR_FEN_HAO)) {
                    helper.addTo(str);
                }
            } else {
                return Definition.ERROR_MSG_MAIL_NOTO;
            }
            //异步发送
            ExecutorService executor = Executors.newFixedThreadPool(2);
            Future<String> future = executor.submit(new Callable<String>() {
                @Override
                public String call() throws Exception {
                    sender.send(msg);
                    return Definition.RESULT_S;
                }
            });
            return Definition.RESULT_S;
        } catch (Exception e) {
            return (Definition.ERROR_MSG_MAIL + "FROM_MAIL:<"
                    + mailAddressFrom + ">;TO_MAIL:<" + strAddressTo + ">" + e.getMessage());
        }
    }
}
