package com.ruoyi.project.workflow.controller;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.BLanguageMs;
import com.ruoyi.project.workflow.domain.db.out.BLanguageMsOut;
import com.ruoyi.project.workflow.domain.input.BLanguageMsSearch;
import com.ruoyi.project.workflow.domain.input.BLanguageMsgNameIn;
import com.ruoyi.project.workflow.domain.input.DeptNameIn;
import com.ruoyi.project.workflow.domain.output.MapData;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/language")
public class BLanguageMsController {

    @Autowired
    private IBLanguageMsService bLanguageMsService;

    //消息
    private String outMsg;
    //消息
    private String userId;
    //language
    private String language;

    @PostMapping ("/getList")
    public MapData getList(@RequestBody BLanguageMsSearch inData) {
        init(inData.getI_language());
        //返回
        Map<String, Object> outData = new HashMap<>();
        List<BLanguageMsOut> list = new ArrayList<>();
        //语言
        BLanguageMs bLanguageMs = new BLanguageMs();
        bLanguageMs.setLanguageType(language);
        if (Definition.ALL_DATA.equals(inData.getI_func_id())) {
            list = bLanguageMsService.selectAllMessage(bLanguageMs);
        } else if(StringUtils.isNotEmpty(inData.getI_func_id())) {
            bLanguageMs.setiFuncId(inData.getI_func_id());
            list = bLanguageMsService.selectAllMessage(bLanguageMs);
        }
        if (list != null && list.size() > 0) {
            for (BLanguageMsOut data : list) {
                outData.put(data.getLabel(), data.getValue());
            }
        }
        MapData rspData = new MapData();
        rspData.setLogin_user(userId);
        rspData.setResultCode(Definition.RESULT_S);
        rspData.setData(outData);

        return rspData;
    }

    @PostMapping ("/getDictName")
    public WorkFlowTableInfo getDictName(@RequestBody BLanguageMsgNameIn inData) {
        init(inData.getI_language());
        //检索相关
        String field = ClsBase.getTrimData(inData.getI_field());
        //操作
        String enable = ClsBase.getTrimData(inData.getI_enable());
        //取得
        List<BLanguageMsOut> list = bLanguageMsService.selectDictTypeName(language, field, enable);
        //返回值
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        if (list != null) {
            respData.setDataList(list);
            respData.setDataCount(String.valueOf(list.size()));
            respData.setI_total(String.valueOf(list.size()));
        }
        respData.setErrorDesc(outMsg);
        respData.setLogin_user(userId);
        respData.setResultCode(Definition.RESULT_S);
        return respData;
    }

    @PostMapping ("/getAllDictType")
    public WorkFlowTableInfo getAllDictType(@RequestBody DeptNameIn inData) {
        init(inData.getI_language());
        //检索相关
        String field = ClsBase.getTrimData(inData.getI_sch_field());
        //返回值
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setErrorDesc(outMsg);
        respData.setLogin_user(userId);
        respData.setResultCode(Definition.RESULT_S);
        if (StringUtils.isEmpty(field)) {
            respData.setDataList(new ArrayList<>());
            respData.setDataCount(Definition.FLG_ZERO);
            respData.setI_total(Definition.FLG_ZERO);
        } else {
            List<BLanguageMsOut> list = bLanguageMsService.selectAllDictType(language, field);
            if (list != null) {
                respData.setDataList(list);
                respData.setDataCount(String.valueOf(list.size()));
                respData.setI_total(String.valueOf(list.size()));
            }
        }
        return respData;
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        outMsg = Definition.NULL_STR_1;
        userId = ClsBase.getCurrentUser();
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }
}
