package com.ruoyi.project.workflow.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.common.ExcelBase;
import com.ruoyi.project.workflow.domain.base.DateFromTo;
import com.ruoyi.project.workflow.domain.base.InBase;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignYilanIn;
import com.ruoyi.project.workflow.domain.db.out.BWordbookMsOut;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignYilanOut;
import com.ruoyi.project.workflow.domain.input.DeptNameIn;
import com.ruoyi.project.workflow.domain.input.WFormDesignSearch;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInit;
import com.ruoyi.project.workflow.service.IBFunctionMsService;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.IBWordbookMsService;
import com.ruoyi.project.workflow.service.WFormDesignService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;


@RestController
@RequestMapping("/workflowList")
public class WFormDesignController {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WFormDesignService wFormDesignService;
    @Autowired
    private IBFunctionMsService iBFunctionMsService;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;

    //消息
    private String outMsg;
    //用户
    private String userId;
    //language
    private String language;

    @PostMapping("/init")
    public WorkFlowTableInit getInit(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());

        WorkFlowTableInit resData = new WorkFlowTableInit();
        //每页显示条数
        resData.setI_perpage(iBWordbookMsService.selectPerPage(language));
        //表单状态
        resData.setStatusList(iBWordbookMsService.selectDataList(language, Definition.WORD_BOOK_TYPE_30_001,
                Definition.FLG_Y));
        //按钮名
        resData.setButton(iBWordbookMsService.selectButtonInfo(language));
        resData.getButton().setI_btn_import(Definition.NULL_STR_1);
        resData.getButton().setI_btn_delete(Definition.NULL_STR_1);
        //得分页条数
        resData.setPerpageList(iBWordbookMsService.selectDataList(language, Definition.WORD_BOOK_TYPE_99_006,
                Definition.FLG_Y));
        resData.setLogin_user(userId);
        resData.setResultCode(Definition.RESULT_S);

        return resData;
    }

    @PostMapping("/getList")
    public WorkFlowTableInfo getList(@RequestBody WFormDesignSearch inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignYilanIn selectIn = editSearch(inData);
        if (StringUtils.isNotEmpty(outMsg)){
            return errorTableInfo();
        }
        //总页数
        Long nTotal = wFormDesignService.selectWFormDesignListCount(selectIn);
        //取得分页数据
        if (inData.getI_pageindex() == 1) {
            selectIn.setOffSet(Long.valueOf(0));
            if (nTotal > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal);
            }
        } else {
            selectIn.setOffSet(inData.getI_perpage() * (inData.getI_pageindex() - 1));
            if ((nTotal - selectIn.getOffSet()) > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal - selectIn.getOffSet());
                if (selectIn.getNum() < 0) {
                    selectIn.setNum(Long.valueOf(0));
                }
            }
        }
        List<WFormDesignYilanOut> getData = wFormDesignService.selectWFormDesignList(selectIn);

        //返回数据
        WorkFlowTableInfo rspData = new WorkFlowTableInfo();
        rspData.setLogin_user(userId);
        if (getData != null) {
            rspData.setDataCount(String.valueOf(getData.size()));
            rspData.setDataList(getData);
        }
        rspData.setI_total(String.valueOf(nTotal));
        rspData.setResultCode(Definition.RESULT_S);

        return rspData;
    }

    @PostMapping("/export")
    @Log(title = Definition.PAGE_301010, businessType = BusinessType.EXPORT)
    public void export(HttpServletResponse response, @RequestBody WFormDesignSearch inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignYilanIn selectIn = editSearch(inData);
        if (StringUtils.isNotEmpty(outMsg)){
            errorResponse(response);
        }
        if (inData.getTable_info() == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0100);
            errorResponse(response);
        }
        //取得数据
        List<WFormDesignYilanOut> getData = wFormDesignService.selectWFormDesignListExport(selectIn);
        //取得页面名称
        String strDownloadFileName = iBFunctionMsService.GetFuncName(language, Definition.PAGE_301010);
        //excel名
        String excelName = ClsBase.getEccelName(strDownloadFileName, userId);
        //导出
        ExcelBase<WFormDesignYilanOut> util = new ExcelBase<>(WFormDesignYilanOut.class);
        util.exportExcel(response, inData.getTable_info(), getData, Definition.EXPORT_INFO, excelName);
    }

    @DeleteMapping("/delete")
    @Log(title = Definition.PAGE_301010, businessType = BusinessType.DELETE)
    public OutBase deleteFormById(@RequestBody WFormDesignSearch inData) {
        //初期化
        init(inData.getI_language());
        //删除表单Id
        String formId = ClsBase.getTrimData(inData.getI_form_id());
        if (StringUtils.isEmpty(formId)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301010, Definition.MSG_301010_0005);
            return errorOutBase();
        }
        //检索条件
        WFormDesignYilanIn selectIn = editSearch(inData);
        //删除
        String resMsg = wFormDesignService.deleteWFormDesign(language, selectIn);
        if (Definition.RESULT_S.equals(resMsg)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0080);
            //返回
            OutBase rspData = new OutBase();
            rspData.setResultCode(Definition.RESULT_S);
            rspData.setErrorDesc(outMsg);
            rspData.setLogin_user(userId);
            return rspData;
        } else {
            outMsg = resMsg;
            return errorOutBase();
        }
    }

    @PostMapping("/allForm")
    public WorkFlowTableInfo getList(@RequestBody DeptNameIn inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        String schField = ClsBase.getTrimData(inData.getI_sch_field());
        //数据取得
        List<BWordbookMsOut> getData = new ArrayList<>();
        if (Definition.ALL_DATA.equals(schField)) {
            getData = wFormDesignService.selectAllForm(Definition.NULL_STR_1);
        } else if (StringUtils.isNotEmpty(schField)) {
            getData = wFormDesignService.selectAllForm(schField);
        }
        //返回数据
        WorkFlowTableInfo rspData = new WorkFlowTableInfo();
        if (getData != null) {
            rspData.setDataCount(String.valueOf(getData.size()));
            rspData.setDataList(getData);
            rspData.setI_total(String.valueOf(getData.size()));
        }
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_S);

        return rspData;
    }
    /*
    检索条件
     */
    private WFormDesignYilanIn editSearch(WFormDesignSearch inData){
        //检索利用开始日_区间
        DateFromTo starDateQj = ClsBase.toDate(inData.getI_sch_use_s_ymd());
        //日期格式yyyy-MM-dd判断
        if (!ClsBase.isDateType10(starDateQj.getDateFrom())) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301010, Definition.MSG_301010_0001);
            return null;
        }
        if (!ClsBase.isDateType10(starDateQj.getDateTo())) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301010, Definition.MSG_301010_0002);
            return null;
        }
        //检索利用终了日_区间
        DateFromTo endDateQj=  ClsBase.toDate(inData.getI_sch_use_e_ymd());
        //日期格式yyyy-MM-dd判断
        if (!ClsBase.isDateType10(endDateQj.getDateFrom())) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301010, Definition.MSG_301010_0003);
            return null;
        }
        if (!ClsBase.isDateType10(endDateQj.getDateTo())) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301010, Definition.MSG_301010_0004);
            return null;
        }

        //检索条件
        WFormDesignYilanIn selectIn = new WFormDesignYilanIn();
        selectIn.setStriFormId(ClsBase.getTrimData(inData.getI_form_id()));
        selectIn.setStriStatus(ClsBase.getTrimData(inData.getI_sch_status()));
        selectIn.setStrLanguage(language);
        selectIn.setStriFormName(ClsBase.getTrimData(inData.getI_sch_form_name()));
        selectIn.setStartDate(starDateQj);
        selectIn.setEndDate(endDateQj);

        return selectIn;
    }

    /**
     *error
     */
    private WorkFlowTableInfo errorTableInfo() {
        WorkFlowTableInfo rspData = new WorkFlowTableInfo();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        return rspData;
    }

    /**
     *error
     */
    private OutBase errorOutBase() {
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        return rspData;
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        userId = ClsBase.getCurrentUser();
        outMsg = Definition.NULL_STR_1;
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }

    /*
    *error
     */
    private void errorResponse(HttpServletResponse response){
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        try {
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            PrintWriter pw = response.getWriter();
            pw.write(rspData.toString());
        } catch (IOException e) {
            log.error("返回异常{}", e.getMessage());
        }
    }
}
