package com.ruoyi.project.workflow.controller;

import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WFormDesign;
import com.ruoyi.project.workflow.domain.WFormUser;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.db.WFormDesignEidt;
import com.ruoyi.project.workflow.domain.input.WFormDesignEidtIn;
import com.ruoyi.project.workflow.domain.output.MapData;
import com.ruoyi.project.workflow.domain.output.WFormDesignEditOut;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.IBWordbookMsService;
import com.ruoyi.project.workflow.service.WFormDesignService;
import com.ruoyi.project.workflow.service.WFormUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/workflow")
public class WFormDesignEidtController {
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WFormDesignService wFormDesignService;
    @Autowired
    private WFormUserService wFormUserService;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;

    //消息
    private String outMsg;
    //消息
    private String userId;
    //language
    private String language;

    @PostMapping("/detail")
    public MapData getDetail(@RequestBody WFormDesignEidtIn inData){
        //初期化
        init(inData.getI_language());
        MapData rspData = new MapData();
        rspData.setLogin_user(userId);
        //[表单Id]不能为空
        if (StringUtils.isEmpty(inData.getI_form_id())) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0009);
            rspData.setResultCode(Definition.RESULT_E);
            rspData.setErrorDesc(outMsg);
            return rspData;
        }
        //表单情报
        WFormDesignEidt wFormDesignEidt = wFormDesignService.selectWFormDesignById(inData.getI_form_id());
        //利用者情报
        List<WFormUser> listWFormUser = wFormUserService.selectWFormUserTypeByIFormId(inData.getI_form_id());
        //数据处理
        if (wFormDesignEidt == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0007);
            rspData.setResultCode(Definition.RESULT_E);
            rspData.setErrorDesc(outMsg);
            return rspData;
        } else {
            wFormDesignEidt.setLogin_user(userId);
            //字典名称
            String evidenceFolder = iBWordbookMsService.selectWordbookNameByKeys(language,
                    Definition.WORD_BOOK_TYPE_99_001, Definition.WORD_BOOK_NO_99_001_0201, Definition.FLG_Y);
            String tempFileFolder = iBWordbookMsService.selectWordbookNameByKeys(language,
                    Definition.WORD_BOOK_TYPE_99_001, Definition.WORD_BOOK_NO_99_001_0211, Definition.FLG_Y);
            wFormDesignEidt.setEvidenceFolder(evidenceFolder);
            wFormDesignEidt.setTempFileFolder(tempFileFolder);
            //利用者选项
            if (listWFormUser != null && listWFormUser.size() > 0) {
                String optionValue = Definition.NULL_STR_1;
                for (WFormUser item :  listWFormUser) {
                    if (optionValue.length() > 0) {
                        optionValue = optionValue + Definition.ROLE_DELIMETER;
                    }
                    optionValue  = optionValue + item.getiUserId();
                }
                wFormDesignEidt.setOptionType(listWFormUser.get(0).getiUserType());
                wFormDesignEidt.setOptionValue(optionValue);
            }
            //出力数据变化
            WFormDesignEditOut wFormDesignEditOut = new WFormDesignEditOut();
            BeanUtils.copyProperties(wFormDesignEidt, wFormDesignEditOut);
            wFormDesignEditOut.setNeedAudit(Boolean.valueOf(wFormDesignEidt.getNeedAudit()));
            wFormDesignEditOut.setMultilingual(Boolean.valueOf(wFormDesignEidt.getMultilingual()));
            //出力
            rspData.setData(ClsBase.beanToMap(wFormDesignEditOut));
        }
        rspData.setResultCode(Definition.RESULT_S);
        return rspData;
    }

    @PostMapping("/saveForm")
    @Log(title = Definition.PAGE_301020, businessType = BusinessType.INSERT)
    public OutBase saveFormInfo(@RequestBody WFormDesignEidt inData) {
        init(inData.getI_language());
        //表单Id
        String striFormId = ClsBase.getTrimData(inData.getI_form_id());
        //表单名
        String striFormName = ClsBase.getTrimData(inData.getTitle());
        if (StringUtils.isEmpty(striFormName)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0001);
            return outResult(Definition.RESULT_E);
        }
        //表单详情
        String descx = ClsBase.getTrimData(inData.getDesc());
        if (StringUtils.isEmpty(descx)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0002);
            return outResult(Definition.RESULT_E);
        }
        if (descx.getBytes().length > 100) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0003);
            return outResult(Definition.RESULT_E);
        }
        //审批要否
        String boolApprFlg = ClsBase.getTrimData(inData.getNeedAudit());
        if (StringUtils.isEmpty(boolApprFlg)) {
            boolApprFlg = Definition.STR_FALSE;
        }
        //多语言
        String multilingual = ClsBase.getTrimData(inData.getMultilingual());
        if (StringUtils.isEmpty(multilingual)) {
            multilingual = Definition.STR_FALSE;
        }
        //采番规则
        String striNumberRules = ClsBase.getTrimData(inData.getPrefix());
        //全大写转换
        striNumberRules = striNumberRules.toUpperCase();
        if (striNumberRules.getBytes().length > 5) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0017);
            return outResult(Definition.RESULT_E);
        }
        //利用开始日
        String starDate = ClsBase.getTrimData(inData.getStartDate());
        //日期类型check[YYYY-MM-DD]
        if (!ClsBase.isDateType10(starDate)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0004);
            return outResult(Definition.RESULT_E);
        }
        //利用开始日不为空的场合
        if (StringUtils.isNotEmpty(starDate)) {
            if (StringUtils.compare(starDate, ClsBase.getNowDate(Definition.DATE_TODAY)) < 0) {
                //[利用开始日]必须大于当日日付
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0012);
                return outResult(Definition.RESULT_E);
            }
        }
        //利用终了日
        String endDate = ClsBase.getTrimData(inData.getEndDate());
        if (!ClsBase.isDateType10(endDate)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0005);
            return outResult(Definition.RESULT_E);
        }
        //利用开始日不能大于利用终了日
        if (StringUtils.isNotEmpty(endDate)) {
            //利用开始日不能为空
            if (StringUtils.isEmpty(starDate)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0013);
                return outResult(Definition.RESULT_E);
            }
            //利用开始日不能大于利用终了日
            if (StringUtils.compare(starDate, endDate) > 0) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0014);
                return outResult(Definition.RESULT_E);
            }
        }
        //利用者选项
        String optionType = ClsBase.getTrimData(inData.getOptionType());
        //利用者
        String optionValue = ClsBase.getTrimData(inData.getOptionValue());
        //表单数据
        String columnConfig = ClsBase.getTrimData(inData.getColumnConfig());
        //[表单数据]不能为空
        if (StringUtils.isEmpty(columnConfig)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0006);
            return outResult(Definition.RESULT_E);
        }
        //流程配置数据
        String auditConfig = ClsBase.getTrimData(inData.getAuditConfig());

        //フォーム設計对象 w_form_design
        WFormDesign wFormDesign = new WFormDesign();
        wFormDesign.setiFormName(striFormName);
        wFormDesign.setiFormDetail(descx);
        wFormDesign.setiFormLayout(columnConfig);
        if (StringUtils.isNotEmpty(starDate)) {
            wFormDesign.setiUseSYmd(ClsBase.strToDate(starDate));
        }
        if (StringUtils.isNotEmpty(endDate)) {
            wFormDesign.setiUseEYmd(ClsBase.strToDate(endDate));
        }
        wFormDesign.setiFlowData(auditConfig);
        wFormDesign.setiFlowFlg(boolApprFlg);
        wFormDesign.setiLanguageFlg(multilingual);
        wFormDesign.setiNumberRules(striNumberRules);
        wFormDesign.setiUpdId(userId);
        wFormDesign.setiEntryId(userId);

        //表单Id不存在，新规表单
        if (StringUtils.isEmpty(striFormId)) {
            String restMsg = wFormDesignService.insertWFormDesign(language, wFormDesign, optionType, optionValue);
            if (Definition.RESULT_S.equals(restMsg)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0090);
                return outResult(Definition.RESULT_S);
            } else {
                outMsg = restMsg;
                return outResult(Definition.RESULT_E);
            }
        }
        //表单Id
        wFormDesign.setiFormId(Long.valueOf(striFormId));
        //取得表单状态
        String strStatus = wFormDesignService.getFormStatusByFormId(striFormId);
        //[表单]不存在
        if (StringUtils.isEmpty(strStatus)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0007);
            return outResult(Definition.RESULT_E);
        }
        //状态
        if (Definition.NO_OPEN.equals(strStatus)) {
            //状态：0（未开示）
            String resMsg = wFormDesignService.UpdateForm(language, wFormDesign, optionType, optionValue);
            if (Definition.RESULT_S.equals(resMsg)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0110);
                return outResult(Definition.RESULT_S);
            } else {
                outMsg = resMsg;
                return outResult(Definition.RESULT_E);
            }
        } else if (Definition.OPENING.equals(strStatus)) {
            //状态：1（利用中）
            String resMsg = wFormDesignService.copyCreateForm(language, wFormDesign, optionType, optionValue);
            if (Definition.RESULT_S.equals(resMsg)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0110);
                return outResult(Definition.RESULT_S);
            } else {
                outMsg = resMsg;
                return outResult(Definition.RESULT_E);
            }
        } else {
            //[表单]已失效
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,  Definition.MSG_301020_0008);
            return outResult(Definition.RESULT_E);
        }
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        outMsg = Definition.NULL_STR_1;
        userId = ClsBase.getCurrentUser();
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }

    /**
     *返回
     */
    private OutBase outResult(String resCode) {
        OutBase resData = new OutBase();
        resData.setLogin_user(userId);
        resData.setErrorDesc(outMsg);
        resData.setResultCode(resCode);
        return resData;
    }

}
