package com.ruoyi.project.workflow.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.common.ExcelBase;
import com.ruoyi.project.workflow.domain.base.InBase;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.base.OutButtonBase;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignInvalidIn;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignInvalidOut;
import com.ruoyi.project.workflow.domain.input.WFormDesignInvalid;
import com.ruoyi.project.workflow.domain.output.MapData;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.service.IBFunctionMsService;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.IBWordbookMsService;
import com.ruoyi.project.workflow.service.WFormDesignInvalidService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

@RestController
@RequestMapping("/invalid")
public class WFormDesignInvalidController {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private IBFunctionMsService iBFunctionMsService;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;
    @Autowired
    private WFormDesignInvalidService wFormDesignInvalidService;

    //消息
    private String outMsg;
    //用户
    private String userId;
    //language
    private String language;

    @PostMapping("/init")
    public MapData getInit(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());
        //返回
        MapData resData = new MapData();
        //按钮名
        OutButtonBase outButtonBase = iBWordbookMsService.selectButtonInfo(language);
        outButtonBase.setI_btn_import(Definition.NULL_STR_1);
        outButtonBase.setI_btn_delete(Definition.NULL_STR_1);
        outButtonBase.setI_btn_add(Definition.NULL_STR_1);
        resData.getData().put(Definition.STR_BUTTON, outButtonBase);
        //每页显示条数
        resData.getData().put(Definition.STR_I_PERPAGE, iBWordbookMsService.selectPerPage(language));
        //得分页条数
        resData.getData().put(Definition.STR_PERPAGELIST, iBWordbookMsService.selectDataList(language,
                Definition.WORD_BOOK_TYPE_99_006, Definition.FLG_Y));
        resData.setLogin_user(userId);
        resData.setResultCode(Definition.RESULT_S);
        return resData;
    }

    @PostMapping("/getList")
    public WorkFlowTableInfo getList(@RequestBody WFormDesignInvalid inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignInvalidIn selectIn = editSearch(inData);
        //总页数
        Long nTotal = wFormDesignInvalidService.selectInvalidCount(selectIn);
        //取得分页数据
        if (inData.getI_pageindex() == 1) {
            selectIn.setOffSet(Long.valueOf(0));
            if (nTotal > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal);
            }
        } else {
            selectIn.setOffSet(inData.getI_perpage() * (inData.getI_pageindex() - 1));
            if ((nTotal - selectIn.getOffSet()) > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal - selectIn.getOffSet());
                if (selectIn.getNum() < 0) {
                    selectIn.setNum(Long.valueOf(0));
                }
            }
        }
        List<WFormDesignInvalidOut> list = wFormDesignInvalidService.selectInvalidList(selectIn);
        //返回数据
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setLogin_user(userId);
        if (list != null) {
            respData.setDataCount(String.valueOf(list.size()));
            respData.setDataList(list);
        }
        respData.setI_total(String.valueOf(nTotal));
        respData.setResultCode(Definition.RESULT_S);

        return respData;
    }

    @PostMapping("/export")
    @Log(title = Definition.PAGE_401050, businessType = BusinessType.EXPORT)
    public void export(HttpServletResponse response, @RequestBody WFormDesignInvalid inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignInvalidIn selectIn = editSearch(inData);
        //取得数据
        List<WFormDesignInvalidOut> list = wFormDesignInvalidService.selectInvalidUnLimt(selectIn);
        //gridInfo
        if (inData.getTable_info() == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0100);
            errorResponse(response);
        }
        //取得页面名称
        String strDownloadFileName = iBFunctionMsService.GetFuncName(language, Definition.PAGE_401050);
        //excel名
        String excelName = ClsBase.getEccelName(strDownloadFileName, userId);
        //导出
        ExcelBase<WFormDesignInvalidOut> util = new ExcelBase<>(WFormDesignInvalidOut.class);
        util.exportExcel(response, inData.getTable_info(), list, Definition.EXPORT_INFO, excelName);
    }

    /*
    检索条件
    */
    private WFormDesignInvalidIn editSearch(WFormDesignInvalid inData){
        WFormDesignInvalidIn wFormDesignInvalidIn = new WFormDesignInvalidIn();
        //用户
        wFormDesignInvalidIn.setUserId(userId);
        //申请关联
        wFormDesignInvalidIn.setiSchField(ClsBase.getTrimData(inData.getI_sch_field()));
        return wFormDesignInvalidIn;
    }

    /*
     *error
     */
    private void errorResponse(HttpServletResponse response){
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        try {
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            PrintWriter pw = response.getWriter();
            pw.write(rspData.toString());
        } catch (IOException e) {
            log.error("返回异常{}", e.getMessage());
        }
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        userId = ClsBase.getCurrentUser();
        outMsg = Definition.NULL_STR_1;
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }
}
