package com.ruoyi.project.workflow.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.common.ExcelBase;
import com.ruoyi.project.workflow.domain.WFormLayout;
import com.ruoyi.project.workflow.domain.base.InBase;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignNewAppLyIn;
import com.ruoyi.project.workflow.domain.db.out.ApplyRulesOut;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignNewApplyOut;
import com.ruoyi.project.workflow.domain.input.WFormDesignNewAppLy;
import com.ruoyi.project.workflow.domain.input.WFormDesignNewAppLyDetail;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInit;
import com.ruoyi.project.workflow.service.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/newApply")
public class WFormDesignNewApplyController {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private WFormDesignNewApplyService wFormDesignNewAppLyService;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private IBFunctionMsService iBFunctionMsService;
    @Autowired
    private WFormDesignService wFormDesignService;
    @Autowired
    private WApplicationService wApplicationService;
    @Autowired
    private WFormLayoutService wFormLayoutService;
    @Autowired
    private SendMailService sendMailService;
    @Autowired
    private WFlowTaskService wFlowTaskService;

    //消息
    private String outMsg;
    //用户
    private String userId;
    //language
    private String language;

    @PostMapping("/init")
    public WorkFlowTableInit getInit(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());
        //返回
        WorkFlowTableInit resData = new WorkFlowTableInit();
        //每页显示条数
        resData.setI_perpage(iBWordbookMsService.selectPerPage(language));
        //得分页条数
        resData.setPerpageList(iBWordbookMsService.selectDataList(language, Definition.WORD_BOOK_TYPE_99_006,
                Definition.FLG_Y));
        //按钮名
        resData.setButton(iBWordbookMsService.selectButtonInfo(language));
        resData.getButton().setI_btn_import(Definition.NULL_STR_1);
        resData.getButton().setI_btn_delete(Definition.NULL_STR_1);
        resData.getButton().setI_btn_add(Definition.NULL_STR_1);

        resData.setLogin_user(userId);
        resData.setResultCode(Definition.RESULT_S);

        return resData;
    }

    @PostMapping("/getList")
    public WorkFlowTableInfo getList(@RequestBody WFormDesignNewAppLy inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignNewAppLyIn selectIn = editSearch(inData);
        //总页数
        Long nTotal = wFormDesignNewAppLyService.selectNewApplyCount(selectIn);
        //取得分页数据
        if (inData.getI_pageindex() == 1) {
            selectIn.setOffSet(Long.valueOf(0));
            if (nTotal > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal);
            }
        } else {
            selectIn.setOffSet(inData.getI_perpage() * (inData.getI_pageindex() - 1));
            if ((nTotal - selectIn.getOffSet()) > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal - selectIn.getOffSet());
                if (selectIn.getNum() < 0) {
                    selectIn.setNum(Long.valueOf(0));
                }
            }
        }
        List<WFormDesignNewApplyOut> list = wFormDesignNewAppLyService.selectNewApplyList(selectIn);
        //返回数据
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setLogin_user(userId);
        if (list != null) {
            respData.setDataCount(String.valueOf(list.size()));
            respData.setDataList(list);
        }
        respData.setI_total(String.valueOf(nTotal));
        respData.setResultCode(Definition.RESULT_S);

        return respData;
    }

    @PostMapping("/export")
    @Log(title = Definition.PAGE_401010, businessType = BusinessType.EXPORT)
    public void export(HttpServletResponse response, @RequestBody WFormDesignNewAppLy inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignNewAppLyIn selectIn = editSearch(inData);
        //gridInfo
        if (inData.getTable_info() == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0100);
            errorResponse(response);
        }
        //取得数据
        List<WFormDesignNewApplyOut> list = wFormDesignNewAppLyService.selectNewApplyUnLimt(selectIn);
        //取得页面名称
        String strDownloadFileName = iBFunctionMsService.GetFuncName(language, Definition.PAGE_401010);
        //excel名
        String excelName = ClsBase.getEccelName(strDownloadFileName, userId);
        //导出
        ExcelBase<WFormDesignNewApplyOut> util = new ExcelBase<>(WFormDesignNewApplyOut.class);
        util.exportExcel(response, inData.getTable_info(), list, Definition.EXPORT_INFO, excelName);
    }

    @PostMapping("/save")
    @Log(title = Definition.PAGE_401010, businessType = BusinessType.UPDATE)
    public OutBase SaveApply(@RequestBody Map<String, Object> inMapData) {
        //入力变换
        WFormDesignNewAppLyDetail inData = toBean(inMapData);
        //初期化
        init(inData.getI_language());
        //表单Id
        String striFormId = ClsBase.getTrimData(inData.getFormId());
        if (StringUtils.isEmpty(striFormId)) {
            //[表单Id]不能为空
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010,  Definition.MSG_401010_0001);
            return returnOutBase(Definition.RESULT_E);
        }
        //前回申请Id
        String striPreApplId = ClsBase.getTrimData(inData.getI_pre_appl_id());
        //提交操作
        if (inData.getSubmit() == null) {
            //[申请操作]不能为空
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010,  Definition.MSG_401010_0006);
            return returnOutBase(Definition.RESULT_E);
        }
        //CC者
        String striReferenceUser = ClsBase.getTrimData(inData.getI_notice_readers());
        //表单状态取得
        String strStatus = wFormDesignService.getFormStatusByFormId(striFormId);
        if (StringUtils.isNotEmpty(strStatus) && Definition.CLOSED.equals(strStatus)) {
            //过期的场合
            if (inData.getSubmit()) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010,  Definition.MSG_401010_0008);
                return returnOutBase(Definition.RESULT_E);
            } else {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010,  Definition.MSG_401010_0007);
                return returnOutBase(Definition.RESULT_E);
            }
        }
        //采番规则取得
        ApplyRulesOut applyRulesOut = wFormDesignNewAppLyService.selectNumberRules(striFormId);
        if (applyRulesOut == null) {
            //[表单]不存在
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010,  Definition.MSG_401010_0002);
            return returnOutBase(Definition.RESULT_E);
        }
        //申请ID采番
        String striApplyId = wApplicationService.getApplyId(applyRulesOut.getiNumberRules());
        //通过表单Id获取申请数据表结构
        List<WFormLayout> listWFormLayout = wFormLayoutService.selectStructureByFormId(striFormId);
        if (listWFormLayout == null || listWFormLayout.size() == 0) {
            //申请书结构查询异常
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010, Definition.MSG_401010_0003);
            return returnOutBase(Definition.RESULT_E);
        } else {
            if (StringUtils.isEmpty(listWFormLayout.get(0).getiFormDataTblname())) {
                //[申请数据表]不存在
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401010, Definition.MSG_401010_0003);
                return returnOutBase(Definition.RESULT_E);
            }
        }
        //表名
        String strTableName = listWFormLayout.get(0).getiFormDataTblname();
        //Sql做成
        String strCols = Definition.LEFT_CURVE + Definition.I_APPLY_ID + Definition.ROLE_DELIMETER;
        String strVals = Definition.LEFT_CURVE + Definition.SINGLE_QUOTATION_MARK + striApplyId +
                Definition.SINGLE_QUOTATION_MARK + Definition.ROLE_DELIMETER;
        //附件
        List<String> uploadFiles = new ArrayList<>();
        List<String> defaultFiles = new ArrayList<>();
        //遍历数据表的列
        for (WFormLayout item : listWFormLayout) {
            String strCol = ClsBase.getTrimData(item.getiFormDataCol());
            if (StringUtils.isNotEmpty(strCol)) {
                String strVal = Definition.NULL_STR_1;
                //控件值
                if (inData.getParams().get(strCol) != null) {
                    strVal = ClsBase.getTrimData(inData.getParams().get(strCol).toString());
                }
                //控件为upload的场合
                if (item.getiFormDataControl().startsWith(Definition.UPLOAD)) {
                    Collections.addAll(uploadFiles, strVal.split(Definition.RADICAL_LINE));
                    if (item.getiFormDataControlInitval() != null) {
                        Collections.addAll(defaultFiles, item.getiFormDataControlInitval().split(Definition.RADICAL_LINE));
                    }
                }
                strCols = strCols + Definition.QUOTATION_MARK + strCol +
                        Definition.QUOTATION_MARK + Definition.ROLE_DELIMETER;
                if (StringUtils.isEmpty(strVal)) {
                    strVals = strVals + Definition.NULL_STR_2 + Definition.ROLE_DELIMETER;
                } else {
                    strVals = strVals + Definition.SINGLE_QUOTATION_MARK + strVal +
                            Definition.SINGLE_QUOTATION_MARK + Definition.ROLE_DELIMETER;
                }
            }
        }
        //末尾逗号去除
        strCols = strCols.substring(0, strCols.length() - 1);
        strVals = strVals.substring(0, strVals.length() - 1);
        strCols = strCols + Definition.RIGHT_CURVE;
        strVals = strVals + Definition.RIGHT_CURVE;
        //创建申请书文件夹，并迁移上传的文件
        createFolderAndMoveFile(striFormId, striApplyId, uploadFiles, defaultFiles);
        if (StringUtils.isNotEmpty(outMsg)) {
            return returnOutBase(Definition.RESULT_E);
        }
        //保存申请数据
        String resMsg = wFormDesignNewAppLyService.saveApplyData(language, striFormId, striApplyId,
                striPreApplId, striReferenceUser, strCols, strVals, strTableName);
        if (!Definition.RESULT_S.equals(resMsg)) {
            outMsg = resMsg;
            return returnOutBase(Definition.RESULT_E);
        }
        //提交的场合，生成流转流程
        if (inData.getSubmit()) {
            //生成流转流程
            resMsg = wFlowTaskService.generateTasks(language, striFormId, striApplyId, Definition.FST_VERSION,
                    applyRulesOut.getiFlowData(), inData.getParams());
            if (!Definition.RESULT_S.equals(resMsg)) {
                outMsg = resMsg;
                return returnOutBase(Definition.RESULT_E);
            }
            //发送审批请求邮件
            resMsg = sendMailService.aduitApplyMail(language, striApplyId, Definition.FST_VERSION, striReferenceUser);
            if (!Definition.RESULT_S.equals(resMsg)) {
                outMsg = resMsg;
                return returnOutBase(Definition.RESULT_E);
            }
            //提交成功
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0120);
            return returnOutBase(Definition.RESULT_S);
        } else {
            //保存成功
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0090);
            return returnOutBase(Definition.RESULT_S);
        }
    }

    /*
    创建申请书文件夹，并迁移上传的文件
     */
    private void createFolderAndMoveFile(String formId, String applyId, List<String> fileNames,
                                         List<String> defaultFileNames) {
        //格纳文件夹
        String strSavePath = iBWordbookMsService.selectWordbookNameByKeys(language, Definition.WORD_BOOK_TYPE_99_001,
                Definition.WORD_BOOK_NO_99_001_0200, Definition.FLG_Y);
        //临时文件夹
        String strTempPath = iBWordbookMsService.selectWordbookNameByKeys(language, Definition.WORD_BOOK_TYPE_99_001,
                Definition.WORD_BOOK_NO_99_001_0210, Definition.FLG_Y);
        //判断格纳文件夹，并创建
        File fileSavePath = new File(strSavePath);
        if (!fileSavePath.exists() || !fileSavePath.isDirectory()) {
            fileSavePath.mkdir();
        }
        String strSavePathzi = strSavePath + Definition.STR_XIEG + formId + Definition.HENG_GANG_X + applyId;
        //判断格纳文件夹
        File fileSavePathzi = new File(strSavePathzi);
        if (fileSavePathzi.exists() && fileSavePathzi.isDirectory()) {
            //删除当前文件夹
            fileSavePathzi.delete();
        }
        //创建申请书文件夹
        fileSavePathzi.mkdir();
        //判断临时文件夹，不存在的场合，跳出方法
        File fileTempPath = new File(strTempPath);
        if (!fileTempPath.exists() || !fileTempPath.isDirectory()) {
            return;
        }
        //文件迁移
        for (String filename : fileNames) {
            if (StringUtils.isNotEmpty(filename)) {
                //临时文件
                Path strSourceFile = Paths.get(strTempPath + Definition.STR_XIEG + filename);
                //格纳文件
                Path strDestFile = Paths.get(fileSavePathzi + Definition.STR_XIEG + filename);
                //文件是否存在
                if (strSourceFile.toFile().exists()) {
                    if (defaultFileNames.contains(filename)) {
                        try {
                            Files.copy(strSourceFile, strDestFile);
                        } catch (Exception e) {
                            outMsg = e.getMessage();
                            return;
                        }
                    } else {
                        try {
                            Files.move(strSourceFile, strDestFile);
                        } catch (Exception e) {
                            outMsg = e.getMessage();
                            return;
                        }
                    }
                } else {
                    outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,
                            Definition.MSG_COMMON_0126) +
                            Definition.LEFT_BRACKET + filename + Definition.RIGHT_BRACKET;
                    return;
                }
            }
        }
    }

    /*
    检索条件
     */
    private WFormDesignNewAppLyIn editSearch (WFormDesignNewAppLy inData) {
        WFormDesignNewAppLyIn selectIn = new WFormDesignNewAppLyIn();
        //检索表单相关
        selectIn.setSchField(ClsBase.getTrimData(inData.getI_sch_field()));
        //检索所属公司
        selectIn.setSchDeptNo(ClsBase.getTrimData(inData.getI_sch_dept()));
        //是否为申请者查询
        if (StringUtils.isEmpty(inData.getI_is_applicant())) {
            selectIn.setIsApplicant(Definition.FLG_1);
        } else {
            selectIn.setIsApplicant(ClsBase.getTrimData(inData.getI_is_applicant()));
        }
        //部门阶层
        SysDept sysDept = ClsBase.getCurrentUserDept();
        if (sysDept != null) {
            selectIn.setDeptAndParents(sysDept.getAncestors() +
                    Definition.ROLE_DELIMETER + String.valueOf(sysDept.getDeptId()));
        }
        //admin用户flg
        selectIn.setAdminFlg(ClsBase.isAdmin());
        //用户id
        selectIn.setUserId(ClsBase.getCurrentUser());

        return selectIn;
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        userId = ClsBase.getCurrentUser();
        outMsg = Definition.NULL_STR_1;
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }

    /*
     *error
     */
    private void errorResponse(HttpServletResponse response){
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        try {
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            PrintWriter pw = response.getWriter();
            pw.write(rspData.toString());
        } catch (IOException e) {
            log.error("返回异常{}", e.getMessage());
        }
    }

    /*
    返回OutBase
     */
    private OutBase returnOutBase(String resultCode) {
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(resultCode);
        return rspData;
    }

    /**
     * 入力变换
     */
    private WFormDesignNewAppLyDetail toBean(Map<String, Object> map) {
        WFormDesignNewAppLyDetail resBean = new WFormDesignNewAppLyDetail();
        if (map != null) {
            for (String inKey : map.keySet()) {
                switch (inKey) {
                    case Definition.STR_FORMID:
                        resBean.setFormId(String.valueOf(map.get(inKey)));
                        break;
                    case Definition.STR_I_PRE_APPL_ID:
                        resBean.setI_pre_appl_id(String.valueOf(map.get(inKey)));
                        break;
                    case Definition.STR_I_NOTICE_READERS:
                        resBean.setI_notice_readers(String.valueOf(map.get(inKey)));
                        break;
                    case Definition.STR_IS_SUBMIT:
                        if (map.get(inKey) != null) {
                            resBean.setSubmit(Boolean.valueOf(map.get(inKey).toString()));
                        }
                        break;
                    case Definition.STR_LOGIN_USER:
                        resBean.setLogin_user(String.valueOf(map.get(inKey)));
                        break;
                    case Definition.I_LANGUAGE:
                        resBean.setI_language(String.valueOf(map.get(inKey)));
                        break;
                    default:
                        resBean.getParams().put(inKey, map.get(inKey));
                        break;
                }
            }
        }
        return resBean;
    }
}
