package com.ruoyi.project.workflow.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.common.ExcelBase;
import com.ruoyi.project.workflow.domain.base.InBase;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.base.OutButtonBase;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignPendingIn;
import com.ruoyi.project.workflow.domain.db.out.ProxyDetailOut;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignPendingOut;
import com.ruoyi.project.workflow.domain.input.ProxyDetailIn;
import com.ruoyi.project.workflow.domain.input.WFormDesignPending;
import com.ruoyi.project.workflow.domain.input.WFormDesignPendingAudit;
import com.ruoyi.project.workflow.domain.output.FlowTask;
import com.ruoyi.project.workflow.domain.output.MapData;
import com.ruoyi.project.workflow.domain.output.WFormPendingAudit;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.service.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

@RestController
@RequestMapping("/pending")
public class WFormDesignPendingController {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private IBWordbookMsService iBWordbookMsService;
    @Autowired
    private WFormDesignPendingService wFormDesignPendingService;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private IBFunctionMsService iBFunctionMsService;
    @Autowired
    private WProxySettingsService wProxySettingsService;
    @Autowired
    private WApplicationService wApplicationService;
    @Autowired
    private WFlowUserService wFlowUserService;
    @Autowired
    private WFlowTaskService wFlowTaskService;
    @Autowired
    private SendMailService sendMailService;

    //消息
    private String outMsg;
    //用户
    private String userId;
    //language
    private String language;

    @PostMapping("/init")
    public MapData getInit(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());
        //返回
        MapData resData = new MapData();
        //按钮名
        OutButtonBase outButtonBase = iBWordbookMsService.selectButtonInfo(language);
        outButtonBase.setI_btn_import(Definition.NULL_STR_1);
        outButtonBase.setI_btn_delete(Definition.NULL_STR_1);
        outButtonBase.setI_btn_add(Definition.NULL_STR_1);
        resData.getData().put(Definition.STR_BUTTON, outButtonBase);
        //每页显示条数
        resData.getData().put(Definition.STR_I_PERPAGE, iBWordbookMsService.selectPerPage(language));
        //得分页条数
        resData.getData().put(Definition.STR_PERPAGELIST, iBWordbookMsService.selectDataList(language,
                Definition.WORD_BOOK_TYPE_99_006, Definition.FLG_Y));
        resData.setLogin_user(userId);
        resData.setResultCode(Definition.RESULT_S);
        return resData;
    }

    @PostMapping("/getList")
    public WorkFlowTableInfo getList(@RequestBody WFormDesignPending inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignPendingIn selectIn = editSearch(inData);
        //总页数
        Long nTotal = wFormDesignPendingService.selectPendingCount(selectIn);
        //取得分页数据
        if (inData.getI_pageindex() == 1) {
            selectIn.setOffSet(Long.valueOf(0));
            if (nTotal > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal);
            }
        } else {
            selectIn.setOffSet(inData.getI_perpage() * (inData.getI_pageindex() - 1));
            if ((nTotal - selectIn.getOffSet()) > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal - selectIn.getOffSet());
                if (selectIn.getNum() < 0) {
                    selectIn.setNum(Long.valueOf(0));
                }
            }
        }
        List<WFormDesignPendingOut> list = wFormDesignPendingService.selectPendingList(selectIn);
        //返回数据
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setLogin_user(userId);
        if (list != null) {
            respData.setDataCount(String.valueOf(list.size()));
            respData.setDataList(list);
        }
        respData.setI_total(String.valueOf(nTotal));
        respData.setResultCode(Definition.RESULT_S);

        return respData;
    }

    @PostMapping("/export")
    @Log(title = Definition.PAGE_401030, businessType = BusinessType.EXPORT)
    public void export(HttpServletResponse response, @RequestBody WFormDesignPending inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WFormDesignPendingIn selectIn = editSearch(inData);
        //取得数据
        List<WFormDesignPendingOut> list = wFormDesignPendingService.selectPendingUnLimt(selectIn);
        //gridInfo
        if (inData.getTable_info() == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0100);
            errorResponse(response);
        }
        //取得页面名称
        String strDownloadFileName = iBFunctionMsService.GetFuncName(language, Definition.PAGE_401030);
        //excel名
        String excelName = ClsBase.getEccelName(strDownloadFileName, userId);
        //导出
        ExcelBase<WFormDesignPendingOut> util = new ExcelBase<>(WFormDesignPendingOut.class);
        util.exportExcel(response, inData.getTable_info(), list, Definition.EXPORT_INFO, excelName);
    }

    @PostMapping("/audit")
    @Log(title = Definition.PAGE_401030, businessType = BusinessType.UPDATE)
    public WFormPendingAudit auditData(@RequestBody WFormDesignPendingAudit inData) {
        //初期化
        init(inData.getI_language());
        //申请Id
        String applyId = ClsBase.getTrimData(inData.getI_apply_id());
        if (StringUtils.isEmpty(applyId)) {
            //[申请编号]不能为空
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401030,  Definition.MSG_401030_0001);
            return outAudit(Definition.RESULT_E, false);
        }
        //审批内容
        String comment = ClsBase.getTrimData(inData.getI_comment());
        //审批结果
        String auditResult = ClsBase.getTrimData(inData.getI_audit_result());
        if (StringUtils.isEmpty(auditResult)) {
            //[审批结果]不能为空
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401030,  Definition.MSG_401030_0003);
            return outAudit(Definition.RESULT_E, false);
        } else {
            if (!Definition.STR_PASS.equals(auditResult) && !Definition.STR_REJECT.equals(auditResult)) {
                //[承认/否决]以外，操作不可
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401030,  Definition.MSG_401030_0002);
                return outAudit(Definition.RESULT_E, false);
            }
        }
        //获取提交回数
        Long submitTimes = wApplicationService.selectApplyMaxTimes(applyId);
        //保存审批数据
        String resMsg = wFlowUserService.updateAudit(language, applyId, submitTimes, auditResult, comment);
        if (!Definition.RESULT_S.equals(resMsg)) {
            outMsg = resMsg;
            return outAudit(Definition.RESULT_E, false);
        }
        //获得审批中节点的信息
        List<FlowTask> flowTaskList = wFlowTaskService.getCurrentNodeById(applyId, submitTimes);
        if (flowTaskList == null || flowTaskList.size() == 0) {
            //审批流程不存在，请确认流程
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_401030,  Definition.MSG_401030_0004);
            return outAudit(Definition.RESULT_E, false);
        }
        //会签或签判定
        Boolean isCounterSign = Definition.FLG_1.equals(flowTaskList.get(0).getwFlowTask().getiNodeRelationship());
        //存在承認済
        Boolean existPassed = false;
        //存在拒否
        Boolean existRejected = false;
        //all承認済
        Boolean allPassed = true;
        //申请承認済
        Boolean applyPass = false;
        for (FlowTask flowTask : flowTaskList) {
            if (Definition.STR_ANY.equals(flowTask.getwFlowTask().getiNodeProperty())){
                //任意的场合
                if (flowTask.getAduitResult_NG().size() > 0) {
                    existRejected = true;
                } else {
                    if (flowTask.getAduitResult_OK().size() > 0) {
                        existPassed = true;
                        allPassed = allPassed && true;
                    } else {
                        allPassed = false;
                    }
                }
            } else if (Definition.STR_ALL.equals(flowTask.getwFlowTask().getiNodeProperty())) {
                //全体的场合
                if (flowTask.getAduitResult_NG().size() > 0) {
                    existRejected = true;
                } else {
                    if (flowTask.getAduitResult_OK().size() == flowTask.getUserList().size()) {
                        existPassed = true;
                        allPassed = allPassed && true;
                    } else {
                        allPassed = false;
                    }
                }
            } else {
                //master及未设定（个人，上司）的场合
                if (flowTask.getAduitResult_NG().size() > 0) {
                    existRejected = true;
                } else {
                    if (flowTask.getAduitResult_OK().size() > 0) {
                        existPassed = true;
                        allPassed = allPassed && true;
                    } else {
                        allPassed = false;
                    }
                }
            }
        }
        //存在审批否决的场合，申请否决
        if (existRejected) {
            //task 完了，appl 否决
            resMsg = wFormDesignPendingService.updateTaskAndApply(language, applyId, submitTimes,
                    Definition.STR_REJECT, Definition.STR_REJECT, Definition.NULL_STR_1);
            if (!Definition.RESULT_S.equals(resMsg)) {
                outMsg = resMsg;
                return outAudit(Definition.RESULT_E, false);
            }
            //邮件通知申请者及承认过的审批者
            resMsg = sendMailService.rejectApplyMail(language, applyId, submitTimes);
            if (!Definition.RESULT_S.equals(resMsg)) {
                outMsg = resMsg;
                return outAudit(Definition.RESULT_E, false);
            }
        } else {
            if ((isCounterSign && allPassed) || (!isCounterSign && existPassed)){
                //task 完了
                if (StringUtils.isEmpty(flowTaskList.get(0).getwFlowTask().getiNexNodeId())) {
                    //apply 承认OK
                    resMsg = wFormDesignPendingService.updateTaskAndApply(language, applyId, submitTimes,
                            Definition.STR_PASS, Definition.STR_PASS, Definition.NULL_STR_1);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        outMsg = resMsg;
                        return outAudit(Definition.RESULT_E, false);
                    }
                    //邮件通知申请者
                    resMsg = sendMailService.passApplyMail(language, applyId, submitTimes);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        outMsg = resMsg;
                        return outAudit(Definition.RESULT_E, false);
                    }
                    applyPass = true;
                } else {
                    resMsg = wFormDesignPendingService.updateTaskAndApply(language, applyId, submitTimes,
                            Definition.STR_PASS, Definition.NULL_STR_1,
                            flowTaskList.get(0).getwFlowTask().getiNexNodeId());
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        outMsg = resMsg;
                        return outAudit(Definition.RESULT_E, false);
                    }
                    //邮件通知下一个节点的承认者
                    resMsg = sendMailService.aduitApplyMail(language, applyId, submitTimes, Definition.NULL_STR_1);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        outMsg = resMsg;
                        return outAudit(Definition.RESULT_E, false);
                    }
                }
            }
        }
        //正常返回
        outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0060);
        return outAudit(Definition.RESULT_S, applyPass);
    }

    @PostMapping("/proxyDetail")
    public WorkFlowTableInfo getProxyDetail(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());
        //获取数据
        List<ProxyDetailOut> list = wProxySettingsService.selectProxyDetail(userId);
        //返回数据
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setLogin_user(userId);
        if (list != null) {
            respData.setDataCount(String.valueOf(list.size()));
            respData.setDataList(list);
            respData.setI_total(String.valueOf(list.size()));
        }
        respData.setResultCode(Definition.RESULT_S);
        return respData;
    }

    @PostMapping("/saveProxy")
    @Log(title = Definition.PAGE_401030, businessType = BusinessType.INSERT)
    public OutBase saveProxy(@RequestBody ProxyDetailIn inData) {
        //初期化
        init(inData.getI_language());
        //解析代理人列表
        if (inData.getDatalist() != null && inData.getDatalist().size() > 0) {
            //删除,插入
           String resMsg = wProxySettingsService.updateProxyDetail(language, userId, inData.getDatalist());
           if (!Definition.RESULT_S.equals(resMsg)) {
               outMsg = resMsg;
               return returnOutBase(Definition.RESULT_E);
           }
        }
        outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0090);
        return returnOutBase(Definition.RESULT_S);
    }

    /*
    检索条件
     */
    private WFormDesignPendingIn editSearch(WFormDesignPending inDat) {
        WFormDesignPendingIn wFormDesignPendingIn = new WFormDesignPendingIn();
        //申请关联
        wFormDesignPendingIn.setiSchField(ClsBase.getTrimData(inDat.getI_sch_field()));
        //申请者关联
        wFormDesignPendingIn.setiSchApplyUser(ClsBase.getTrimData(inDat.getI_sch_apply_user()));
        //公司
        wFormDesignPendingIn.setiSchDept(ClsBase.getTrimData(inDat.getI_sch_dept()));
        //admin用户flg
        wFormDesignPendingIn.setAdminFlg(ClsBase.isAdmin());
        //用户
        wFormDesignPendingIn.setUserid(userId);
        return wFormDesignPendingIn;
    }

    /*
     *error
     */
    private void errorResponse(HttpServletResponse response){
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        try {
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            PrintWriter pw = response.getWriter();
            pw.write(rspData.toString());
        } catch (IOException e) {
            log.error("返回异常{}", e.getMessage());
        }
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        userId = ClsBase.getCurrentUser();
        outMsg = Definition.NULL_STR_1;
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }

    /*
    返回OutBase
    */
    private OutBase returnOutBase(String resultCode) {
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(resultCode);
        return rspData;
    }

    /*
    返回WFormPendingAudit
    */
    private WFormPendingAudit outAudit(String resultCode, Boolean applyPass) {
        WFormPendingAudit rspData = new WFormPendingAudit();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setApplyPass(applyPass);
        rspData.setResultCode(resultCode);
        return rspData;
    }
}
