package com.ruoyi.project.workflow.controller;

import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.aspectj.lang.annotation.Log;
import com.ruoyi.framework.aspectj.lang.enums.BusinessType;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.common.ExcelBase;
import com.ruoyi.project.workflow.domain.WGroup;
import com.ruoyi.project.workflow.domain.base.InBase;
import com.ruoyi.project.workflow.domain.base.OutBase;
import com.ruoyi.project.workflow.domain.base.OutButtonBase;
import com.ruoyi.project.workflow.domain.db.in.WGroupYilanIn;
import com.ruoyi.project.workflow.domain.db.out.BLanguageMsOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupDetailOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupMemberOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupYilanOut;
import com.ruoyi.project.workflow.domain.input.*;
import com.ruoyi.project.workflow.domain.output.MapData;
import com.ruoyi.project.workflow.domain.output.WorkFlowTableInfo;
import com.ruoyi.project.workflow.service.IBFunctionMsService;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.IBWordbookMsService;
import com.ruoyi.project.workflow.service.WGroupService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/groupManage")
public class WGroupController {

    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private IBFunctionMsService iBFunctionMsService;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;
    @Autowired
    private WGroupService wGroupService;

    //消息
    private String outMsg;
    //用户
    private String userId;
    //language
    private String language;

    @PostMapping("/init")
    public MapData getInit(@RequestBody InBase inData) {
        //初期化
        init(inData.getI_language());
        //返回
        MapData resData = new MapData();
        //按钮名
        OutButtonBase outButtonBase = iBWordbookMsService.selectButtonInfo(language);
        outButtonBase.setI_btn_import(Definition.NULL_STR_1);
        outButtonBase.setI_btn_delete(Definition.NULL_STR_1);
        resData.getData().put(Definition.STR_BUTTON, outButtonBase);
        //每页显示条数
        resData.getData().put(Definition.STR_I_PERPAGE, iBWordbookMsService.selectPerPage(language));
        //得分页条数
        resData.getData().put(Definition.STR_PERPAGELIST, iBWordbookMsService.selectDataList(language,
                Definition.WORD_BOOK_TYPE_99_006, Definition.FLG_Y));
        //有效flg
        String flg = iBWordbookMsService.selectWordbookNameByKeys(language, Definition.WORD_BOOK_TYPE_10_001,
                Definition.FLG_Y, Definition.FLG_Y);
        if (StringUtils.isEmpty(flg)) {
            resData.getData().put(Definition.STR_I_ENABLE_FLAG, Definition.NULL_STR_1);
        } else {
            resData.getData().put(Definition.STR_I_ENABLE_FLAG, Definition.FLG_Y);
        }
        //取得有效标志
        resData.getData().put(Definition.STR_I_ENABLE_LIST, iBWordbookMsService.selectDataList(language,
                Definition.WORD_BOOK_TYPE_10_001, Definition.FLG_Y));
        //user
        resData.setLogin_user(userId);
        //结果
        resData.setResultCode(Definition.RESULT_S);

        return resData;
    }

    @PostMapping("/getList")
    public WorkFlowTableInfo getList(@RequestBody WGroupYilan inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WGroupYilanIn selectIn = editSearch(inData);
        //总页数
        Long nTotal = wGroupService.selectWGroupCount(selectIn);
        //取得分页数据
        if (inData.getI_pageindex() == 1) {
            selectIn.setOffSet(Long.valueOf(0));
            if (nTotal > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal);
            }
        } else {
            selectIn.setOffSet(inData.getI_perpage() * (inData.getI_pageindex() - 1));
            if ((nTotal - selectIn.getOffSet()) > inData.getI_perpage()) {
                selectIn.setNum(inData.getI_perpage());
            } else {
                selectIn.setNum(nTotal - selectIn.getOffSet());
                if (selectIn.getNum() < 0) {
                    selectIn.setNum(Long.valueOf(0));
                }
            }
        }
        List<WGroupYilanOut> list = wGroupService.selectWGroupList(selectIn);
        //返回数据
        WorkFlowTableInfo respData = new WorkFlowTableInfo();
        respData.setLogin_user(userId);
        if (list != null) {
            respData.setDataCount(String.valueOf(list.size()));
            respData.setDataList(list);
        }
        respData.setI_total(String.valueOf(nTotal));
        respData.setResultCode(Definition.RESULT_S);

        return respData;
    }

    @PostMapping("/getDetail")
    public MapData getDetail(@RequestBody WGroupDetail inData) {
        //初期化
        init(inData.getI_language());
        //操作
        String operate = ClsBase.getTrimData(inData.getI_operate());
        if (StringUtils.isEmpty(operate)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0020);
            return reMapData(Definition.RESULT_E);
        } else {
            if (!Definition.STR_ADD.equals(operate) && !Definition.STR_UPDATE.equals(operate)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0021);
                return reMapData(Definition.RESULT_E);
            }
        }
        //角色ID
        if (Definition.STR_UPDATE.equals(operate)) {
            if (inData.getI_role_id() == null) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0001);
                return reMapData(Definition.RESULT_E);
            }
        }
        //取得角色详细信息
        List<WGroupDetailOut> list = new ArrayList<>();
        WGroupDetailOut wGroupDetailOut = wGroupService.selectWGroupDetailById(inData.getI_role_id());
        list.add(wGroupDetailOut);
        //按钮
        String strBtnSave = Definition.NULL_STR_1;
        if (Definition.STR_ADD.equals(operate) || Definition.STR_UPDATE.equals(operate)) {
            strBtnSave = iBWordbookMsService.selectWordbookNameByKeys(language, Definition.WORD_BOOK_TYPE_99_004,
                    Definition.WORD_BOOK_NO_SAVE, Definition.FLG_Y);
        }
        //成员
        List<WGroupMemberOut> members = wGroupService.selectWGroupMemberById(inData.getI_role_id());
        MapData memberData = reMapData(Definition.RESULT_S);
        if (members != null && members.size() >0) {
            for (WGroupMemberOut member : members) {
                memberData.getData().put(member.getI_role_user(), member.getI_role_user_name());
            }
        }
        //返回
        MapData resData = reMapData(Definition.RESULT_S);
        resData.getData().put(Definition.STR_I_BTN_SAVE, strBtnSave);
        resData.getData().put(Definition.STR_I_BTN_DELETE, Definition.NULL_STR_1);
        resData.getData().put(Definition.STR_I_MSG_DELETE, Definition.NULL_STR_1);
        resData.getData().put(Definition.STR_DT_DETAIL, list);
        resData.getData().put(Definition.STR_MEMBER, memberData);

        return resData;
    }

    @PostMapping("/save")
    @Log(title = Definition.PAGE_801070, businessType = BusinessType.INSERT)
    public OutBase saveInfo(@RequestBody WGroupDetailIn inData) {
        //初期化
        init(inData.getI_language());
        //操作
        String operate = ClsBase.getTrimData(inData.getI_operate());
        if (StringUtils.isEmpty(operate)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0020);
            return reOutBase(Definition.RESULT_E);
        } else {
            if (!Definition.STR_ADD.equals(operate) && !Definition.STR_UPDATE.equals(operate)) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0021);
                return reOutBase(Definition.RESULT_E);
            }
        }
        //角色ID
        Long roleId = inData.getI_role_id();
        if (!Definition.STR_ADD.equals(operate)) {
            if (roleId == null) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0001);
                return reOutBase(Definition.RESULT_E);
            }
            if (String.valueOf(roleId).length() > 16) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0002);
                return reOutBase(Definition.RESULT_E);
            }
        }
        //角色ID采番
        if (Definition.STR_ADD.equals(operate)) {
            roleId =  wGroupService.selectNewGroupId();
        }
        //角色名称
        String roleName = ClsBase.getTrimData(inData.getI_role_name());
        if (StringUtils.isEmpty(roleName)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0004);
            return reOutBase(Definition.RESULT_E);
        }
        if (roleName.length() > 50) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0005);
            return reOutBase(Definition.RESULT_E);
        }
        //检查角色名是否存在
        List<String> roleNames = wGroupService.selectCheckExists(roleName, roleId);
        if (roleNames != null && roleNames.size() > 0) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0003);
            return reOutBase(Definition.RESULT_E);
        }
        //角色负责人
        String roleMaster = ClsBase.getTrimData(inData.getI_role_master());
        if (StringUtils.isEmpty(roleMaster)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0006);
            return reOutBase(Definition.RESULT_E);
        }
        if (roleMaster.length() > 16) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0007);
            return reOutBase(Definition.RESULT_E);
        }
        //角色说明
        String roleComment = ClsBase.getTrimData(inData.getI_role_comment());
        if (roleComment.length() > 500) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0008);
            return reOutBase(Definition.RESULT_E);
        }
        //有效标志
        String roleFlg = ClsBase.getTrimData(inData.getI_role_flg());
        if (StringUtils.isEmpty(roleFlg)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0040);
            return reOutBase(Definition.RESULT_E);
        }
        if (!Definition.FLG_Y.equals(roleFlg) && !Definition.FLG_N.equals(roleFlg)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0041);
            return reOutBase(Definition.RESULT_E);
        }
        //邮件地址
        String roleMl = ClsBase.getTrimData(inData.getI_role_ml());
        if (roleMl.length() > 64) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0009);
            return reOutBase(Definition.RESULT_E);
        }
        //角色成员
        String members = ClsBase.getTrimData(inData.getI_member());
        if (StringUtils.isEmpty(members)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0010);
            return reOutBase(Definition.RESULT_E);
        }
        String[] listMembers = members.split(Definition.ROLE_DELIMETER);
        if (listMembers.length == 0) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0010);
            return reOutBase(Definition.RESULT_E);
        }
        for (String mem : listMembers) {
            if (mem.length() > 16) {
                outMsg = bLanguageMsService.GetMessage(language, Definition.PAGE_801070,  Definition.MSG_801070_0011);
                return reOutBase(Definition.RESULT_E);
            }
        }
        //设值
        WGroup wGroup = new WGroup();
        wGroup.setiGroupId(roleId);//角色ID
        wGroup.setiGroupName(roleName);//角色名称
        wGroup.setiGroupComment(roleComment);//角色说明
        wGroup.setiGroupFlg(roleFlg);//有效标志
        wGroup.setiGroupMaster(roleMaster);//角色负责人
        wGroup.setiGroupMl(roleMl);//角色邮件地址
        //保存数据
        String resMsg = wGroupService.saveData(language, operate, wGroup, listMembers);
        if (!Definition.RESULT_S.equals(resMsg)) {
            outMsg = resMsg;
            return reOutBase(Definition.RESULT_E);
        }
        //正常返回
        outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0090);
        return reOutBase(Definition.RESULT_S);
    }

    @DeleteMapping("/delete")
    @Log(title = Definition.PAGE_801070, businessType = BusinessType.DELETE)
    public OutBase delete(@RequestBody WGroupDetailDelete inData) {
        //初期化
        init(inData.getI_language());
        //角色ID
        String roleIds = ClsBase.getTrimData(inData.getI_role_id());
        if (StringUtils.isEmpty(roleIds)) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0071);
            return reOutBase(Definition.RESULT_S);
        }
        //删除数据
        String resMsg = wGroupService.deleteData(language, roleIds);
        if (!Definition.RESULT_S.equals(resMsg)) {
            outMsg = resMsg;
            return reOutBase(Definition.RESULT_E);
        }
        //正常返回
        outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0080);
        return reOutBase(Definition.RESULT_S);
    }

    @PostMapping("/export")
    @Log(title = Definition.PAGE_801070, businessType = BusinessType.EXPORT)
    public void export(HttpServletResponse response, @RequestBody WGroupYilan inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        WGroupYilanIn selectIn = editSearch(inData);
        //取得数据
        List<WGroupYilanOut> list = wGroupService.selectWGroupUnLimt(selectIn);
        //gridInfo
        if (inData.getTable_info() == null) {
            outMsg = bLanguageMsService.GetMessage(language, Definition.COMMON_MSG,  Definition.MSG_COMMON_0100);
            errorResponse(response);
        }
        //取得页面名称
        String strDownloadFileName = iBFunctionMsService.GetFuncName(language, Definition.PAGE_801070);
        //excel名
        String excelName = ClsBase.getEccelName(strDownloadFileName, userId);
        //导出
        ExcelBase<WGroupYilanOut> util = new ExcelBase<>(WGroupYilanOut.class);
        util.exportExcel(response, inData.getTable_info(), list, Definition.EXPORT_INFO, excelName);
    }

    @PostMapping("/allRole")
    public WorkFlowTableInfo getAllRole(@RequestBody WGroupDetailAll inData) {
        //初期化
        init(inData.getI_language());
        //检索条件
        String roleIds = ClsBase.getTrimData(inData.getI_sch_field());
        //数据
        List<BLanguageMsOut> list = wGroupService.selectAllRole(roleIds);
        //返回
        WorkFlowTableInfo resData = new WorkFlowTableInfo();
        resData.setDataList(list);
        if (list != null) {
            resData.setDataCount(String.valueOf(list.size()));
            resData.setI_total(String.valueOf(list.size()));
        }
        resData.setLogin_user(userId);
        resData.setErrorDesc(outMsg);
        resData.setResultCode(Definition.RESULT_S);

        return resData;
    }

    /*
    检索条件
    */
    private WGroupYilanIn editSearch(WGroupYilan inData) {
        WGroupYilanIn wGroupYilanIn = new WGroupYilanIn();
        //语言
        wGroupYilanIn.setLanguage(language);
        //角色关联
        wGroupYilanIn.setiSchFieldRole(inData.getI_sch_field_role());
        //负责人关联
        wGroupYilanIn.setiSchFieldHead(inData.getI_sch_field_head());
        //申请状态
        wGroupYilanIn.setiEnable(ClsBase.getTrimData(inData.getI_enable()));
        return wGroupYilanIn;
    }

    /*
    返回 MapData
    */
    private MapData reMapData(String iCode){
        MapData resData = new MapData();
        resData.setErrorDesc(outMsg);
        resData.setLogin_user(userId);
        resData.setResultCode(iCode);
        return resData;
    }

    /*
   返回 MapData
   */
    private OutBase reOutBase(String iCode){
        OutBase resData = new OutBase();
        resData.setErrorDesc(outMsg);
        resData.setLogin_user(userId);
        resData.setResultCode(iCode);
        return resData;
    }

    /*
     *error
     */
    private void errorResponse(HttpServletResponse response){
        OutBase rspData = new OutBase();
        rspData.setLogin_user(userId);
        rspData.setErrorDesc(outMsg);
        rspData.setResultCode(Definition.RESULT_E);
        try {
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            PrintWriter pw = response.getWriter();
            pw.write(rspData.toString());
        } catch (IOException e) {
            log.error("返回异常{}", e.getMessage());
        }
    }

    /**
     *初期化
     */
    private void init(String inLanguage) {
        userId = ClsBase.getCurrentUser();
        outMsg = Definition.NULL_STR_1;
        if (StringUtils.isEmpty(inLanguage)) {
            language = ClsBase.defaultLanguage();
        } else {
            language = inLanguage;
        }
    }
}
