package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.BAttachM;
import com.ruoyi.project.workflow.domain.db.out.ApplyInfoUseForMailOut;
import com.ruoyi.project.workflow.domain.db.out.ApplyResumeOut;
import com.ruoyi.project.workflow.domain.db.out.TaskUserMailOut;
import com.ruoyi.project.workflow.domain.db.out.UserMailOut;
import com.ruoyi.project.workflow.domain.input.MailIn;
import com.ruoyi.project.workflow.mapper.SysUserDeptMapper;
import com.ruoyi.project.workflow.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.mail.internet.MimeMessage;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;


@Service("sendMailService")
public class SendMailServiceImpl implements SendMailService {
    @Autowired
    private SysUserDeptMapper sysUserDeptMapper;
    @Autowired
    private IBWordbookMsService iBWordbookMsService;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WFlowUserService wFlowUserService;
    @Autowired
    private WApplicationService wApplicationService;
    @Autowired
    private WFlowTaskService wFlowTaskService;
    @Autowired
    private BAttachMService bAttachMService;
    @Autowired
    private WReferenceUserService wReferenceUserService;

    /*
    审批否决的场合，通知申请者及承认完了的节点
     */
    @Override
    public String rejectApplyMail(String language, String applyId, Long applyTimes) {
        //用户
        String userId = ClsBase.getCurrentUser();
        //check
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //邮件入力编辑
        MailIn mailIn = new MailIn();
        mailIn.setLanguage(language);
        mailIn.setApplyId(applyId);
        mailIn.setApplyTimes(applyTimes);
        mailIn.setFormName(applyInfo.getiFormName());
        mailIn.setApplUserName(applUserName);
        mailIn.setApplStatusName(applyInfo.getiStatusName());
        mailIn.setMailOprType(Definition.MSG_MAIL_OPR_0003);
        mailIn.setMailTipType(Definition.MSG_MAIL_TIP_003);
        mailIn.setMailTipMsg(Definition.NULL_STR_1);
        mailIn.setMailToUserId(applyInfo.getiApplUser());
        mailIn.setToApplyUserFlg(true);
        mailIn.setFiles(null);
        //发送申请者邮件
        String resMsg = createMailAndSend(mailIn);
        if (!Definition.RESULT_S.equals(resMsg)) {
            return resMsg;
        }
        //承认者取得
        List<TaskUserMailOut>  listUsers = wFlowTaskService.selectRejectApplyMail(applyId, applyTimes, userId);
        //发送审批者邮件
        List<String> mailList = new ArrayList<>();
        if (listUsers != null && listUsers.size() > 0) {
            //邮件入力编辑
            MailIn mailReject = new MailIn();
            mailReject.setLanguage(language);
            mailReject.setApplyId(applyId);
            mailReject.setApplyTimes(applyTimes);
            mailReject.setFormName(applyInfo.getiFormName());
            mailReject.setApplUserName(applUserName);
            mailReject.setApplStatusName(applyInfo.getiStatusName());
            mailReject.setMailOprType(Definition.MSG_MAIL_OPR_0003);
            mailReject.setMailTipType(Definition.MSG_MAIL_TIP_003);
            mailReject.setMailTipMsg(Definition.NULL_STR_1);
            mailReject.setToApplyUserFlg(false);
            mailReject.setFiles(null);
            for (TaskUserMailOut outUser : listUsers) {
                if (!mailList.contains(outUser.getiEmail())) {
                    mailReject.setMailToUserId(outUser.getUserId());
                    //发送申请者邮件
                    resMsg = createMailAndSend(mailReject);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        return resMsg;
                    }
                    mailList.add(outUser.getiEmail());
                }
            }
        }
        //CC者的邮件取得
        List<TaskUserMailOut> ccList = wReferenceUserService.selectEmailByApplyId(applyId);
        if (ccList != null && ccList.size() > 0) {
            //邮件入力编辑
            MailIn mailCC = new MailIn();
            mailCC.setLanguage(language);
            mailCC.setApplyId(applyId);
            mailCC.setApplyTimes(applyTimes);
            mailCC.setFormName(applyInfo.getiFormName());
            mailCC.setApplUserName(applUserName);
            mailCC.setApplStatusName(applyInfo.getiStatusName());
            mailCC.setMailOprType(Definition.MSG_MAIL_OPR_0005);
            mailCC.setMailTipType(Definition.MSG_MAIL_TIP_003);
            mailCC.setMailTipMsg(Definition.NULL_STR_1);
            mailCC.setToApplyUserFlg(false);
            mailCC.setFiles(null);
            for (TaskUserMailOut ccUser : ccList) {
                if (!mailList.contains(ccUser.getiEmail())) {
                    mailCC.setMailToUserId(ccUser.getUserId());
                    //发送cc者邮件
                    resMsg = createMailAndSend(mailCC);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        return resMsg;
                    }
                    mailList.add(ccUser.getiEmail());
                }
            }
        }
        return Definition.RESULT_S;
    }

    /*
    申请通过的场合，通知申请者
    */
    @Override
    public String passApplyMail(String language, String applyId, Long applyTimes) {
        //check
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //邮件入力编辑
        MailIn mailin = new MailIn();
        mailin.setLanguage(language);
        mailin.setApplyId(applyId);
        mailin.setApplyTimes(applyTimes);
        mailin.setFormName(applyInfo.getiFormName());
        mailin.setApplUserName(applUserName);
        mailin.setApplStatusName(applyInfo.getiStatusName());
        mailin.setMailOprType(Definition.MSG_MAIL_OPR_0004);
        mailin.setMailTipType(Definition.MSG_MAIL_TIP_004);
        mailin.setMailTipMsg(Definition.NULL_STR_1);
        mailin.setMailToUserId(applyInfo.getiApplUser());
        mailin.setToApplyUserFlg(true);
        mailin.setFiles(null);
        //发送邮件
        String resMsg = createMailAndSend(mailin);
        if (!Definition.RESULT_S.equals(resMsg)) {
            return resMsg;
        }
        return Definition.RESULT_S;
    }

    /*
    通知
     */
    @Override
    public String noticeMail(String language, String applyId, String noticeUsers) {
        //申请回数
        Long applyTimes = wApplicationService.selectApplyMaxTimes(applyId);
        //申请信息
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //通知者的邮件取得
        if (noticeUsers != null ) {
            //邮件入力编辑
            MailIn mailIn = new MailIn();
            mailIn.setLanguage(language);
            mailIn.setApplyId(applyId);
            mailIn.setApplyTimes(applyTimes);
            mailIn.setFormName(applyInfo.getiFormName());
            mailIn.setApplUserName(applUserName);
            mailIn.setApplStatusName(applyInfo.getiStatusName());
            mailIn.setMailOprType(Definition.MSG_MAIL_OPR_0005);
            mailIn.setMailTipType(Definition.MSG_MAIL_TIP_005);
            mailIn.setMailTipMsg(Definition.NULL_STR_1);
            mailIn.setToApplyUserFlg(false);
            mailIn.setFiles(null);
            //通知者的邮件取得
            List<TaskUserMailOut> listUsers = sysUserDeptMapper.getEmailByIds(noticeUsers);
            if (listUsers != null && listUsers.size() > 0) {
                for (TaskUserMailOut userinfo : listUsers) {
                    //MAIL TO
                    mailIn.setMailToUserId(userinfo.getUserId());
                    //邮件发送
                    String resMsg = createMailAndSend(mailIn);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        return resMsg;
                    }
                }
            }
        }
        return Definition.RESULT_S;
    }

    /*
    推送
     */
    @Override
    public String pushMail(String language, String applyId, String pushUserIds,
                           String pushComment, String pushFiles) {
        //申请回数
        Long applyTimes = wApplicationService.selectApplyMaxTimes(applyId);
        //申请信息
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //判断格纳文件夹，不存在的场合退出
        String uploadURL = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_001, Definition.WORD_BOOK_NO_99_001_0210, Definition.FLG_Y);
        File fileTempPath = new File(uploadURL);
        if (!fileTempPath.exists() || !fileTempPath.isDirectory()) {
            return Definition.ERROR_MSG_PATH + uploadURL;
        }
        //邮件入力编辑
        MailIn mailIn = new MailIn();
        mailIn.setLanguage(language);
        mailIn.setApplyId(applyId);
        mailIn.setApplyTimes(applyTimes);
        mailIn.setFormName(applyInfo.getiFormName());
        mailIn.setApplUserName(applUserName);
        mailIn.setApplStatusName(applyInfo.getiStatusName());
        mailIn.setMailOprType(Definition.MSG_MAIL_OPR_0001);
        mailIn.setMailTipType(Definition.MSG_MAIL_TIP_001);
        mailIn.setMailTipMsg(pushComment);
        mailIn.setToApplyUserFlg(false);
        //附件
        if (StringUtils.isNotEmpty(pushFiles)) {
            for (String inFile : pushFiles.split(Definition.RADICAL_LINE)) {
                //文件名取得
                BAttachM bAttachM = bAttachMService.selectFileName(inFile);
                File file = new File(uploadURL + Definition.STR_XIEG + inFile);
                if (file.exists()) {
                    mailIn.getFiles().add(file);
                }
            }
        }
        //推送
        if (StringUtils.isNotEmpty(pushUserIds)) {
            for (String pushUserId : pushUserIds.split(Definition.ROLE_DELIMETER)) {
                mailIn.setMailToUserId(pushUserId);
                String resMsg = createMailAndSend(mailIn);
                if (!Definition.RESULT_S.equals(resMsg)) {
                    return resMsg;
                }
            }
        }
        return Definition.RESULT_S;
    }

    /*
    取消申请的场合，通知审批者及阅览者
     */
    @Override
    public String cancelApplyMail(String language, String applyId, Long applyTimes) {
        //申请信息
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //承认者取得(取消)
        List<TaskUserMailOut> listTaskUser = wFlowTaskService.selectCancelApplyMail(applyId, applyTimes);
        //收件人一览做成
        List<String> mailList = new ArrayList<>();
        if (listTaskUser != null && listTaskUser.size() > 0) {
            MailIn mailIn = new MailIn();
            mailIn.setLanguage(language);
            mailIn.setApplyId(applyId);
            mailIn.setApplyTimes(applyTimes);
            mailIn.setFormName(applyInfo.getiFormName());
            mailIn.setApplUserName(applUserName);
            mailIn.setApplStatusName(applyInfo.getiStatusName());
            mailIn.setMailOprType(Definition.MSG_MAIL_OPR_0002);
            mailIn.setMailTipType(Definition.MSG_MAIL_TIP_002);
            mailIn.setToApplyUserFlg(false);
            mailIn.setFiles(null);
            for (TaskUserMailOut userMail : listTaskUser) {
                mailIn.setMailToUserId(userMail.getUserId());
                //创建邮件
                String resMsg = createMailAndSend(mailIn);
                if (!Definition.RESULT_S.equals(resMsg)) {
                    return resMsg;
                }
                mailList.add(userMail.getiEmail());
            }
        }
        //CC者的邮件取得
        List<TaskUserMailOut> ccList = wReferenceUserService.selectEmailByApplyId(applyId);
        if (ccList != null && ccList.size() > 0) {
            //CC一览做成
            MailIn mailInCc = new MailIn();
            mailInCc.setLanguage(language);
            mailInCc.setApplyId(applyId);
            mailInCc.setApplyTimes(applyTimes);
            mailInCc.setFormName(applyInfo.getiFormName());
            mailInCc.setApplUserName(applUserName);
            mailInCc.setApplStatusName(applyInfo.getiStatusName());
            mailInCc.setMailOprType(Definition.MSG_MAIL_OPR_0005);
            mailInCc.setMailTipType(Definition.MSG_MAIL_TIP_002);
            mailInCc.setToApplyUserFlg(false);
            mailInCc.setFiles(null);
            for (TaskUserMailOut ccUser : ccList) {
                if (!mailList.contains(ccUser.getiEmail())) {
                    mailInCc.setMailToUserId(ccUser.getUserId());
                    //创建邮件
                    String resMsg = createMailAndSend(mailInCc);
                    if (!Definition.RESULT_S.equals(resMsg)) {
                        return resMsg;
                    }
                }
            }
        }
        return Definition.RESULT_S;
    }

    /*
    申请，审批的场合，通知审批者及阅览
     */
    @Override
    public String aduitApplyMail(String language, String applyId, Long applyTimes, String ccUsers) {
        String resMsg = Definition.RESULT_S;
        //申请信息
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取申请ID相关情报
        ApplyInfoUseForMailOut applyInfo = wApplicationService.selectApplyInfoUseForMail(language, applyId, applyTimes);
        //获取申请人名
        String applUserName = sysUserDeptMapper.selectNickNameById(applyInfo.getiApplUser());
        //承认者取得
        List<TaskUserMailOut> listTaskUser = wFlowTaskService.selectAduitApplyMail(applyId, applyTimes);
        //给审批人发邮件
        List<String> mailList = new ArrayList<>();
        if (listTaskUser != null && listTaskUser.size() > 0) {
            MailIn mailIn = new MailIn();
            mailIn.setLanguage(language);
            mailIn.setApplyId(applyId);
            mailIn.setApplyTimes(applyTimes);
            mailIn.setFormName(applyInfo.getiFormName());
            mailIn.setApplUserName(applUserName);
            mailIn.setApplStatusName(applyInfo.getiStatusName());
            mailIn.setMailOprType(Definition.MSG_MAIL_OPR_0001);
            mailIn.setMailTipType(Definition.MSG_MAIL_TIP_001);
            mailIn.setToApplyUserFlg(false);
            mailIn.setFiles(null);
            for (TaskUserMailOut userMail : listTaskUser) {
                mailIn.setMailToUserId(userMail.getUserId());
                //创建邮件
                resMsg = createMailAndSend(mailIn);
                if (!Definition.RESULT_S.equals(resMsg)) {
                    return resMsg;
                }
                mailList.add(userMail.getiEmail());
            }
        }
        //给阅览者发邮件
        if (StringUtils.isNotEmpty(ccUsers)) {
            List<TaskUserMailOut> listCcUser = sysUserDeptMapper.getEmailByIds(ccUsers);
            if (listCcUser != null && listCcUser.size() > 0) {
                MailIn mailInCc = new MailIn();
                mailInCc.setLanguage(language);
                mailInCc.setApplyId(applyId);
                mailInCc.setApplyTimes(applyTimes);
                mailInCc.setFormName(applyInfo.getiFormName());
                mailInCc.setApplUserName(applUserName);
                mailInCc.setApplStatusName(applyInfo.getiStatusName());
                mailInCc.setMailOprType(Definition.MSG_MAIL_OPR_0005);
                mailInCc.setMailTipType(Definition.MSG_MAIL_TIP_005);
                mailInCc.setToApplyUserFlg(false);
                mailInCc.setFiles(null);
                for (TaskUserMailOut ccMail : listTaskUser) {
                    if (!mailList.contains(ccMail.getiEmail())) {
                        mailInCc.setMailToUserId(ccMail.getUserId());
                        resMsg = createMailAndSend(mailInCc);
                        if (!Definition.RESULT_S.equals(resMsg)) {
                            return resMsg;
                        }
                    }
                }
            }
        }
        return resMsg;
    }

    /*
    创建邮件
     */
    private String createMailAndSend(MailIn mailIn) {
        //url
        String strURL =  iBWordbookMsService.selectWordbookNameByKeys(mailIn.getLanguage(),
                Definition.WORD_BOOK_TYPE_99_005, Definition.WORD_BOOK_NO_99_005_0020, Definition.FLG_Y);
        //申请信息
        if (StringUtils.isEmpty(mailIn.getApplyId()) || mailIn.getApplyTimes() == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //获取送信人信息
        UserMailOut userMailOut = sysUserDeptMapper.getEmailById(mailIn.getMailToUserId());
        if (userMailOut == null) {
            return Definition.ERROR_MSG_MAIL_NOTO;
        }
        //取得员工姓名
        String toUserName = sysUserDeptMapper.selectNickNameById(mailIn.getMailToUserId());
        //邮件主题做成
        String strSubject = bLanguageMsService.GetMessage(mailIn.getLanguage(),
                Definition.MSG_MAIL, mailIn.getMailOprType());
        strSubject = strSubject + ": " + mailIn.getFormName();
        //邮件正文做成
        String strBody = "<!DOCTYPE html>";
        strBody = strBody + "<html><body>";
        strBody = strBody + "<div style=\"width: 630px; margin: 0px auto;text-align: center;\">";
        strBody = strBody + "<table style=\"width: 600px;text-align: center;\">";
        //'logo
        strBody = strBody + "<tr><td style=\"padding: 8px 2px\">";
        strBody = strBody + "<img alt=\"IWORKFLOW LOGO\"src=\"cid:logo\">";
        strBody = strBody + "</td></tr>";
        //收件人部门
        strBody = strBody + "<tr><td style=\"padding: 2px\">";
        strBody = strBody + userMailOut.getCorporateName();
        strBody = strBody + "</td></tr>";
        //收件人姓名
        strBody = strBody + "<tr><td style=\"padding: 2px\">";
        strBody = strBody + bLanguageMsService.GetMessage(mailIn.getLanguage(), Definition.MSG_MAIL,
                Definition.MSG_MAIL_TAG).replace("{1}", toUserName);
        strBody = strBody + "</td></tr>";
        //【提示文言】
        strBody = strBody + "<tr><td style=\"padding: 8px 4px\">";
        if (StringUtils.isNotEmpty(mailIn.getMailTipMsg())) {
            strBody = strBody + mailIn.getMailTipMsg();
        } else {
            strBody = strBody + bLanguageMsService.GetMessage(mailIn.getLanguage(),
                    Definition.MSG_MAIL, mailIn.getMailTipType());
        }
        strBody = strBody + "</td></tr>";
        strBody = strBody + "</table>";
        //【申请内容】大标题
        strBody = strBody + "<table style=\"width: 600px;text-align: left;\">";
        strBody = strBody + "<tr><td>";
        strBody = strBody + "<h3>" + bLanguageMsService.GetMessage(mailIn.getLanguage(),
                Definition.MSG_MAIL, Definition.MSG_MAIL_APPLY_CONTENT) + "</h3>";
        strBody = strBody + "</td></tr>";
        strBody = strBody + "</table>";
        //【申请内容】正文
        strBody = strBody + "<table style=\"width: 600px;margin-left: 30px;text-align: left;\">";
        //【申请内容】申请者
        strBody = strBody + "<tr>";
        strBody = strBody + "<td style=\"width: 150px;padding: 6px 2px\">" +
                bLanguageMsService.GetMessage(mailIn.getLanguage(), Definition.MSG_MAIL,
                        Definition.MSG_MAIL_APPLY_USER) + "</td>";
        strBody = strBody + "<td style=\" padding: 6px 2px\">" + mailIn.getApplUserName() + "</td>";
        strBody = strBody + "</tr>";
        //【申请内容】title
        strBody = strBody + "<tr>";
        strBody = strBody + "<td style=\"width: 150px;padding: 6px 2px\">" +
                bLanguageMsService.GetMessage(mailIn.getLanguage(), Definition.MSG_MAIL,
                        Definition.MSG_MAIL_APPLY_TITLE) + "</td>";
        strBody = strBody + "<td style=\" padding: 6px 2px\">" + mailIn.getFormName() + "</td>";
        strBody = strBody + "</tr>";
        //【申请内容】申请No
        strBody = strBody + "<tr>";
        strBody = strBody + "<td style=\"width: 150px;padding: 6px 2px\">" +
                bLanguageMsService.GetMessage(mailIn.getLanguage(), Definition.MSG_MAIL,
                        Definition.MSG_MAIL_APPLY_ID) + "</td>";
        strBody = strBody + "<td style=\" padding: 6px 2px\">" + mailIn.getApplyId() + "</td>";
        strBody = strBody + "</tr>";
        //【申请内容】状态
        strBody = strBody + "<tr>";
        strBody = strBody + "<td style=\"width: 150px;padding: 6px 2px\">" +
                bLanguageMsService.GetMessage(mailIn.getLanguage(), Definition.MSG_MAIL,
                        Definition.MSG_MAIL_APPLY_STATUS) + "</td>";
        strBody = strBody + "<td style=\" padding: 6px 2px\">" + mailIn.getApplStatusName() + "</td>";
        strBody = strBody + "</tr>";
        strBody = strBody + "</table>";
        //【审批履历】
        if (mailIn.getToApplyUserFlg()
                && Definition.MSG_MAIL_OPR_0003.equals(mailIn.getMailOprType())) {
            //获取审批履历
            List<ApplyResumeOut> resumes = wFlowUserService.selectApplyResume(mailIn.getLanguage(),
                    mailIn.getApplyId(), mailIn.getApplyTimes());
            if (resumes != null && resumes.size() > 0) {
                //【审批履历】大标题
                strBody = strBody + "<table style=\"width: 600px;text-align: left;\">";
                strBody = strBody + "<tr><td>";
                strBody = strBody + "<h3>" + bLanguageMsService.GetMessage(mailIn.getLanguage(),
                        Definition.MSG_MAIL, Definition.MSG_MAIL_ADUIT_HIS) + "</h3>";
                strBody = strBody + "</td></tr>";
                strBody = strBody + "</table>";
                //【审批履历】正文
                strBody = strBody + "<table style=\"width: 600px;margin-left: 30px;text-align: left;\">";
                for (ApplyResumeOut resume : resumes) {
                    strBody = strBody + "<tr>";
                    strBody = strBody + "<td style=\"width: 100px; padding: 6px 2px\">" +
                            resume.getiUpdDate() + "</td>";
                    String namedt = sysUserDeptMapper.selectNickNameById(resume.getiAduitUserId());
                    if (StringUtils.isNotEmpty(namedt)) {
                        strBody = strBody + "<td style=\"width: 100px; padding: 6px 2px\">" + namedt + "</td>";
                    }
                    strBody = strBody + "<td style=\"padding: 6px 2px\">" + resume.getAduitComment() + "</td>";
                    strBody = strBody + "</tr>";
                }
                strBody = strBody + "</table>";
            }
        }
        strBody = strBody + "</br>";
        strBody = strBody + "<table style=\"width: 600px;text-align: center; \">";
        //【提示文言2】
        strBody = strBody + "<tr><td style=\"padding: 8px 2px\">";
        strBody = strBody + bLanguageMsService.GetMessage(mailIn.getLanguage(),
                Definition.MSG_MAIL, Definition.MSG_MAIL_TIP2);
        strBody = strBody + "</td></tr>";
        //URL
        strBody = strBody + "<tr><td style=\"padding: 4px 2px\">";
        strBody = strBody + "<a href=\"" + strURL + "\">" + strURL + "</a>";
        strBody = strBody + "</td></tr>";
        //【提示文言3】
        strBody = strBody + "<tr><td style=\"text-align: left;padding: 8px 2px\">";
        strBody = strBody + bLanguageMsService.GetMessage(mailIn.getLanguage(),
                Definition.MSG_MAIL, Definition.MSG_MAIL_COMMENT);
        strBody = strBody + "</td></tr>";
        strBody = strBody + "</table>";
        strBody = strBody + "</div>";
        strBody = strBody + "</body></html>";
        //发送
        String reMsg = SendMail(mailIn.getLanguage(), strSubject, strBody, userMailOut.getEmail(),
                Definition.NULL_STR_1, mailIn.getFiles());
        return reMsg;
    }

    /*
    发送
     */
    private String SendMail(String language, String strMailSubject, String strBody,
                           String strAddressTo, String strAddressCC, List<File> files) {
        //取得邮件参数信息
        String mailSmtp = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_003, Definition.WORD_BOOK_NO_99_003_0010, Definition.FLG_Y);
        //送信邮箱
        String mailAddressFrom = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_003, Definition.WORD_BOOK_NO_99_003_0020, Definition.FLG_Y);
        //送信人
        String mailUserFrom = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_003, Definition.WORD_BOOK_NO_99_003_0030, Definition.FLG_Y);
        //授权码
        String mailPswFrom = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_003, Definition.WORD_BOOK_NO_99_003_0040, Definition.FLG_Y);
        //Logo图标文件夹
        String logoURL = iBWordbookMsService.selectWordbookNameByKeys(language,
                Definition.WORD_BOOK_TYPE_99_001, Definition.WORD_BOOK_NO_99_001_0100, Definition.FLG_Y);
        File folder = new File(logoURL);
        if (!folder.exists() || !folder.isDirectory())  {
            return Definition.ERROR_MSG_LOGO_1 + logoURL;
        }
        //Logo图标文件
        String logoPath = logoURL + Definition.STR_XIEG + Definition.LOGO_NAME;
        File logoFile = new File(logoPath);
        if (!logoFile.exists()){
            return Definition.ERROR_MSG_LOGO_2 + logoPath;
        }
        //发送
        return ClsBase.sendMail(mailSmtp, mailAddressFrom, mailPswFrom, mailUserFrom, strMailSubject, strBody,
                strAddressTo, strAddressCC, logoFile, files);
    }
}
