package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.system.domain.SysDept;
import com.ruoyi.project.workflow.domain.db.out.BLanguageMsOut;
import com.ruoyi.project.workflow.domain.db.out.UserDeptListOut;
import com.ruoyi.project.workflow.domain.output.DeptTreeSelect;
import com.ruoyi.project.workflow.mapper.SysUserDeptMapper;
import com.ruoyi.project.workflow.service.SysUserDeptService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service("sysUserDeptService")
public class SysUserDeptServiceImpl implements SysUserDeptService {

    @Autowired
    private SysUserDeptMapper sysUserDeptMapper;

    @Override
    public List<UserDeptListOut> selectUserList(String str) {
        return sysUserDeptMapper.selectUserList(str);
    }

    @Override
    public UserDeptListOut getUserById(String userId){
        return sysUserDeptMapper.getUserById(userId);
    }

    @Override
    public List<DeptTreeSelect> getDeptTreeWork() {
        List<DeptTreeSelect> trees = new ArrayList<>();
        List<SysDept> allDept = getAllDept();
        if (allDept != null && allDept.size() > 0) {
            DeptTreeSelect newTree = newDeptTree(allDept.get(0));
            newTree.setChildren(getAllChildrenDept(allDept, allDept.get(0).getDeptId()));
            trees.add(newTree);
        }
        return trees;
    }

    @Override
    public List<BLanguageMsOut> selectDeptNamesByDeptIds(String deptIds) {
        return sysUserDeptMapper.selectDeptNamesByDeptIds(deptIds);
    }

    /*
    获取上级公司
    */
    @Override
    public List<BLanguageMsOut> selectDeptNamesByParentId(long parentId) {
        return sysUserDeptMapper.selectDeptNamesByParentId(parentId);
    }

    /**
     *获取所有部门
     */
    private List<SysDept> getAllDept() {
        SysDept sysDept = new SysDept();
        return sysUserDeptMapper.selectAllDept(sysDept);
    }

    /**
     * 获取所有子部门
     */
    private List<DeptTreeSelect> getAllChildrenDept(List<SysDept> allDept, Long deptId) {
        List<DeptTreeSelect> childrenTrees = new ArrayList<>();
        if (allDept != null && allDept.size() > 0) {
            for (SysDept sDept : allDept) {
                if (sDept.getParentId().equals(deptId)) {
                    DeptTreeSelect dt = newDeptTree(sDept);
                    dt.setChildren(getAllChildrenDept(allDept, sDept.getDeptId()));
                    childrenTrees.add(dt);
                } else {
                    if (childrenTrees.size() > 0) {
                        break;
                    }
                }
            }
        }
        if (childrenTrees.size() == 0) {
            return null;
        } else {
            return childrenTrees;
        }
    }

    /**
     * 设值
     */
    private DeptTreeSelect newDeptTree(SysDept inDept) {
        DeptTreeSelect oneTree = new DeptTreeSelect();
        oneTree.setLabel(inDept.getDeptName());
        oneTree.setValue(String.valueOf(inDept.getDeptId()));
        return oneTree;
    }
}
