package com.ruoyi.project.workflow.service.impl;


import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WApplication;
import com.ruoyi.project.workflow.domain.WFlowTask;
import com.ruoyi.project.workflow.domain.WFlowUser;
import com.ruoyi.project.workflow.domain.WFormLayout;
import com.ruoyi.project.workflow.domain.db.out.CurrentNodeOut;
import com.ruoyi.project.workflow.domain.db.out.FlowTaskUserOut;
import com.ruoyi.project.workflow.domain.db.out.TaskUserMailOut;
import com.ruoyi.project.workflow.domain.output.FlowTask;
import com.ruoyi.project.workflow.domain.output.NodeConfig;
import com.ruoyi.project.workflow.mapper.*;
import com.ruoyi.project.workflow.service.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.util.*;

@Service("wFlowTaskService")
public class WFlowTaskServiceImpl implements WFlowTaskService {
    private static final Logger log = LoggerFactory.getLogger(ExcelUtil.class);

    @Autowired
    private WFlowTaskMapper wFlowTaskMapper;
    @Autowired
    private WFlowUserMapper wFlowUserMapper;
    @Autowired
    private WApplicationMapper wApplicationMapper;
    @Autowired
    private SysUserDeptMapper sysUserDeptMapper;
    @Autowired
    private WFormLayoutService wFormLayoutService;
    @Autowired
    private WApplicationService wApplicationService;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WGroupService wGroupService;
    @Autowired
    private SysPostPartService sysPostPartService;

    @Override
    public List<TaskUserMailOut> selectAduitApplyMail(String applyId, Long applyTimes){
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        return wFlowTaskMapper.selectAduitApplyMail(wFlowTask);
    }

    @Override
    public List<TaskUserMailOut> selectCancelApplyMail(String applyId, Long applyTimes) {
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        return wFlowTaskMapper.selectCancelApplyMail(wFlowTask);
    }

    @Override
    public List<TaskUserMailOut> selectRejectApplyMail(String applyId, Long applyTimes, String userId) {
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        wFlowTask.setSearchValue(userId);
        return wFlowTaskMapper.selectRejectApplyMail(wFlowTask);
    }

    /*
    生成TASK
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String generateTasks(String language, String formId, String applyId, Long applyTimes,
                                String flowData, Map<String, Object> params) {
        if (StringUtils.isEmpty(applyId) || applyTimes == 0) {
            return Definition.ERROR_MSG_APPLY;
        }
        //入力审批流程
        NodeConfig ncf = JSONObject.parseObject(flowData,NodeConfig.class);
        //根据条件生成审批流程
        List<NodeConfig> auditors = extractAuditors(formId, applyId, applyTimes, params, ncf);
        //时间
        Date nowDate = new Date();
        //userId
        String userid = ClsBase.getCurrentUser();
        //路径中没有审批人则直接通过
        if (auditors == null || auditors.size() == 0) {
            try {
                //申请管理记录状态更新
                WApplication wApplication = new WApplication();
                wApplication.setiApplyId(applyId);
                wApplication.setiSubmitTimes(applyTimes);
                wApplication.setiStatus(Definition.STR_PASS);
                wApplication.setiSubmitDate(nowDate);
                wApplication.setiUpdId(userid);
                wApplication.setiUpdDate(nowDate);
                wApplicationMapper.updateWApplication(wApplication);
            } catch (Exception e) {
                //回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return e.getMessage();
            }
            return Definition.RESULT_S;
        }
        //设置前后置关系及状态
        NodeConfig prevNode = null;
        for (int i = 0; i < auditors.size(); i++) {
            if (prevNode != null) {
                auditors.get(i).setPrevTasks(prevNode.getNodeId());
                prevNode.setNextTasks(auditors.get(i).getNodeId());
                auditors.get(i).setStatus(Definition.INT_INACTIVE);
            } else {
                auditors.get(i).setStatus(Definition.INT_AUDITING);
            }
            prevNode = auditors.get(i);
        }
        //以审批人为单位生成任务
        List<WFlowTask> flowTaskList = new ArrayList<>();
        for (NodeConfig node : auditors) {
            int subId = 0;
            //节点的审批对象者为空的场合
            if (StringUtils.isEmpty(node.getOptionValue())) {
                WFlowTask flowTask = new WFlowTask();
                //申請ID
                flowTask.setiApplyId(applyId);
                //提出回数
                flowTask.setiSubmitTimes(applyTimes);
                //节点ID
                flowTask.setiNodeId(node.getNodeId());
                //节点SUBID
                flowTask.setiNodeSubId(String.valueOf(subId + 1));
                //节点名称
                flowTask.setiNodeName(node.getNodeName());
                //节点对象者类型
                flowTask.setiNodeObjType(String.valueOf(node.getOptionType()));
                //节点对象者ID
                flowTask.setiNodeObjId(Definition.NULL_STR_1);
                //节点属性
                flowTask.setiNodeProperty(String.valueOf(node.getNodeProperty()));
                //节点关系
                flowTask.setiNodeRelationship(String.valueOf(node.getSignType()));
                //先行节点ID
                flowTask.setiPreNodeId(node.getPrevTasks());
                //后续节点ID
                flowTask.setiNexNodeId(node.getNextTasks());
                //节点状态
                flowTask.setiNodeStatus(String.valueOf(node.getStatus()));
                //追加
                flowTaskList.add(flowTask);
            } else {
                for (String item : node.getOptionValue().split(Definition.ROLE_DELIMETER)) {
                    WFlowTask flowTask = new WFlowTask();
                    //申請ID
                    flowTask.setiApplyId(applyId);
                    //提出回数
                    flowTask.setiSubmitTimes(applyTimes);
                    //节点ID
                    flowTask.setiNodeId(node.getNodeId());
                    //节点SUBID
                    subId = subId + 1;
                    flowTask.setiNodeSubId(String.valueOf(subId));
                    //节点名称
                    flowTask.setiNodeName(node.getNodeName());
                    //节点对象者类型
                    flowTask.setiNodeObjType(String.valueOf(node.getOptionType()));
                    //节点对象者ID
                    flowTask.setiNodeObjId(item);
                    //节点属性
                    flowTask.setiNodeProperty(String.valueOf(node.getNodeProperty()));
                    //节点关系
                    flowTask.setiNodeRelationship(String.valueOf(node.getSignType()));
                    //先行节点ID
                    flowTask.setiPreNodeId(node.getPrevTasks());
                    //后续节点ID
                    flowTask.setiNexNodeId(node.getNextTasks());
                    //节点状态
                    flowTask.setiNodeStatus(String.valueOf(node.getStatus()));
                    //追加
                    flowTaskList.add(flowTask);
                }
            }
        }
        try {
            //申请管理记录状态更新
            WApplication wApplication = new WApplication();
            wApplication.setiApplyId(applyId);
            wApplication.setiSubmitTimes(applyTimes);
            wApplication.setiStatus(Definition.STR_AUDITING);
            wApplication.setiSubmitDate(nowDate);
            wApplication.setiUpdId(userid);
            wApplication.setiUpdDate(nowDate);
            wApplicationMapper.updateWApplication(wApplication);
            //根据TASK生成TASK承认者一览
            if (flowTaskList != null && flowTaskList.size() > 0) {
                for (WFlowTask ft : flowTaskList) {
                    //FLOW TASK管理记录追加
                    ft.setiEntryId(userid);
                    ft.setiEntryDate(nowDate);
                    ft.setiUpdId(userid);
                    ft.setiUpdDate(nowDate);
                    wFlowTaskMapper.insertWFlowTask(ft);
                    //TASK承认者记录追加
                    switch (ft.getiNodeObjType()) {
                        case Definition.STR_PERSON:
                            //个人承认的场合
                            WFlowUser pFlowUser = setWFlowUser(ft, ft.getiNodeObjId(), userid, nowDate);
                            wFlowUserMapper.insertWFlowUser(pFlowUser);
                            break;
                        case Definition.STR_DEPT:
                            //责任者的场合
                            if (Definition.STR_MASTER.equals(ft.getiNodeProperty())){
                                String leader = sysUserDeptMapper.selectLeaderByDeptId(ft.getiNodeObjId());
                                if (leader != null) {
                                    WFlowUser leaFlowUser = setWFlowUser(ft, leader, userid, nowDate);
                                    wFlowUserMapper.insertWFlowUser(leaFlowUser);
                                }
                            } else {
                                //任一的场合
                                List<String> deptUsers = sysUserDeptMapper.selectUserIdByDeptId(ft.getiNodeObjId());
                                if (deptUsers != null && deptUsers.size() > 0) {
                                    for (String deptUser : deptUsers) {
                                        WFlowUser dFlowUser = setWFlowUser(ft, deptUser, userid, nowDate);
                                        wFlowUserMapper.insertWFlowUser(dFlowUser);
                                    }
                                }
                            }
                            break;
                        case Definition.STR_ROLE:
                            //责任者的场合
                            if (Definition.STR_MASTER.equals(ft.getiNodeProperty())){
                                String master = wGroupService.selectGroupMasterByKey(ft.getiNodeObjId());
                                WFlowUser mFlowUser = setWFlowUser(ft, master, userid, nowDate);
                                wFlowUserMapper.insertWFlowUser(mFlowUser);
                            } else {
                                List<String> groupUsers =
                                        wGroupService.selectGroupUsersByKey(ft.getiNodeObjId());
                                if (groupUsers != null && groupUsers.size() > 0) {
                                    for (String groupUser : groupUsers) {
                                        WFlowUser gFlowUser = setWFlowUser(ft, groupUser, userid, nowDate);
                                        wFlowUserMapper.insertWFlowUser(gFlowUser);
                                    }
                                }
                            }
                            break;
                        case Definition.STR_POST:
                            //岗位的场合
                            List<String> postUsers = sysPostPartService.selectUserIdsByPostId(ft.getiNodeObjId());
                            if (postUsers != null && postUsers.size() > 0) {
                                for (String postUser : postUsers) {
                                    WFlowUser postFlowUser = setWFlowUser(ft, postUser, userid, nowDate);
                                    wFlowUserMapper.insertWFlowUser(postFlowUser);
                                }
                            }
                            break;
                        case Definition.STR_DEPT_POST:
                            //部门+岗位
                            List<String> deptPostUsers = sysPostPartService.selectUserByDeptPost(ft.getiNodeObjId());
                            if (deptPostUsers != null && deptPostUsers.size() > 0) {
                                for (String postDeptUser : deptPostUsers) {
                                    WFlowUser postFlowUser = setWFlowUser(ft, postDeptUser, userid, nowDate);
                                    wFlowUserMapper.insertWFlowUser(postFlowUser);
                                }
                            }
                            break;
                        case Definition.STR_PART:
                            //角色
                            List<String> partUsers = sysPostPartService.selectUserByRoleId(ft.getiNodeObjId());
                            if (partUsers != null && partUsers.size() > 0) {
                                for (String partUser : partUsers) {
                                    WFlowUser partFlowUser = setWFlowUser(ft, partUser, userid, nowDate);
                                    wFlowUserMapper.insertWFlowUser(partFlowUser);
                                }
                            }
                            break;
                    }
                }
            }
            //给下一个审批节点的审批者设定代理人
            WFlowUser updateFlowUser = new WFlowUser();
            updateFlowUser.setiApplyId(applyId);
            updateFlowUser.setiSubmitTimes(applyTimes);
            updateFlowUser.setiUpdId(userid);
            updateFlowUser.setiUpdDate(nowDate);
            updateFlowUser.getParams().put(Definition.STR_FORMID, formId);
            wFlowUserMapper.updateNextProxyUser(updateFlowUser);
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0121);
        }
        return Definition.RESULT_S;
    }

    /*
    WFlowUser设定
     */
    private WFlowUser setWFlowUser(WFlowTask wFlowTask, String applyUserId,
                                   String sysUserId, Date nowDate) {
        WFlowUser wFlowUser = new WFlowUser();
        wFlowUser.setiApplyId(wFlowTask.getiApplyId());
        wFlowUser.setiSubmitTimes(wFlowTask.getiSubmitTimes());
        wFlowUser.setiNodeId(wFlowTask.getiNodeId());
        wFlowUser.setiNodeSubId(wFlowTask.getiNodeSubId());
        wFlowUser.setiUserId(applyUserId);
        wFlowUser.setiProxyUserId(null);
        wFlowUser.setiUserOperation(Definition.FLG_ZERO);
        wFlowUser.setiResult(null);
        wFlowUser.setiComment(null);
        wFlowUser.setiEntryId(sysUserId);
        wFlowUser.setiEntryDate(nowDate);
        wFlowUser.setiUpdId(sysUserId);
        wFlowUser.setiUpdDate(nowDate);
        return wFlowUser;
    }

    /*
    根据条件生成审批流程
     */
    private List<NodeConfig> extractAuditors(String formId, String applyId, Long applyTimes,
                                             Map<String, Object> params, NodeConfig ncf) {
        //next节点
        NodeConfig nextNode = new NodeConfig();
        //取得第一个节点（发起人外第一个节点）
        if (ncf == null) {
            nextNode = null;
        } else {
             if (ncf.getChildNode() == null) {
                 nextNode = null;
             } else {
                 BeanUtils.copyProperties(ncf.getChildNode(), nextNode);
             }
        }
        //表单ID
        if (StringUtils.isEmpty(formId)) {
            WApplication wApplication = wApplicationService.selectApplyInfoByApplyIds(applyId, applyTimes);
            if (wApplication != null) {
                formId = String.valueOf(wApplication.getiFormId());
            }
        }
        //取得表单结构
        List<WFormLayout> listWFormLayout = wFormLayoutService.selectStructureByFormId(formId);
        //解析
        Map<String, Object> appObj = new HashMap<>();
        if (listWFormLayout != null && listWFormLayout.size() > 0) {
            for (WFormLayout item : listWFormLayout) {
                Object strValue =  params.get(item.getiFormDataCol());
                if (strValue != null) {
                    appObj.put(item.getiFormDataCol(), strValue);
                } else {
                    appObj.put(item.getiFormDataCol(), Definition.NULL_STR_1);
                }
            }
        }
        String strAppObj = JSONObject.toJSONString(appObj);
        //取得申请人信息
        Map<String, Object> proObj = new HashMap<>();
        proObj.put(Definition.I_SYS_NO, ClsBase.getCurrentUser());
        proObj.put(Definition.I_DEPT_NO, ClsBase.getUserDeptId());
        String strProObj = JSONObject.toJSONString(proObj);
        //next节点
        List<NodeConfig> auditNodes = new ArrayList<>();
        while (nextNode != null) {
            switch (nextNode.getType()) {
                case Definition.INT_PROMOTER:
                    break;
                case Definition.INT_AUDITOR:
                    auditNodes.add(setNodeConfig(nextNode));
                    NodeConfig child = setNodeConfig(nextNode.getChildNode());
                    NodeConfig parenta = setNodeConfig(nextNode.getParent());
                    if (child != null) {
                        child.setParent(setNodeConfig(parenta));
                        nextNode = setNodeConfig(child);
                    } else if (parenta != null) {
                        //如果是子流程结束需要回到父流程继续
                        nextNode = setNodeConfig(parenta.getChildNode());
                        //由于只有路由中的条件节点才存在父节点，所以，需要为路由中的审批节点的父节点设上路由节点作为父节点
                        if (nextNode != null) {
                            nextNode.setParent(setNodeConfig(parenta.getParent()));
                        }
                    } else {
                        nextNode = null;
                    }
                    break;
                case Definition.INT_CC:
                    //CC节点的场合，获取下一个子节点i
                    nextNode = setNodeConfig(nextNode.getChildNode());
                    break;
                case Definition.INT_CONDITION:
                    //获取当前节点的父节点
                    NodeConfig parentb = setNodeConfig(nextNode.getParent());
                    //当前节点没有子节点的场合
                    if ((nextNode.getChildNode() == null) &&  parentb != null) {
                        //将当前节点的父节点的子节点取出
                        nextNode = setNodeConfig(parentb.getChildNode());
                        //将当前节点的父节点设为下一节点的父节点
                        if (nextNode != null) {
                            nextNode.setParent(setNodeConfig(parentb.getParent()));
                        }
                    } else {
                        //将当前节点的父节点设为下一节点的父节点
                        nextNode = setNodeConfig(nextNode.getChildNode());
                        nextNode.setParent(setNodeConfig(parentb));
                    }
                    break;
                case Definition.INT_ROUTER:
                    //根据条件选择进入哪条子流程
                    if (nextNode.getConditionNodes() != null && nextNode.getConditionNodes().size() > 0) {
                        for (NodeConfig condition : nextNode.getConditionNodes()) {
                            String expression = condition.getExpression();
                            int selectKen = 0;
                            if (StringUtils.isEmpty(expression)) {
                                expression = Definition.STR_TRUE;
                                selectKen = 1;
                            }
                            //运行js
                            try {
                                ScriptEngine engine = new ScriptEngineManager().getEngineByName(Definition.JAVA_SCRIPT);
                                engine.eval(Definition.SCRIPT_DATA + strAppObj);
                                engine.eval(Definition.SCRIPT_PROMOTER + strProObj);
                                Boolean boolResult  = Boolean.valueOf(String.valueOf(engine.eval(expression)));
                                if (boolResult) {
                                    NodeConfig routeStart = setNodeConfig(condition);
                                    routeStart.setParent(setNodeConfig(nextNode));
                                    nextNode = setNodeConfig(routeStart);
                                    break;
                                }
                            } catch (Exception e) {
                                log.error(Definition.JAVA_SCRIPT + "异常{}", e.getMessage());
                            }
                        }
                    }
                    break;
            }
        }
        return auditNodes;
    }

    /*
    取得审批流程
     */
    @Override
    public Map<String, Object> getFlowTaskByApplyId(String language, String applyId) {
        Map<String, Object> dtHash1 = new HashMap<>();
        List<Map<String, Object>> list1 = new ArrayList<>();
        //取得最大申请回数（最新状态）
        Long submitTimes = wApplicationService.selectApplyMaxTimes(applyId);
        if (submitTimes == null || submitTimes == 0) {
            return dtHash1;
        }
        //提交回数
        dtHash1.put(Definition.STR_TIMES, submitTimes);
        //取消flg
        Boolean cancelFlg = false;
        //删除flg
        Boolean deleteFlg = false;
        for (int i = 0; i < submitTimes; i++) {
            Long itime = Long.valueOf(i + 1);
            //Node个数
            Long dtNum = selectCountNodenum(applyId, itime);
            //Node拼接, 配置申请人节点, 取得当前回数的申请详细
            WApplication dtApply = wApplicationService.selectApplyInfoByApplyIds(applyId, itime);
            if (dtApply != null) {
                Map<String, Object> dtHash2 = new HashMap<>();
                Map<String, Object> dtHash3 = new HashMap<>();
                Map<String, Object> dtHash4 = new HashMap<>();
                List<Map<String, Object>> list2 = new ArrayList<>();
                List<Map<String, Object>> list3 = new ArrayList<>();
                //申请人详细
                String applyUserName =  sysUserDeptMapper.selectNickNameById(dtApply.getiApplUser());
                dtHash4.put(Definition.STR_I_USER_ID, dtApply.getiApplUser());
                dtHash4.put(Definition.STR_I_USER_NAME, applyUserName);
                //提交时间
                dtHash4.put(Definition.STR_I_UPD_DATE, dtApply.getiSubmitDate());
                //提交状态
                if (dtApply.getiSubmitDate() != null) {
                    dtHash4.put(Definition.STR_I_RESULT, Definition.FLG_2);
                } else {
                    dtHash4.put(Definition.STR_I_RESULT, Definition.NULL_STR_1);
                }
                list3.add(dtHash4);
                //已提交的场合
                if (dtApply.getiSubmitDate() != null) {
                    if (dtApply.getiCancelDate() != null) {
                        //申请人取消flg设定
                        cancelFlg = true;
                    }
                    if (dtApply.getiDeleteDate() != null) {
                        //申请人删除flg设定
                        deleteFlg = true;
                    }
                    //初回
                    if (Definition.FST_VERSION.equals(dtApply.getiSubmitTimes())) {
                        //申请人首次提交
                        dtHash2.put(Definition.STR_I_NODE_NAME,
                                bLanguageMsService.GetMessage(language, Definition.PAGE_401020,
                                        Definition.MSG_401020_0019));
                    } else {
                        //申请人再提交
                        dtHash2.put(Definition.STR_I_NODE_NAME,
                                bLanguageMsService.GetMessage(language, Definition.PAGE_401020,
                                        Definition.MSG_401020_0020));
                    }
                    dtHash3.put(Definition.STR_USER, list3);
                    list2.add(dtHash3);
                    dtHash2.put(Definition.STR_SUBNODE, list2);
                    list1.add(dtHash2);
                } else {
                    //未提交
                    //申请人草稿
                    dtHash2.put(Definition.STR_I_NODE_NAME,
                            bLanguageMsService.GetMessage(language, Definition.PAGE_401020,
                                    Definition.MSG_401020_0023));
                    //已删除
                    if (dtApply.getiDeleteDate() != null) {
                        //申请人删除flg设定
                        deleteFlg = true;
                    } else {
                        dtHash3.put(Definition.STR_USER, list3);
                        list2.add(dtHash3);
                        dtHash2.put(Definition.STR_SUBNODE, list2);
                        list1.add(dtHash2);
                    }
                }
            }
            //配置审批人节点
            String nexNodeid = Definition.NULL_STR_1;
            for (int j = 0; j < dtNum; j++) {
                Map<String, Object> dtHash2 = new HashMap<>();
                //取得流程详情
                WFlowTask dtNode = selectFlowTaskDetail(applyId, itime, nexNodeid, Definition.FLG_1);
                if (dtNode != null) {
                    nexNodeid = dtNode.getiNexNodeId();
                    //SubNode个数
                    Long subnum = selectCountSubnum(applyId, itime, dtNode.getiNodeId());
                    //未激活节点除外
                    if (itime == submitTimes || !Definition.FLG_ZERO.equals(dtNode.getiNodeStatus())) {
                        dtHash2.put(Definition.STR_SUBNUM, subnum);
                        dtHash2.put(Definition.STR_I_NODE_ID, dtNode.getiNodeId());
                        dtHash2.put(Definition.STR_I_NODE_STATUS, dtNode.getiNodeStatus());
                        dtHash2.put(Definition.STR_I_NODE_NAME, dtNode.getiNodeName());
                        dtHash2.put(Definition.STR_I_SUBMIT_TIMES, dtNode.getiSubmitTimes());
                        //SubNode拼接
                        List<Map<String, Object>> list2 = new ArrayList<>();
                        for (int k = 0; k < subnum; k++) {
                            Map<String, Object> dtHash3 = new HashMap<>();
                            Long isub = Long.valueOf(k + 1);
                            String nodeSubId = isub.toString();
                            WFlowTask dtSub = selectFlowTaskDetail(applyId, itime, dtNode.getiNodeId(), nodeSubId);
                            if (dtSub != null) {
                                //取得子节点描述
                                String strDesc = getDescription(dtSub.getiNodeObjType(), dtSub.getiNodeObjId(),
                                        dtSub.getiNodeProperty(), dtSub.getiNodeRelationship(), isub, language);
                                dtHash3.put(Definition.STR_DESCRIPTION, strDesc);
                            }
                            //user拼接
                            List<FlowTaskUserOut> dtUsers = selectFlowTaskUser(applyId, itime,
                                    dtNode.getiNodeId(), nodeSubId);
                            List<Map<String, Object>> list3 = new ArrayList<>();
                            if (dtUsers != null && dtUsers.size() > 0) {
                                for (FlowTaskUserOut dtUser : dtUsers) {
                                    Map<String, Object> dtHash4 = new HashMap<>();
                                    dtHash4.put(Definition.STR_I_USER_ID, dtUser.getiUserId());
                                    dtHash4.put(Definition.STR_I_USER_NAME, dtUser.getiUserName());
                                    if (StringUtils.isEmpty(dtUser.getiSignImage())) {
                                        dtHash4.put(Definition.STR_I_SIGN_IMAGE, Definition.NULL_STR_1);
                                    } else {
                                        dtHash4.put(Definition.STR_I_SIGN_IMAGE, dtUser.getiSignImage());
                                    }
                                    dtHash4.put(Definition.STR_I_PROXY_USER_ID, dtUser.getiProxyUserId());
                                    dtHash4.put(Definition.STR_I_PROXY_USER_NAME, dtUser.getiProxyUserName());
                                    if (StringUtils.isEmpty(dtUser.getiProxySignImage())) {
                                        dtHash4.put(Definition.STR_I_PROXY_SIGN_IMAGE, Definition.NULL_STR_1);
                                    } else {
                                        dtHash4.put(Definition.STR_I_PROXY_SIGN_IMAGE, dtUser.getiProxySignImage());
                                    }
                                    dtHash4.put(Definition.STR_I_USER_OPERATION, dtUser.getiUserOperation());
                                    dtHash4.put(Definition.STR_I_RESULT, dtUser.getiResult());
                                    dtHash4.put(Definition.STR_I_COMMENT, dtUser.getiComment());
                                    dtHash4.put(Definition.STR_I_UPD_DATE, dtUser.getiUpdDate());
                                    list3.add(dtHash4);
                                }
                            }
                            dtHash3.put(Definition.STR_USER, list3);
                            list2.add(dtHash3);
                        }
                        dtHash2.put(Definition.STR_SUBNODE, list2);
                        list1.add(dtHash2);
                    }
                }
            }
            //取消，删除的场合
            if (cancelFlg || deleteFlg) {
                Map<String, Object> dtHash2 = new HashMap<>();
                Map<String, Object> dtHash3 = new HashMap<>();
                List<Map<String, Object>> list2 = new ArrayList<>();
                //申请人详细
                String dtUserName =  sysUserDeptMapper.selectNickNameById(dtApply.getiApplUser());
                //取消
                if (cancelFlg) {
                    //申请人取消再提交
                    dtHash2.put(Definition.STR_I_NODE_NAME, bLanguageMsService.GetMessage(language,
                            Definition.PAGE_401020, Definition.MSG_401020_0021));
                    cancelFlg = false;
                    List<Map<String, Object>> list3 = new ArrayList<>();
                    if (StringUtils.isNotEmpty(dtUserName)) {
                        Map<String, Object> dtHash4 = new HashMap<>();
                        dtHash4.put(Definition.STR_I_USER_ID, dtApply.getiApplUser());
                        dtHash4.put(Definition.STR_I_USER_NAME, dtUserName);
                        //取消时间
                        dtHash4.put(Definition.STR_I_UPD_DATE, dtApply.getiCancelDate());
                        //取消状态
                        dtHash4.put(Definition.STR_I_RESULT, Definition.FLG_2);

                        list3.add(dtHash4);
                    }
                    dtHash3.put(Definition.STR_USER, list3);
                    list2.add(dtHash3);
                    dtHash2.put(Definition.STR_SUBNODE, list2);
                    list1.add(dtHash2);
                }
                //删除
                if (deleteFlg) {
                    //申请人删除
                    dtHash2.put(Definition.STR_I_NODE_NAME, bLanguageMsService.GetMessage(language,
                            Definition.PAGE_401020, Definition.MSG_401020_0022));
                    deleteFlg = false;
                    List<Map<String, Object>> list3 = new ArrayList<>();
                    if (StringUtils.isNotEmpty(dtUserName)) {
                        Map<String, Object> dtHash4 = new HashMap<>();
                        dtHash4.put(Definition.STR_I_USER_ID, dtApply.getiApplUser());
                        dtHash4.put(Definition.STR_I_USER_NAME, dtUserName);
                        //删除时间
                        dtHash4.put(Definition.STR_I_UPD_DATE, dtApply.getiDeleteDate());
                        //删除状态
                        dtHash4.put(Definition.STR_I_RESULT, Definition.FLG_3);
                        list3.add(dtHash4);
                    }
                    dtHash3.put(Definition.STR_USER, list3);
                    list2.add(dtHash3);
                    dtHash2.put(Definition.STR_SUBNODE, list2);
                    list1.add(dtHash2);
                }
            }
        }
        //存在审批流程
        dtHash1.put(Definition.STR_NODE, list1);
        return dtHash1;
    }

    /*
     取得流程审批人
     */
    @Override
    public List<FlowTaskUserOut> selectFlowTaskUser(String iApplyId, Long iSubmitTimes,
                                                    String iNodeId, String iNodeSubId) {
        WFlowUser wFlowUser = new WFlowUser();
        wFlowUser.setiApplyId(iApplyId);
        wFlowUser.setiSubmitTimes(iSubmitTimes);
        wFlowUser.setiNodeId(iNodeId);
        wFlowUser.setiNodeSubId(iNodeSubId);
        return wFlowUserMapper.selectFlowTaskUser(wFlowUser);
    }

    /*
    取得流程详情
    */
    @Override
    public WFlowTask selectFlowTaskDetail(String applyId, Long applyTimes,
                                          String nodeId, String nodeSubId) {
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        wFlowTask.setiNodeId(nodeId);
        wFlowTask.setiNodeSubId(nodeSubId);
        return wFlowTaskMapper.selectFlowTaskDetail(wFlowTask);
    }

    /*
   取得Node数
    */
    @Override
    public Long selectCountNodenum(String applyId, Long applyTimes) {
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        return wFlowTaskMapper.selectCountNodenum(wFlowTask);
    }

    /*
   取得SUBID数
    */
    @Override
    public Long selectCountSubnum(String applyId, Long applyTimes, String nodeId) {
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(applyId);
        wFlowTask.setiSubmitTimes(applyTimes);
        wFlowTask.setiNodeId(nodeId);
        return wFlowTaskMapper.selectCountSubnum(wFlowTask);
    }

    /*
    取得审批中节点
     */
    @Override
    public List<FlowTask> getCurrentNodeById(String iApplyId, Long iSubmitTimes) {
        List<FlowTask> resList = new ArrayList<>();
        //空
        if (StringUtils.isEmpty(iApplyId) || iSubmitTimes == 0) {
            return resList;
        }
        //取得审批节点
        WFlowTask wFlowTask = new WFlowTask();
        wFlowTask.setiApplyId(iApplyId);
        wFlowTask.setiSubmitTimes(iSubmitTimes);
        List<CurrentNodeOut> listData = wFlowTaskMapper.selectCurrentNodeById(wFlowTask);
        String preSubId = Definition.NULL_STR_1;
        if (listData != null && listData.size() >0) {
            FlowTask flowTask = new FlowTask();
            //获取第一个sub节点的信息
            preSubId = listData.get(0).getiNodeSubId();
            //申請ID
            flowTask.getwFlowTask().setiApplyId(listData.get(0).getiApplyId());
            //申請回数
            flowTask.getwFlowTask().setiSubmitTimes(listData.get(0).getiSubmitTimes());
            //节点ID
            flowTask.getwFlowTask().setiNodeId(listData.get(0).getiNodeId());
            //节点SUBID
            flowTask.getwFlowTask().setiNodeSubId(listData.get(0).getiNodeSubId());
            //节点名称
            flowTask.getwFlowTask().setiNodeName(listData.get(0).getiNodeName());
            //节点对象者类型
            flowTask.getwFlowTask().setiNodeObjType(listData.get(0).getiNodeObjType());
            //节点对象者ID
            flowTask.getwFlowTask().setiNodeObjId(listData.get(0).getiNodeObjId());
            //节点属性
            flowTask.getwFlowTask().setiNodeProperty(listData.get(0).getiNodeProperty());
            //节点关系
            flowTask.getwFlowTask().setiNodeRelationship(listData.get(0).getiNodeRelationship());
            //先行节点ID
            flowTask.getwFlowTask().setiPreNodeId(listData.get(0).getiPreNodeId());
            //后续节点ID
            flowTask.getwFlowTask().setiNexNodeId(listData.get(0).getiNexNodeId());
            //节点状态
            flowTask.getwFlowTask().setiNodeStatus(listData.get(0).getiNodeStatus());
            //全部审批信息
            flowTask.setUserList(new ArrayList<>());
            //SUBID的审批结果
            flowTask.setAduitResult_OK(new ArrayList<>());
            flowTask.setAduitResult_NG(new ArrayList<>());

            for (CurrentNodeOut nodeOut : listData) {
                if (!preSubId.equals(nodeOut.getiNodeSubId())) {
                    resList.add(flowTask);
                    flowTask = new FlowTask();
                    //节点SUBID
                    preSubId = nodeOut.getiNodeSubId();
                    //申請ID
                    flowTask.getwFlowTask().setiApplyId(nodeOut.getiApplyId());
                    //申請回数
                    flowTask.getwFlowTask().setiSubmitTimes(nodeOut.getiSubmitTimes());
                    //节点ID
                    flowTask.getwFlowTask().setiNodeId(nodeOut.getiNodeId());
                    //节点SUBID
                    flowTask.getwFlowTask().setiNodeSubId(nodeOut.getiNodeSubId());
                    //节点名称
                    flowTask.getwFlowTask().setiNodeName(nodeOut.getiNodeName());
                    //节点对象者类型
                    flowTask.getwFlowTask().setiNodeObjType(nodeOut.getiNodeObjType());
                    //节点对象者ID
                    flowTask.getwFlowTask().setiNodeObjId(nodeOut.getiNodeObjId());
                    //节点属性
                    flowTask.getwFlowTask().setiNodeProperty(nodeOut.getiNodeProperty());
                    //节点关系
                    flowTask.getwFlowTask().setiNodeRelationship(nodeOut.getiNodeRelationship());
                    //先行节点ID
                    flowTask.getwFlowTask().setiPreNodeId(nodeOut.getiPreNodeId());
                    //后续节点ID
                    flowTask.getwFlowTask().setiNexNodeId(nodeOut.getiNexNodeId());
                    //节点状态
                    flowTask.getwFlowTask().setiNodeStatus(nodeOut.getiNodeStatus());
                    //全部审批信息
                    flowTask.setUserList(new ArrayList<>());
                    //SUBID的审批结果
                    flowTask.setAduitResult_OK(new ArrayList<>());
                    flowTask.setAduitResult_NG(new ArrayList<>());
                }
                WFlowUser flowUser = new WFlowUser();
                //申請ID
                flowUser.setiApplyId(flowTask.getwFlowTask().getiApplyId());
                //申請回数
                flowUser.setiSubmitTimes(flowTask.getwFlowTask().getiSubmitTimes());
                //节点ID
                flowUser.setiNodeId(flowTask.getwFlowTask().getiNodeId());
                //节点SUBID
                flowUser.setiNodeSubId(flowTask.getwFlowTask().getiNodeSubId());
                //审批者ID
                flowUser.setiUserId(nodeOut.getiUserId());
                //代理审批者ID
                flowUser.setiProxyUserId(nodeOut.getiProxyUserId());
                //审批者操作
                flowUser.setiUserOperation(nodeOut.getiUserOperation());
                //审批结果
                flowUser.setiResult(nodeOut.getiResult());
                flowTask.getUserList().add(flowUser);
                //承認済
                if (Definition.STR_PASS.equals(flowUser.getiResult())) {
                    flowTask.getAduitResult_OK().add(Definition.STR_PASS);
                }
                //拒否
                if (Definition.STR_REJECT.equals(flowUser.getiResult())) {
                    flowTask.getAduitResult_NG().add(Definition.STR_REJECT);
                }
            }
            resList.add(flowTask);
        }
        return resList;
    }

    /*
    NodeConfig设值
     */
    private NodeConfig setNodeConfig(NodeConfig inData) {
        if (inData == null) {
            return null;
        } else {
            NodeConfig outData = new NodeConfig();
            BeanUtils.copyProperties(inData, outData);
            return outData;
        }
    }

    /*
    取得子节点描述
    */
    private String getDescription(String nodeObjType, String nodeObjId, String nodeProperty,
                                  String nodeRelationship, Long isub, String language) {
        //描述
        String description = Definition.NULL_STR_1;
        //节点关系（会签/或签）
        if (!Definition.FST_VERSION.equals(isub)) {

            if (Definition.FLG_1.equals(nodeRelationship)) {
                //且
                description = getMsgP401020(Definition.MSG_401020_0017, Definition.NULL_STR_1, language);
            } else if (Definition.FLG_2.equals(nodeRelationship)) {
                //或
                description = getMsgP401020(Definition.MSG_401020_0018, Definition.NULL_STR_1, language);
            }
        }
        //节点类型
        switch(nodeObjType) {
            case Definition.STR_PERSON:
                //个人
                description = description + getMsgP401020(Definition.MSG_401020_0013, Definition.NULL_STR_1, language);
                break;
            case Definition.STR_DEPT:
                //部门
                description = description + getAnyAllMaster(language, nodeProperty,
                        sysUserDeptMapper.selectDeptNameByDeptId(nodeObjId));
                break;
            case Definition.STR_ROLE:
                //分组
                description = description + getAnyAllMaster(language, nodeProperty,
                        wGroupService.selectRroupNameByKey(nodeObjId));
                break;
            case Definition.STR_POST:
                //岗位
                description = description + getAnyAllMaster(language, nodeProperty,
                        sysPostPartService.selectPostNameByPostId(nodeObjId));
                break;
            case Definition.STR_DEPT_POST:
                String strName = Definition.NULL_STR_1;
                //部门+岗位
                if (StringUtils.isNotEmpty(nodeObjId)) {
                    //默认[0]部门,[1]岗位
                    String[] deptPost = nodeObjId.split(Definition.RADICAL_LINE);
                    strName = sysUserDeptMapper.selectDeptNameByDeptId(deptPost[0]);
                    if (deptPost.length == 2) {
                        strName = strName + Definition.RADICAL_LINE3 +
                                sysPostPartService.selectPostNameByPostId(deptPost[1]);
                    }
                }
                description = description + getAnyAllMaster(language, nodeProperty, strName);
                break;
            case Definition.STR_PART:
                //角色
                description = description + getAnyAllMaster(language, nodeProperty,
                        sysPostPartService.selectRoleNameByRoleId(nodeObjId));
                break;
        }
        return description;
    }

    /*
    取得消息(PAGE_401020)
     */
    private String getMsgP401020(String msgCode, String strName, String language) {
        String resStr = bLanguageMsService.GetMessage(language, Definition.PAGE_401020, msgCode);
        if (StringUtils.isNotEmpty(strName)) {
            resStr = resStr.replace(Definition.STR_HOSI, strName);
        }
        return resStr;
    }

    /**
     * 取得任意，全员，负责人的msg
     */
    private String getAnyAllMaster(String language, String inType, String strName) {
        String resStr = Definition.NULL_STR_1;
        switch(inType) {
            case Definition.STR_ALL:
                resStr = getMsgP401020(Definition.MSG_401020_0015, strName, language);
                break;
            case Definition.STR_ANY:
                resStr = getMsgP401020(Definition.MSG_401020_0014, strName, language);
                break;
            case Definition.STR_MASTER:
                resStr = getMsgP401020(Definition.MSG_401020_0016, strName, language);
                break;
        }
        return resStr;
    }
}
