package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WFlowUser;
import com.ruoyi.project.workflow.domain.db.out.ApplyResumeOut;
import com.ruoyi.project.workflow.mapper.WFlowUserMapper;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WFlowUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.List;

@Service("wFlowUserService")
public class WFlowUserServiceImpl implements WFlowUserService {
    @Autowired
    private WFlowUserMapper wFlowUserMapper;
    @Autowired
    private IBLanguageMsService bLanguageMsService;

    @Override
    public List<ApplyResumeOut> selectApplyResume(String language, String applyId, Long applyTimes) {
        WFlowUser wFlowUser = new WFlowUser();
        wFlowUser.setLanguageType(language);
        wFlowUser.setiApplyId(applyId);
        wFlowUser.setiSubmitTimes(applyTimes);
        return wFlowUserMapper.selectApplyResume(wFlowUser);
    }

    /*
    更新审批结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String updateAudit(String language, String applyId, Long submitTimes, String result, String comment) {
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        try {
            //保存审批数据
            WFlowUser wFlowUser = new WFlowUser();
            wFlowUser.setiApplyId(applyId);
            wFlowUser.setiSubmitTimes(submitTimes);
            wFlowUser.setiUserId(userId);
            wFlowUser.setiResult(result);
            wFlowUser.setiComment(comment);
            wFlowUser.setiUpdId(userId);
            wFlowUser.setiUpdDate(nowDate);
            int updCnt = wFlowUserMapper.updateAuditData(wFlowUser);
            if (updCnt == 0) {
                //代理保存审批数据
                WFlowUser proxyFlowUser = new WFlowUser();
                //admin代理审批的场合，需要将代理user更新为admin
                if (Definition.FLG_ZERO.equals(ClsBase.isAdmin())) {
                    proxyFlowUser.setiProxyUserId(userId);
                } else {
                    proxyFlowUser.setiProxyUserId(null);
                }
                proxyFlowUser.setiApplyId(applyId);
                proxyFlowUser.setiSubmitTimes(submitTimes);
                proxyFlowUser.setiResult(result);
                proxyFlowUser.setiComment(comment);
                proxyFlowUser.setiUpdId(userId);
                proxyFlowUser.setiUpdDate(nowDate);
                updCnt = wFlowUserMapper.updateProxyAuditData(proxyFlowUser);
                //更新0件的场合
                if (updCnt == 0) {
                    //申请书状态已发生变更，请确认状态
                    return bLanguageMsService.GetMessage(language, Definition.PAGE_401030, Definition.MSG_401030_0005);
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0061);
        }
        return  Definition.RESULT_S;
    }
}
