package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WApplication;
import com.ruoyi.project.workflow.domain.WFormLayout;
import com.ruoyi.project.workflow.domain.WReferenceUser;
import com.ruoyi.project.workflow.domain.db.in.WFormDataInsert;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignMyAppLyIn;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignMyApplyOut;
import com.ruoyi.project.workflow.mapper.*;
import com.ruoyi.project.workflow.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.*;

@Service("wFormDesignMyAppLyService")
public class WFormDesignMyApplyServiceImpl implements WFormDesignMyApplyService {

    @Autowired
    private WApplicationService wApplicationService;
    @Autowired
    private WFormDataService wFormDataService;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WReferenceUserService wReferenceUserService;
    @Autowired
    private WApplicationMapper wApplicationMapper;
    @Autowired
    private BAttachMMapper bAttachMMapper;
    @Autowired
    private WFormDataMapper wFormDataMapper;
    @Autowired
    private WFormDesignMyApplyMapper wFormDesignMyAppLyMapper;
    @Autowired
    private WReferenceUserMapper wReferenceUserMapper;

    /**
     *我的申请一览
     **/
    @Override
    public List<WFormDesignMyApplyOut> selectMyApplyList(WFormDesignMyAppLyIn wFormDesignMyAppLyIn) {
        return wFormDesignMyAppLyMapper.selectMyApplyList(wFormDesignMyAppLyIn);
    }

    /**
     *我的申请一览件数
     **/
    @Override
    public Long selectMyApplyCount(WFormDesignMyAppLyIn wFormDesignMyAppLyIn) {
        return wFormDesignMyAppLyMapper.selectMyApplyCount(wFormDesignMyAppLyIn);
    }

    /**
     *我的申请一览（导出）
     **/
    @Override
    public List<WFormDesignMyApplyOut> selectMyApplyUnLimt(WFormDesignMyAppLyIn wFormDesignMyAppLyIn) {
        return wFormDesignMyAppLyMapper.selectMyApplyUnLimt(wFormDesignMyAppLyIn);
    }

    /*
    取得申请数据
     */
    @Override
    public Map<String, Object> selectFormDataByApplyId(String applyId, Long submitTimes) {
        Map<String, Object> dtHash = new HashMap<>();
        //取得表单结构
        String tableName = wApplicationService.selectFormLayoutByApplyId(applyId, submitTimes, Definition.TBL_NAME);
        //判断表名是否存在
        if (wFormDataService.selectCheckTableExists(Definition.TABLE_SCHEMA, tableName)){
            dtHash = wFormDataService.selectFormDataByApplyId(tableName, applyId);
        };
        return dtHash;
    }

    /*
    追加阅览者到DB
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addReferenceUsers(String language, String applyId, String users) {
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        //取得阅览人详细
        List<String> referenceUsers = wReferenceUserService.selectReferenceUser(applyId);
        try {
            if (users != null) {
                for (String refUser : users.split(Definition.ROLE_DELIMETER)) {
                    if (!referenceUsers.contains(refUser)) {
                        WReferenceUser wReferenceUser = new WReferenceUser();
                        wReferenceUser.setiApplyId(applyId);
                        wReferenceUser.setiUserId(refUser);
                        wReferenceUser.setiEntryId(userId);
                        wReferenceUser.setiEntryDate(nowDate);
                        wReferenceUser.setiUpdId(userId);
                        wReferenceUser.setiUpdDate(nowDate);
                        wReferenceUserMapper.insertWReferenceUser(wReferenceUser);
                    }
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0211);
        }

        return Definition.RESULT_S;
    }

    /*
    删除申请
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteApplyByApplyId(String language, String applyId) {
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        try {
            //删除申请（[草稿/审批未通过]状态，可删除）
            WApplication wApplication = new WApplication();
            wApplication.setiApplyId(applyId);
            wApplication.setiDeleteDate(nowDate);
            wApplication.setiStatus(Definition.FLG_4);
            wApplication.setiUpdId(userId);
            wApplication.setiUpdDate(nowDate);
            int updCnt = wApplicationMapper.deleteApply(wApplication);
            if (updCnt == 0) {
                //删除失败：申请书状态不是[草稿/审批未通过]场合，不可删除
                return bLanguageMsService.GetMessage(language, Definition.PAGE_401020, Definition.MSG_401020_0009);
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0081);
        }

        return Definition.RESULT_S;
    }

     /*
     取消申请
     */
     @Override
     @Transactional(rollbackFor = Exception.class)
     public String cancelApplyByApplyId(String language, String applyId, Long applyTimes) {
         //时间
         Date nowDate = new Date();
         //用户
         String userId = ClsBase.getCurrentUser();
         try {
            //取消申请（[承认中]状态，可取消）
             WApplication cancelApp = new WApplication();
             cancelApp.setiApplyId(applyId);
             cancelApp.setiSubmitTimes(applyTimes);
             cancelApp.setiStatus(Definition.FLG_1);
             cancelApp.setiCancelDate(nowDate);
             cancelApp.setiUpdId(userId);
             cancelApp.setiUpdDate(nowDate);
             int updCnt = wApplicationMapper.cancelApply(cancelApp);
            //取消申请书失败的场合
             if (updCnt == 0) {
                 //申请书状态不是[承认中]，无法取消
                 return bLanguageMsService.GetMessage(language, Definition.PAGE_401020, Definition.MSG_401020_0010);
             }
             //默认追加一个回数+1的申请
             if (updCnt > 0) {
                 WApplication newApp = new WApplication();
                 newApp.setiApplyId(applyId);
                 newApp.setiSubmitTimes(applyTimes);
                 newApp.setiStatus(Definition.FLG_ZERO);
                 newApp.setiEntryId(userId);
                 newApp.setiEntryDate(nowDate);
                 newApp.setiUpdId(userId);
                 newApp.setiUpdDate(nowDate);
                 wApplicationMapper.insertAddSubmitTimes(newApp);
             }
         } catch (Exception e) {
             //回滚事务
             TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
             return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0131);
         }

         return Definition.RESULT_S;
     }

    /*
   文件删除
    */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteAttach(String language, String fileNames) {
        try {
            bAttachMMapper.deleteByFileNames(fileNames);
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0091);
        }

        return Definition.RESULT_S;
    }

    /*
    更新数据
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String updateDatas(String language, List<WFormLayout> listWFormLayout, Map<String, Object> params,
                              String strTblName, String applayId, String preApplayId, Long appalyTimes,
                              String formId, String status, String ccList) {
        try {
            //一时保存、申请未通过的场合，更新数据
            if (Definition.STR_INACTIVE.equals(status)
                    || Definition.STR_REJECT.equals(status)) {
                if (listWFormLayout != null && listWFormLayout.size() > 0) {
                    //更新项目设定
                    List<String> sqlList = new ArrayList<>();
                    for (WFormLayout item : listWFormLayout) {
                        //组件对象
                        String strCol = ClsBase.getTrimData(item.getiFormDataCol());
                        if (StringUtils.isNotEmpty(strCol)) {
                            //组件对象存在的场合
                            if (params.get(strCol) != null) {
                                String sql = Definition.QUOTATION_MARK + strCol + Definition.QUOTATION_MARK;
                                String strVal = ClsBase.getTrimData(params.get(strCol).toString());
                                if (StringUtils.isEmpty(strVal)) {
                                    sql = sql + Definition.STR_EQUAL + Definition.NULL_STR_2;
                                } else {
                                    sql = sql + Definition.STR_EQUAL + Definition.SINGLE_QUOTATION_MARK +
                                            strVal + Definition.SINGLE_QUOTATION_MARK;
                                }
                                sqlList.add(sql);
                            }
                        }
                    }
                    //更新申请数据
                    if (sqlList != null && sqlList.size() > 0) {
                        wFormDataMapper.updateByApplyId(strTblName, applayId, sqlList);
                    }
                }
            } else {
                //Sql做成
                String strCols = Definition.LEFT_CURVE + Definition.I_APPLY_ID + Definition.ROLE_DELIMETER;
                String strVals = Definition.LEFT_CURVE + Definition.SINGLE_QUOTATION_MARK + applayId +
                        Definition.SINGLE_QUOTATION_MARK + Definition.ROLE_DELIMETER;
                if (listWFormLayout != null && listWFormLayout.size() > 0) {
                    for (WFormLayout item : listWFormLayout) {
                        //组件对象
                        String strCol = ClsBase.getTrimData(item.getiFormDataCol());
                        if (StringUtils.isNotEmpty(strCol)) {
                            strCols = strCols + Definition.QUOTATION_MARK + strCol +
                                    Definition.QUOTATION_MARK + Definition.ROLE_DELIMETER;
                            String strVal = Definition.NULL_STR_1;
                            //组件对象存在的场合
                            if (params.get(strCol) == null) {
                                return bLanguageMsService.GetMessage(language, Definition.PAGE_401020,
                                        Definition.MSG_401020_0008);
                            } else {
                                strVal = ClsBase.getTrimData(params.get(strCol).toString());
                            }
                            if (StringUtils.isNotEmpty(strVal)) {
                                strVals = strVals + Definition.SINGLE_QUOTATION_MARK + strVal +
                                        Definition.SINGLE_QUOTATION_MARK + Definition.ROLE_DELIMETER;
                            } else {
                                strVals = strVals + Definition.NULL_STR_2 + Definition.ROLE_DELIMETER;
                            }
                        }
                    }
                    //末尾逗号去除
                    strCols = strCols.substring(0, strCols.length() - 1);
                    strVals = strVals.substring(0, strVals.length() - 1);
                    strCols = strCols + Definition.RIGHT_CURVE;
                    strVals = strVals + Definition.RIGHT_CURVE;
                    //保存申请数据
                    WFormDataInsert wFormDataInsert = new WFormDataInsert();
                    wFormDataInsert.setStrTblName(strTblName);
                    wFormDataInsert.setStrCols(strCols);
                    wFormDataInsert.setStrVals(strVals);
                    wFormDataMapper.insertWFormData(wFormDataInsert);
                }
            }
            //时间
            Date nowDate = new Date();
            //用户
            String userId = ClsBase.getCurrentUser();
            //申请书状态为一时保存以外的场合
            if (!Definition.STR_INACTIVE.equals(status)) {
                WApplication wApplication = new WApplication();
                wApplication.setiApplyId(applayId);
                wApplication.setiSubmitTimes(appalyTimes);
                wApplication.setiFormId(Long.valueOf(formId));
                wApplication.setiApplUser(userId);
                wApplication.setiPreApplId(preApplayId);
                wApplication.setiStatus(Definition.STR_INACTIVE);
                wApplication.setiSubmitDate(null);
                wApplication.setiEntryId(userId);
                wApplication.setiEntryDate(nowDate);
                wApplication.setiUpdId(userId);
                wApplication.setiUpdDate(nowDate);
                wApplicationMapper.insertWApplication(wApplication);
            }
            //删除前回保存的cc一览, 一时保存/申请未通过的场合，删除, 申请通过的场合，0件删除
            wReferenceUserMapper.deleteByApplyId(applayId);
            //保存cc一览
            if (StringUtils.isNotEmpty(ccList)) {
                for (String ccUser : ccList.split(Definition.ROLE_DELIMETER)) {
                    WReferenceUser insertData = new WReferenceUser();
                    insertData.setiApplyId(applayId);
                    insertData.setiUserId(ccUser);
                    insertData.setiEntryId(userId);
                    insertData.setiEntryDate(nowDate);
                    insertData.setiUpdId(userId);
                    insertData.setiUpdDate(nowDate);
                    wReferenceUserMapper.insertWReferenceUser(insertData);
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0091);
        }

        return Definition.RESULT_S;
    }
}
