package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WApplication;
import com.ruoyi.project.workflow.domain.WFlowTask;
import com.ruoyi.project.workflow.domain.WFlowUser;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignPendingIn;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignPendingOut;
import com.ruoyi.project.workflow.mapper.WApplicationMapper;
import com.ruoyi.project.workflow.mapper.WFlowTaskMapper;
import com.ruoyi.project.workflow.mapper.WFlowUserMapper;
import com.ruoyi.project.workflow.mapper.WFormDesignPendingMapper;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WApplicationService;
import com.ruoyi.project.workflow.service.WFormDesignPendingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.List;

@Service("wFormDesignPendingService")
public class WFormDesignPendingServiceImpl implements WFormDesignPendingService {

    @Autowired
    private WFormDesignPendingMapper wFormDesignPendingMapper;
    @Autowired
    private WFlowTaskMapper wFlowTaskMapper;
    @Autowired
    private WApplicationMapper wApplicationMapper;
    @Autowired
    private WFlowUserMapper wFlowUserMapper;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WApplicationService wApplicationService;

    /**
     * 待申请列表
     */
    @Override
    public List<WFormDesignPendingOut> selectPendingList(WFormDesignPendingIn wFormDesignPendingIn) {
        return wFormDesignPendingMapper.selectPendingList(wFormDesignPendingIn);
    }

    /**
     * 待申请列表一览件数
     */
    @Override
    public Long selectPendingCount(WFormDesignPendingIn wFormDesignPendingIn) {
        return wFormDesignPendingMapper.selectPendingCount(wFormDesignPendingIn);
    }

    /**
     * 待申请列表（无limit）
     */
    @Override
    public List<WFormDesignPendingOut> selectPendingUnLimt(WFormDesignPendingIn wFormDesignPendingIn) {
        return wFormDesignPendingMapper.selectPendingUnLimt(wFormDesignPendingIn);
    }

    /*
    更新task及申请书
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String updateTaskAndApply(String language, String applyId, Long submitTimes, String taskStatus,
                                     String applicationStatus, String nextTask) {
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        //取得申请书的表单ID
        WApplication wApplication = wApplicationService.selectApplyInfoByApplyIds(applyId, submitTimes);
        //更新状态
        Boolean updRtnFlg = true;
        int updCnt = 0;
        try {
            //task状态不为空，更新w_flow_task
            if (StringUtils.isNotEmpty(taskStatus)) {
                WFlowTask wFlowTask = new WFlowTask();
                wFlowTask.setiApplyId(applyId);
                wFlowTask.setiSubmitTimes(submitTimes);
                wFlowTask.setiNodeStatus(taskStatus);
                wFlowTask.setiUpdId(userId);
                wFlowTask.setiUpdDate(nowDate);
                updCnt = wFlowTaskMapper.updateWFlowTaskStatus(wFlowTask);
                if (updCnt == 0) {
                    updRtnFlg = false;
                }
            }
            //application状态不为空，更新w_application
            if (updRtnFlg && StringUtils.isNotEmpty(applicationStatus)) {
                WApplication updateWApp = new WApplication();
                updateWApp.setiApplyId(applyId);
                updateWApp.setiSubmitTimes(submitTimes);
                updateWApp.setiStatus(applicationStatus);
                updateWApp.setiUpdId(userId);
                updateWApp.setiUpdDate(nowDate);
                updCnt = wApplicationMapper.updateWApplication(updateWApp);
                if (updCnt == 0) {
                    updRtnFlg = false;
                }
            }
            //后续节点不为空，更新w_flow_task
            if (updRtnFlg && StringUtils.isNotEmpty(nextTask)) {
                WFlowTask wFlowTask = new WFlowTask();
                wFlowTask.setiApplyId(applyId);
                wFlowTask.setiSubmitTimes(submitTimes);
                wFlowTask.setiNodeStatus(Definition.STR_AUDITING);
                wFlowTask.setiNodeId(nextTask);
                wFlowTask.setiUpdId(userId);
                wFlowTask.setiUpdDate(nowDate);
                wFlowTaskMapper.updateWFlowTaskNextTask(wFlowTask);
                //给下一个审批节点的审批者设定代理人
                WFlowUser updateFlowUser = new WFlowUser();
                updateFlowUser.setiApplyId(applyId);
                updateFlowUser.setiSubmitTimes(submitTimes);
                updateFlowUser.setiUpdId(userId);
                updateFlowUser.setiUpdDate(nowDate);
                updateFlowUser.getParams().put(Definition.STR_FORMID, wApplication.getiFormId());
                wFlowUserMapper.updateNextProxyUser(updateFlowUser);
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0061);
        }
        //更新失败的场合
        if (!updRtnFlg) {
            //申请书状态已发生变更，请确认状态
            return bLanguageMsService.GetMessage(language, Definition.PAGE_401030, Definition.MSG_401030_0005);
        }
        return Definition.RESULT_S;
    }
}
