package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WFormDesign;
import com.ruoyi.project.workflow.domain.WFormLayout;
import com.ruoyi.project.workflow.domain.WFormUser;
import com.ruoyi.project.workflow.domain.base.SeachBase;
import com.ruoyi.project.workflow.domain.db.WFormDesignEidt;
import com.ruoyi.project.workflow.domain.db.in.WFormDataCreate;
import com.ruoyi.project.workflow.domain.db.in.WFormDesignYilanIn;
import com.ruoyi.project.workflow.domain.db.out.BWordbookMsOut;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignCopyCreate;
import com.ruoyi.project.workflow.domain.db.out.WFormDesignYilanOut;
import com.ruoyi.project.workflow.domain.input.WFControlToTableCol;
import com.ruoyi.project.workflow.mapper.*;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WFormDesignService;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Date;

@Service("wFormDesignService")
public class WFormDesignServiceImpl implements WFormDesignService {

    @Autowired
    private WFormDesignMapper wFormDesignMapper;
    @Autowired
    private WFormLayoutMapper wFormLayoutMapper;
    @Autowired
    private WFormUserMapper wFormUserMapper;
    @Autowired
    private WFormDataMapper wFormDataMapper;
    @Autowired
    private SysUserDeptMapper sysUserDeptMapper;
    @Autowired
    private IBLanguageMsService bLanguageMsService;

    /*
      取得wf列表内容
    */
    @Override
    public List<WFormDesignYilanOut> selectWFormDesignList(WFormDesignYilanIn wFormDesignYilanIn){
        return wFormDesignMapper.selectWFormDesignList(wFormDesignYilanIn);
    }

    /*
      取得wf列表件数
    */
    @Override
    public Long selectWFormDesignListCount(WFormDesignYilanIn wFormDesignYilanIn) {
        return wFormDesignMapper.selectWFormDesignListCount(wFormDesignYilanIn);
    }

    /*
      导出
    */
    @Override
    public List<WFormDesignYilanOut> selectWFormDesignListExport(WFormDesignYilanIn wFormDesignYilanIn){
        return wFormDesignMapper.selectWFormDesignListUnLimt(wFormDesignYilanIn);
    }

    /*
      删除
    */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteWFormDesign(String language, WFormDesignYilanIn wFormDesignYilanIn) {
        //取得数据(主键)
        List<WFormDesignYilanOut> listWFormDesign =  wFormDesignMapper.selectWFormDesignListUnLimt(wFormDesignYilanIn);
        WFormDesignYilanOut firstWFormDesign = listWFormDesign.get(0);

        //查询表单数据结构表中的版本追加列
        WFormLayout inFormLayout = new WFormLayout();
        inFormLayout.setiFormDataTblname(firstWFormDesign.getI_form_data_tblname());
        inFormLayout.setiFormVersion(firstWFormDesign.getI_form_version());
        List<WFormLayout> listWFormLayout = wFormLayoutMapper.selectWFormLayoutList(inFormLayout);

        try {
            //删除表单设计表数据
            wFormDesignMapper.deleteWFormDesignByIFormId(firstWFormDesign.getI_form_id());

            //删除表单数据结构表数据
            wFormLayoutMapper.deleteWFormLayoutByIds(inFormLayout);

            //删除利用者表数据
            wFormUserMapper.deleteWFormUserByIFormId(firstWFormDesign.getI_form_id());

            //初版的场合
            if (Definition.FST_VERSION == firstWFormDesign.getI_form_version()) {
                //删除表单数据表
                wFormDataMapper.dropWFormData(firstWFormDesign.getI_form_data_tblname());
            } else {
                //还原前版本表单的失效日
                wFormDesignMapper.updateIUseEYmd(firstWFormDesign.getI_pre_ver_form_id());

                //更新前版本表单的编辑FLG
                wFormDesignMapper.updateIOprFlg(firstWFormDesign.getI_pre_ver_form_id());

                //删除表单数据表中的列
                for (WFormLayout item : listWFormLayout) {
                    wFormDataMapper.dropColumn(item);
                }
            }
            return Definition.RESULT_S;
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0081);
        }
    }

    /*
    主键检索
     */
    @Override
    public WFormDesignEidt selectWFormDesignById(String iFormId){
        return  wFormDesignMapper.selectWFormDesignById(iFormId);
    }

    /*
    插入
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String insertWFormDesign(String language, WFormDesign inData,
                                    String optionType, String optionValue) {
        //输入
        WFormDesign wFormDesign= new WFormDesign();
        BeanUtils.copyProperties(inData, wFormDesign);
        //从表单中解析出控件名
        Map<String, WFControlToTableCol> controlNameDict = null;
        try {
           controlNameDict = convertJsonControlName(inData.getiFormLayout());
        } catch (Exception e) {
            String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0010);
            msg = editMsg(msg, e.getMessage());
            return msg;
        }
        //表单名重复性check
        if (checkFormNameExist(inData.getiFormName(), Definition.NULL_STR_1)) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0019);
        }
        //表单ID采番
        Long newFormId = wFormDesignMapper.getNextIFormId();
        wFormDesign.setiFormId(newFormId);
        //申请数据表名
        String wFormDataTblname = Definition.W_FORM_DATA + newFormId.toString();
        //申请数据表名存在check
        if (checkTblNameExist(wFormDataTblname, Definition.NULL_STR_1)) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0015);
        }
        wFormDesign.setiFormDataTblname(wFormDataTblname);
        //所属公司No取得
        String deptId = ClsBase.getUserDeptId();
        if (StringUtils.isNotEmpty(deptId)) {
            wFormDesign.setiCompanyNo(sysUserDeptMapper.getCompanyId(deptId));
        }
        //版本
        wFormDesign.setiFormVersion(Definition.FST_VERSION);
        //操作
        wFormDesign.setiOprFlg(Definition.FLG_ZERO);
        //时间
        Date nowDate = new Date();
        wFormDesign.setiEntryDate(nowDate);
        wFormDesign.setiUpdDate(nowDate);
        try {
            //保存表单数据
            wFormDesignMapper.insertWFormDesign(wFormDesign);
            //保存利用者数据
            if (StringUtils.isNotEmpty(optionValue) && StringUtils.isNotEmpty(optionType)) {
                for (String iUserId : optionValue.split(Definition.ROLE_DELIMETER)) {
                    WFormUser wFormUser = new WFormUser();
                    wFormUser.setiFormId(newFormId);
                    wFormUser.setiUserType(optionType);
                    wFormUser.setiUserId(iUserId);
                    wFormUser.setiEntryId(inData.getiEntryId());
                    wFormUser.setiEntryDate(nowDate);
                    wFormUser.setiUpdId(inData.getiUpdId());
                    wFormUser.setiUpdDate(nowDate);
                    wFormUserMapper.insertWFormUser(wFormUser);
                }
            }
            //创建申请表
            WFormDataCreate wFormDataCreate = new WFormDataCreate();
            wFormDataCreate.setFormDataTblname(wFormDataTblname);
            if (controlNameDict != null && controlNameDict.size() > 0) {
                List<WFControlToTableCol> dataList = new ArrayList<>();
                for (String strKey : controlNameDict.keySet()) {
                    //申请表
                    dataList.add(controlNameDict.get(strKey));
                    //保存控件名
                    WFormLayout wFormLayout = new WFormLayout();
                    wFormLayout.setiFormDataTblname(wFormDataTblname);
                    wFormLayout.setiFormDataCol(strKey);
                    wFormLayout.setiFormVersion(Definition.FST_VERSION);
                    wFormLayout.setiFormDataControl(controlNameDict.get(strKey).getControl());
                    wFormLayout.setiFormDataControlInitval(controlNameDict.get(strKey).getControlDefaultValue());
                    wFormLayout.setiEntryId(inData.getiEntryId());
                    wFormLayout.setiEntryDate(nowDate);
                    wFormLayout.setiUpdId(inData.getiUpdId());
                    wFormLayout.setiUpdDate(nowDate);
                    wFormLayoutMapper.insertWFormLayout(wFormLayout);
                }
                wFormDataCreate.setdList(dataList);
                //创建申请表
                wFormDataMapper.dropWFormData(wFormDataTblname);
                wFormDataMapper.creatWFormDataTable(wFormDataCreate);
            }
            return  Definition.RESULT_S;
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0091);
        }
    }

    /*
    取得表单状态
     */
    @Override
    public String getFormStatusByFormId(String iFormId) {
        return wFormDesignMapper.getFormStatusByFormId(Long.valueOf(iFormId));
    }

    /*
    拷贝新规表单信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String copyCreateForm(String language, WFormDesign inData, String optionType, String optionValue) {
        //输入
        WFormDesign wFormDesign= new WFormDesign();
        BeanUtils.copyProperties(inData, wFormDesign);
        //表单名重复性check
        if (checkFormNameExist(inData.getiFormName(), inData.getiFormId().toString())) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0019);
        }
        //表单ID采番
        Long newFormId = wFormDesignMapper.getNextIFormId();
        wFormDesign.setiFormId(newFormId);
        //所属公司No取得
        String deptId = ClsBase.getUserDeptId();
        if (StringUtils.isNotEmpty(deptId)) {
            wFormDesign.setiCompanyNo(sysUserDeptMapper.getCompanyId(deptId));
        }
        //从表单中解析出控件名
        Map<String, WFControlToTableCol> controlNameDict = null;
        try {
            controlNameDict = convertJsonControlName(inData.getiFormLayout());
        } catch (Exception e) {
            String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0010);
            msg = editMsg(msg, e.getMessage());
            return msg;
        }
        //表单ID对应表单数据取得
        WFormDesignCopyCreate wFormDesignCopyCreate = wFormDesignMapper.getDataForCopyByFormId(inData.getiFormId());
        if (wFormDesignCopyCreate == null) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0007);
        }
        //版本号+1
        Long nextFormVersion = wFormDesignCopyCreate.getiFormVersion() + Definition.FST_VERSION;
        //设值
        wFormDesign.setiFormVersion(nextFormVersion);
        wFormDesign.setiPreVerFormId(inData.getiFormId());
        wFormDesign.setiFstVerFormId(wFormDesignCopyCreate.getiFstVerFormId());
        wFormDesign.setiFormDataTblname(wFormDesignCopyCreate.getiFormDataTblname());
        //从表单中解析出控件名
        Map<String, WFControlToTableCol> preControlList = null;
        try {
            preControlList = convertJsonControlName(wFormDesignCopyCreate.getiFormLayout());
        } catch (Exception e) {
            String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0010);
            msg = editMsg(msg, e.getMessage());
            return msg;
        }
        //去除已存在的控件
        for (String preKey : preControlList.keySet()) {
            controlNameDict.remove(preKey);
        }
        //履历版本表单列数据取得
        if (controlNameDict.keySet().size() > 0) {
            WFormDataCreate searchIn = new WFormDataCreate();
            //表单名
            searchIn.setFormDataTblname(wFormDesignCopyCreate.getiFormDataTblname());
            for (String aftKey : controlNameDict.keySet()) {
                searchIn.getdList().add(controlNameDict.get(aftKey));
            }
            List<WFormLayout> dataList = wFormLayoutMapper.checkDataColExist(searchIn);
            if (dataList != null && dataList.size() > 0) {
                String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0010);
                msg = editMsg(msg, dataList.get(0).getiFormDataCol());
                return msg;
            }
        }
        //操作
        wFormDesign.setiOprFlg(Definition.FLG_ZERO);
        //时间
        Date nowDate = new Date();
        wFormDesign.setiEntryDate(nowDate);
        wFormDesign.setiUpdDate(nowDate);
        try {
            //保存表单数据
            wFormDesignMapper.insertWFormDesign(wFormDesign);
            //前版本表单设计表数据更新
            WFormDesign upWFormDesign = new WFormDesign();
            upWFormDesign.setiFormId(inData.getiFormId());
            if (inData.getiUseSYmd() != null) {
                upWFormDesign.setiUseEYmd(DateUtils.addDays(inData.getiUseSYmd(), -1));
            } else {
                upWFormDesign.setiUseEYmd(null);
            }
            upWFormDesign.setiUpdId(inData.getiUpdId());
            upWFormDesign.setiUpdDate(nowDate);
            wFormDesignMapper.updatePreWFormDesign(upWFormDesign);
            //保存利用者数据
            if (StringUtils.isNotEmpty(optionValue) && StringUtils.isNotEmpty(optionType)) {
                for (String iUserId : optionValue.split(Definition.ROLE_DELIMETER)) {
                    WFormUser wFormUser = new WFormUser();
                    wFormUser.setiFormId(newFormId);
                    wFormUser.setiUserType(optionType);
                    wFormUser.setiUserId(iUserId);
                    wFormUser.setiEntryId(inData.getiEntryId());
                    wFormUser.setiEntryDate(nowDate);
                    wFormUser.setiUpdId(inData.getiUpdId());
                    wFormUser.setiUpdDate(nowDate);
                    wFormUserMapper.insertWFormUser(wFormUser);
                }
            }
            //保存控件名及创建数据表列
            WFormDataCreate wFormDataCreate = new WFormDataCreate();
            if (controlNameDict != null && controlNameDict.size() > 0) {
                wFormDataCreate.setFormDataTblname(wFormDesignCopyCreate.getiFormDataTblname());
                List<WFControlToTableCol> dataList = new ArrayList<>();
                for (String strKey : controlNameDict.keySet()) {
                    //申请表
                    dataList.add(controlNameDict.get(strKey));
                    //保存控件名
                    WFormLayout wFormLayout = new WFormLayout();
                    wFormLayout.setiFormDataTblname(wFormDesignCopyCreate.getiFormDataTblname());
                    wFormLayout.setiFormDataCol(strKey);
                    wFormLayout.setiFormVersion(nextFormVersion);
                    wFormLayout.setiFormDataControl(controlNameDict.get(strKey).getControl());
                    wFormLayout.setiFormDataControlInitval(controlNameDict.get(strKey).getControlDefaultValue());
                    wFormLayout.setiEntryId(inData.getiEntryId());
                    wFormLayout.setiEntryDate(nowDate);
                    wFormLayout.setiUpdId(inData.getiUpdId());
                    wFormLayout.setiUpdDate(nowDate);
                    wFormLayoutMapper.insertWFormLayout(wFormLayout);
                }
                wFormDataCreate.setdList(dataList);
                //创建数据表列
                wFormDataMapper.addColumn(wFormDataCreate);
            }
            return Definition.RESULT_S;
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0111);
        }
    }

    /*
    更新表单信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String UpdateForm(String language, WFormDesign inData, String optionType, String optionValue) {
        //表单名重复性check
        if (checkFormNameExist(inData.getiFormName(), inData.getiFormId().toString())) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0019);
        }
        //表单ID对应表单数据取得
        WFormDesignEidt preData = wFormDesignMapper.selectWFormDesignById(inData.getiFormId().toString());
        if (preData == null) {
            return bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0007);
        }
        //从表单中解析出控件名
        Map<String, WFControlToTableCol> controlNameDict = null;
        try {
            controlNameDict = convertJsonControlName(inData.getiFormLayout());
        } catch (Exception e) {
            String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020, Definition.MSG_301020_0010);
            msg = editMsg(msg, e.getMessage());
            return msg;
        }
        //既存版本重复性check
        List<String> lsitDataCol = new ArrayList<>();
        if (!Definition.FLG_1.equals(preData.getFormVersion())) {
            //前回版本表单列数据取得
            String preFormLayOut = wFormDesignMapper.getLayoutByFormId(Long.valueOf(preData.getPreformId()));
            if (StringUtils.isNotEmpty(preFormLayOut)) {
                //前回版本组件数据取得
                Map<String, WFControlToTableCol> preControlList = null;
                try {
                    preControlList = convertJsonControlName(preFormLayOut);
                } catch (Exception e) {
                    String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,
                            Definition.MSG_301020_0010);
                    msg = editMsg(msg, e.getMessage());
                    return msg;
                }
                //去除已存在的控件
                for (String preKey : preControlList.keySet()) {
                    controlNameDict.remove(preKey);
                }
            }
            //当前版本表单列数据取得
            WFormLayout inDataColLayout = new WFormLayout();
            inDataColLayout.setiFormDataTblname(preData.getTableName());
            inDataColLayout.setiFormVersion(Long.valueOf(preData.getFormVersion()));
            lsitDataCol = wFormLayoutMapper.getDataColByNameVersion(inDataColLayout);
            //履历版本表单列数据取得
            if (controlNameDict.keySet().size() > 0) {
                WFormLayout inPreData = new WFormLayout();
                inPreData.setiFormDataTblname(preData.getTableName());
                inPreData.setiFormVersion(Long.valueOf(preData.getFormVersion()));
                for (String strKey : controlNameDict.keySet()) {
                    inPreData.setiFormDataCol(strKey);
                    List<WFormLayout> preList = wFormLayoutMapper.checkPreDataCol(inPreData);
                    if (preList != null && preList.size() > 0) {
                        String msg = bLanguageMsService.GetMessage(language, Definition.PAGE_301020,
                                Definition.MSG_301020_0010);
                        msg = editMsg(msg, preList.get(0).getiFormDataCol());
                        return msg;
                    }
                }
            }
        }
        try {
            //时间
            Date nowDate = new Date();
            WFormDesign updateData = null;
            //更新表单名
            if (StringUtils.compare(preData.getTitle(), inData.getiFormName()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiFormName(inData.getiFormName());
                wFormDesignMapper.updateFormNameByKey(updateData);
            }
            //更新表单详细
            if (StringUtils.compare(preData.getDesc(), inData.getiFormDetail()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiFormDetail(inData.getiFormDetail());
                wFormDesignMapper.updateFormDetailByKey(updateData);
            }
            //更新表单结构
            if (StringUtils.compare(preData.getColumnConfig(), inData.getiFormLayout()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiFormLayout(inData.getiFormLayout());
                wFormDesignMapper.updateFormLayoutByKey(updateData);
            }
            //更新利用开始日
            if (dateBijiao(preData.getStartDate(), inData.getiUseSYmd())) {
                updateData = commonset(inData, nowDate);
                updateData.setiUseSYmd(inData.getiUseSYmd());
                wFormDesignMapper.updateUseSYmdByKey(updateData);
                //非初版的场合，需要更新前版本的利用终了日
                if (!Definition.FST_VERSION.toString().equals(preData.getFormVersion())) {
                    //前版本表单设计表数据更新
                    updateData = commonset(inData, nowDate);
                    updateData.setiFormId(Long.valueOf(preData.getPreformId()));
                    if (inData.getiUseSYmd() != null) {
                        updateData.setiUseEYmd(DateUtils.addDays(inData.getiUseSYmd(), -1));
                    } else {
                        updateData.setiUseEYmd(null);
                    }
                    wFormDesignMapper.updateUseEYmdByKey(updateData);
                }
            }
            //更新利用终了日
            if (dateBijiao(preData.getEndDate(), inData.getiUseEYmd())) {
                updateData = commonset(inData, nowDate);
                updateData.setiUseEYmd(inData.getiUseEYmd());
                wFormDesignMapper.updateUseEYmdByKey(updateData);
            }
            //更新采番规则
            if (StringUtils.compare(preData.getPrefix(), inData.getiNumberRules()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiNumberRules(inData.getiNumberRules());
                wFormDesignMapper.updateNumberRulesByKey(updateData);
            }
            //更新FLOW结构
            if (StringUtils.compare(preData.getAuditConfig(), inData.getiFlowData()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiFlowData(inData.getiFlowData());
                wFormDesignMapper.updateFlowDataByKey(updateData);
            }
            //更新审批要否开关状态
            if (StringUtils.compare(preData.getNeedAudit(), inData.getiFlowFlg()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiFlowFlg(inData.getiFlowFlg());
                wFormDesignMapper.updateFlowFlgByKey(updateData);
            }
            //更新多语言开关状态
            if (StringUtils.compare(preData.getMultilingual(), inData.getiLanguageFlg()) != 0) {
                updateData = commonset(inData, nowDate);
                updateData.setiLanguageFlg(inData.getiLanguageFlg());
                wFormDesignMapper.updateLanguageFlgByKey(updateData);
            }
            //保存利用者数据
            if (StringUtils.isNotEmpty(optionValue) && StringUtils.isNotEmpty(optionType)) {
                //删除前回保存的利用者
                wFormUserMapper.deleteWFormUserByIFormId(inData.getiFormId());
                //重新保存利用者
                for (String iUserId : optionValue.split(Definition.ROLE_DELIMETER)) {
                    WFormUser wFormUser = new WFormUser();
                    wFormUser.setiFormId(inData.getiFormId());
                    wFormUser.setiUserType(optionType);
                    wFormUser.setiUserId(iUserId);
                    wFormUser.setiEntryId(inData.getiEntryId());
                    wFormUser.setiEntryDate(nowDate);
                    wFormUser.setiUpdId(inData.getiUpdId());
                    wFormUser.setiUpdDate(nowDate);
                    wFormUserMapper.insertWFormUser(wFormUser);
                }
            }
            //数据表结构存在变更的场合
            if (StringUtils.compare(preData.getColumnConfig(), inData.getiFormLayout()) != 0) {
                //初版的场合
                if (Definition.FST_VERSION.toString().equals(preData.getFormVersion())) {
                    //删除数据结构表中当前版本的数据
                    WFormLayout deleteWFormLayout = new WFormLayout();
                    deleteWFormLayout.setiFormDataTblname(preData.getTableName());
                    deleteWFormLayout.setiFormVersion(Long.valueOf(preData.getFormVersion()));
                    wFormLayoutMapper.deleteWFormLayoutByIds(deleteWFormLayout);
                    //保存控件名及创建数据表列
                    WFormDataCreate wFormDataCreate = new WFormDataCreate();
                    if (controlNameDict != null && controlNameDict.size() > 0) {
                        wFormDataCreate.setFormDataTblname(preData.getTableName());
                        List<WFControlToTableCol> dataList = new ArrayList<>();
                        for (String strKey : controlNameDict.keySet()) {
                            //申请表
                            dataList.add(controlNameDict.get(strKey));
                            //保存控件名
                            WFormLayout wFormLayout = new WFormLayout();
                            wFormLayout.setiFormDataTblname(preData.getTableName());
                            wFormLayout.setiFormDataCol(strKey);
                            wFormLayout.setiFormVersion(Long.valueOf(preData.getFormVersion()));
                            wFormLayout.setiFormDataControl(controlNameDict.get(strKey).getControl());
                            wFormLayout.setiFormDataControlInitval(controlNameDict.get(strKey).getControlDefaultValue());
                            wFormLayout.setiEntryId(inData.getiEntryId());
                            wFormLayout.setiEntryDate(nowDate);
                            wFormLayout.setiUpdId(inData.getiUpdId());
                            wFormLayout.setiUpdDate(nowDate);
                            wFormLayoutMapper.insertWFormLayout(wFormLayout);
                        }
                        wFormDataCreate.setdList(dataList);
                        //创建数据表列
                        wFormDataMapper.dropWFormData(preData.getTableName());
                        wFormDataMapper.creatWFormDataTable(wFormDataCreate);
                    }
                } else {
                    //删除表单数据表中当前版本的列
                    if (lsitDataCol != null && lsitDataCol.size() > 0) {
                        WFormLayout dropWFormLayout = new WFormLayout();
                        dropWFormLayout.setiFormDataTblname(preData.getTableName());
                        for (String item : lsitDataCol) {
                            dropWFormLayout.setiFormDataCol(item);
                            wFormDataMapper.dropColumn(dropWFormLayout);
                        }
                    }
                    //删除数据结构表中当前版本的数据
                    WFormLayout deleteWFormLayout = new WFormLayout();
                    deleteWFormLayout.setiFormDataTblname(preData.getTableName());
                    deleteWFormLayout.setiFormVersion(Long.valueOf(preData.getFormVersion()));
                    wFormLayoutMapper.deleteWFormLayoutByIds(deleteWFormLayout);
                    //保存控件名及创建数据表列
                    WFormDataCreate wFormDataCreate = new WFormDataCreate();
                    if (controlNameDict != null && controlNameDict.size() > 0) {
                        wFormDataCreate.setFormDataTblname(preData.getTableName());
                        List<WFControlToTableCol> dataList = new ArrayList<>();
                        for (String strKey : controlNameDict.keySet()) {
                            //申请表
                            dataList.add(controlNameDict.get(strKey));
                            //保存控件名
                            WFormLayout wFormLayout = new WFormLayout();
                            wFormLayout.setiFormDataTblname(preData.getTableName());
                            wFormLayout.setiFormDataCol(strKey);
                            wFormLayout.setiFormVersion(Long.valueOf(preData.getFormVersion()));
                            wFormLayout.setiFormDataControl(controlNameDict.get(strKey).getControl());
                            wFormLayout.setiFormDataControlInitval(controlNameDict.get(strKey).getControlDefaultValue());
                            wFormLayout.setiEntryId(inData.getiEntryId());
                            wFormLayout.setiEntryDate(nowDate);
                            wFormLayout.setiUpdId(inData.getiUpdId());
                            wFormLayout.setiUpdDate(nowDate);
                            wFormLayoutMapper.insertWFormLayout(wFormLayout);
                        }
                        wFormDataCreate.setdList(dataList);
                        //创建数据表列
                        wFormDataMapper.addColumn(wFormDataCreate);
                    }
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0111);
        }

        return Definition.RESULT_S;
    }

    /*
    取得全部表单（或指定表单）
     */
    @Override
    public List<BWordbookMsOut> selectAllForm(String schField) {
       return wFormDesignMapper.selectAllForm(schField);
    }

    /*
    比较(true：不等于)
     */
    private Boolean dateBijiao(String str , Date inDate) {
        Boolean re = false;
        if (inDate == null) {
            if (StringUtils.isNotEmpty(str)) {
                re = true;
            }
        } else {
            if (StringUtils.isNotEmpty(str)) {
                Date sDate = ClsBase.strToDate(str);
                if (sDate == null) {
                    re = true;
                } else {
                    if (!DateUtils.isSameDay(sDate, inDate)) {
                        re = true;
                    }
                }
            } else {
                re = true;
            }
        }

        return re;
    }

    /*
    共同设定
    */
    private WFormDesign commonset(WFormDesign inData, Date nowDate) {
        WFormDesign repData = new WFormDesign();
        repData.setiFormId(inData.getiFormId());
        repData.setiUpdId(inData.getiUpdId());
        repData.setiUpdDate(nowDate);
        return repData;
    }

    /*
    申请数据表名存在
     */
    private Boolean checkTblNameExist(String formName, String formID) {
        SeachBase bEntity = new SeachBase();
        Map<String, Object> mpData = new HashMap<>();
        mpData.put(Definition.TBL_NAME, formName);
        mpData.put(Definition.OWN_FORM_ID, formID);
        bEntity.setParams(mpData);
        int existflg = wFormDesignMapper.checkTblNameExist(bEntity);
        if (existflg == 0) {
            return false;
        } else {
            return true;
        }
    }

    /*
    表单名重复性check
     */
    private Boolean checkFormNameExist(String formName, String formID) {
        SeachBase bEntity= new SeachBase();
        Map<String, Object> mpData = new HashMap<>();
        mpData.put(Definition.I_FORM_NAME, formName);
        mpData.put(Definition.OWN_FORM_ID, formID);
        bEntity.setParams(mpData);
        int existflg = wFormDesignMapper.checkFormNameExist(bEntity);
        if (existflg == 0) {
            return false;
        } else {
            return true;
        }
    }

    /*
    从表单JSON中解析出控件名
     */
    private Map<String, WFControlToTableCol> convertJsonControlName(String str) throws Exception {
        List<Map<String, String>> mapList = ClsBase.jsonoMapList(str);
        Map<String, WFControlToTableCol> controlNameDict = new HashMap<>();
        if (mapList == null || mapList.size() == 0) {
            return controlNameDict;
        }
        for (Map<String, String> map : mapList) {
            if (StringUtils.isNotEmpty(map.get(Definition.JSON_CONTROL))
                    && StringUtils.isNotEmpty(map.get(Definition.JSON_PROP))) {
                String strKey = map.get(Definition.JSON_PROP);
                WFControlToTableCol wFControlToTableCol = controlToTableColOther(map);
                if (controlNameDict.containsKey(strKey)) {
                    throw new Exception(strKey);
                } else {
                    controlNameDict.put(strKey, wFControlToTableCol);
                }
            }
        }

        return controlNameDict;
    }

    /*
    控件名生成DB列属性之其他控件
     */
    private WFControlToTableCol controlToTableColOther(Map<String, String> map) {
        WFControlToTableCol wFControlToTableCol = new WFControlToTableCol();
        //控件
        wFControlToTableCol.setControl(map.get(Definition.JSON_CONTROL));
        //控件初期值
        if (StringUtils.isNotEmpty(map.get(Definition.JSON_DEFAULT_PROP_VALUE))) {
            wFControlToTableCol.setControlDefaultValue(map.get(Definition.JSON_DEFAULT_PROP_VALUE));
        }
        //列名
        wFControlToTableCol.setColName(map.get(Definition.JSON_PROP));
        //注释
        wFControlToTableCol.setColComment(map.get(Definition.JSON_LABELCH));
        //类型
        wFControlToTableCol.setColType(Definition.COL_TYPE_TEXT);

        return wFControlToTableCol;
    }

    /*
     编辑
     */
    private String editMsg(String msg1, String msg2) {
        return (msg1 + Definition.STR_COLON + Definition.LEFT_BRACKET + msg2 + Definition.RIGHT_BRACKET);
    }
}
