package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WGroup;
import com.ruoyi.project.workflow.domain.WGroupUser;
import com.ruoyi.project.workflow.domain.db.in.WGroupYilanIn;
import com.ruoyi.project.workflow.domain.db.out.BLanguageMsOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupDetailOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupMemberOut;
import com.ruoyi.project.workflow.domain.db.out.WGroupYilanOut;
import com.ruoyi.project.workflow.mapper.WGroupMapper;
import com.ruoyi.project.workflow.mapper.WGroupUserMapper;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WGroupService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.List;

@Service("wGroupService")
public class WGroupServiceImpl implements WGroupService {

    @Autowired
    private WGroupMapper wGroupMapper;
    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WGroupUserMapper wGroupUserMapper;

    /*
    责任者
    */
    @Override
    public String selectGroupMasterByKey(String iGroupId) {
        return wGroupMapper.selectGroupMasterByKey(Long.valueOf(iGroupId));
    }

    /*
    分组用户
     */
    @Override
    public List<String> selectGroupUsersByKey(String iGroupId) {
        return wGroupMapper.selectGroupUsersByKey(Long.valueOf(iGroupId));
    }

    /**
     * 分组一览
     */
    @Override
    public List<WGroupYilanOut> selectWGroupList(WGroupYilanIn wGroupYilanIn) {
        return wGroupMapper.selectWGroupList(wGroupYilanIn);
    }

    /**
     * 分组一览件数
     */
    @Override
    public Long selectWGroupCount(WGroupYilanIn wGroupYilanIn) {
        return wGroupMapper.selectWGroupCount(wGroupYilanIn);
    }

    /**
     * 分组一览（无limit）
     */
    @Override
    public List<WGroupYilanOut> selectWGroupUnLimt(WGroupYilanIn wGroupYilanIn) {
        return wGroupMapper.selectWGroupUnLimt(wGroupYilanIn);
    }

    /**
     * 详细
     */
    @Override
    public WGroupDetailOut selectWGroupDetailById(Long iGroupId) {
        return wGroupMapper.selectWGroupDetailById(iGroupId);
    }

    /**
     * 成员
     */
    @Override
    public List<WGroupMemberOut> selectWGroupMemberById(Long iGroupId) {
        return wGroupMapper.selectWGroupMemberById(iGroupId);
    }

    /**
     * 角色ID采番
     */
    @Override
    public Long selectNewGroupId() {
        return wGroupMapper.selectNewGroupId();
    }

    /**
     *判断是否已经存在
     */
    @Override
    public List<String> selectCheckExists(String roleName, Long roleId) {
        WGroup wGroup = new WGroup();
        wGroup.setiGroupName(roleName);
        wGroup.setiGroupId(roleId);
        return wGroupMapper.selectCheckExists(wGroup);
    }

    /**
     * 保存数据
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveData(String language, String operate, WGroup inData, String[] members) {
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        //入力
        WGroup wGroup = new WGroup();
        BeanUtils.copyProperties(inData, wGroup);
        wGroup.setiUpdId(userId);
        wGroup.setiUpdDate(nowDate);
        try {
            //新增
            if (Definition.STR_ADD.equals(operate)) {
                wGroup.setiEntryId(userId);
                wGroup.setiEntryDate(nowDate);
                wGroupMapper.insertWGroup(wGroup);
            } else if (Definition.STR_UPDATE.equals(operate)) {
                wGroupMapper.updateWGroup(wGroup);
            }
            //角色成员删除
            wGroupUserMapper.deleteByGroupId(inData.getiGroupId());
            //角色成员添加
            if (members !=null && members.length > 0) {
                for (String member : members) {
                    WGroupUser wGroupUser = new WGroupUser();
                    wGroupUser.setiGroupId(inData.getiGroupId());
                    wGroupUser.setiGroupUser(member);
                    wGroupUser.setiEntryId(userId);
                    wGroupUser.setiEntryDate(nowDate);
                    wGroupUser.setiUpdId(userId);
                    wGroupUser.setiUpdDate(nowDate);
                    wGroupUserMapper.insertWGroupUser(wGroupUser);
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0091);
        }

        return Definition.RESULT_S;
    }

    /**
     * 删除数据
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteData(String language, String groupIds) {
        try {
            //角色主表
            wGroupMapper.deleteByGroupIds(groupIds);
            //角色成员表
            wGroupUserMapper.deleteByGroupIds(groupIds);
        }  catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0081);
        }

        return Definition.RESULT_S;
    }

    /**
     * 取得全部角色（或指定角色）
     */
    @Override
    public List<BLanguageMsOut> selectAllRole(String iGroupIds){
        return wGroupMapper.selectAllRole(iGroupIds);
    }

    @Override
    public String selectRroupNameByKey(String iGroupId) {
        return wGroupMapper.selectRroupNameByKey(Long.valueOf(iGroupId));
    }
}
