package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WProxySettings;
import com.ruoyi.project.workflow.domain.base.DateFromTo;
import com.ruoyi.project.workflow.domain.db.out.ProxyDetailOut;
import com.ruoyi.project.workflow.domain.input.ProxyDetai;
import com.ruoyi.project.workflow.mapper.WProxySettingsMapper;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WProxySettingsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.List;

@Service("wProxySettingsService")
public class WProxySettingsServiceImpl implements WProxySettingsService {

    @Autowired
    private WProxySettingsMapper wProxySettingsMapper;
    @Autowired
    private IBLanguageMsService bLanguageMsService;

    /*
    取得代理人详细
     */
    @Override
    public List<ProxyDetailOut> selectProxyDetail(String userId) {
        return wProxySettingsMapper.selectProxyDetail(userId);
    }

    /*
    更新代理人信息
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String updateProxyDetail(String language, String userid, List<ProxyDetai> data) {
        //时间
        Date nowDate = new Date();
        try {
            //删除当前用户的代理人信息
            wProxySettingsMapper.deleteByUserId(userid);
            //插入当前用户的代理人信息
            if (data!= null && data.size() > 0 ) {
                for (ProxyDetai detail : data) {
                    WProxySettings insertData = new WProxySettings();
                    //用户
                    insertData.setiUserId(userid);
                    //代理用户ID
                    insertData.setiProxyUserId(detail.getI_proxy_user_id());
                    //Date from to
                    DateFromTo dateFromTo = ClsBase.toDate(detail.getI_proxy_date());
                    //代理开始日期
                    if (StringUtils.isNotEmpty(dateFromTo.getDateFrom())
                            && ClsBase.isDateType10(dateFromTo.getDateFrom())) {
                        insertData.setiProxySYmd(ClsBase.strToDate(dateFromTo.getDateFrom()));
                    }
                    //代理结束日期
                    if (StringUtils.isNotEmpty(dateFromTo.getDateTo())
                            && ClsBase.isDateType10(dateFromTo.getDateTo())) {
                        insertData.setiProxyEYmd(ClsBase.strToDate(dateFromTo.getDateTo()));
                    }
                    //代理Form
                    insertData.setiProxyFormId(detail.getI_proxy_form_id());
                    //创建者
                    insertData.setiEntryId(userid);
                    //创建时间
                    insertData.setiEntryDate(nowDate);
                    //更新者
                    insertData.setiUpdId(userid);
                    //更新时间
                    insertData.setiUpdDate(nowDate);
                    //插入
                    wProxySettingsMapper.insertWProxySettings(insertData);
                }
            }
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0091);
        }

        return Definition.RESULT_S;
    }
}
