package com.ruoyi.project.workflow.service.impl;

import com.ruoyi.project.workflow.common.ClsBase;
import com.ruoyi.project.workflow.common.Definition;
import com.ruoyi.project.workflow.domain.WUserSettings;
import com.ruoyi.project.workflow.mapper.WUserSettingsMapper;
import com.ruoyi.project.workflow.service.IBLanguageMsService;
import com.ruoyi.project.workflow.service.WUserSettingsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;

@Service("wUserSettingsService")
public class WUserSettingsServiceImpl implements WUserSettingsService {

    @Autowired
    private IBLanguageMsService bLanguageMsService;
    @Autowired
    private WUserSettingsMapper wUserSettingsMapper;

    /**
     * 新增個人設定
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String insertWUserSettings(String language, String signImage){
        //时间
        Date nowDate = new Date();
        //用户
        String userId = ClsBase.getCurrentUser();
        //入力
        WUserSettings wUserSettings = new WUserSettings();
        wUserSettings.setiUserId(userId);
        wUserSettings.setiSignImage(signImage);
        wUserSettings.setiEntryId(userId);
        wUserSettings.setiEntryDate(nowDate);
        wUserSettings.setiUpdId(userId);
        wUserSettings.setiUpdDate(nowDate);
        try {
            //删除
            wUserSettingsMapper.deleteWUserSettingsByUserId(userId);
            //保存
            wUserSettingsMapper.insertWUserSettings(wUserSettings);
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0122);
        }
        return Definition.RESULT_S;
    }

    /**
     * 删除個人設定
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String deleteWUserSettingsByKeys(String language, String signImage) {
        //用户
        String userId = ClsBase.getCurrentUser();
        try {
            WUserSettings wUserSettings = new WUserSettings();
            wUserSettings.setiUserId(userId);
            wUserSettings.setiSignImage(signImage);
            wUserSettingsMapper.deleteWUserSettingsByKeys(wUserSettings);
        } catch (Exception e) {
            //回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return bLanguageMsService.GetMessage(language, Definition.COMMON_MSG, Definition.MSG_COMMON_0081);
        }
        return Definition.RESULT_S;
    }

    /**
     * 取得個人設定
     */
    public WUserSettings selectWUserSettingsByUserId(String iUserId) {
        return wUserSettingsMapper.selectWUserSettingsByUserId(iUserId);
    }
}
